
#ifndef DINO_AGENT_H
#define DINO_AGENT_H

#define AGENT_SMOOTH_CNT	5

#define RTOD 57.295780f
#define DTOR 0.017453f


//const float gravity = (-9.8f*3.2808399f)/(1000.0f*1000.0f);	// meters per seconds squared to feet per millisecond second squared
#define GRAVITY_VAL -0.00003215223f

// iterator flags
#define AGENT_ITOR_SKIP	0x00000001

// steering behaviors
typedef struct SteeringData
{
	// behavior flags
	uns32 flags;

	// behavior params
	crd seek_target[3];
	crd arrive_pt[3];
	float arrive_deceleration;

	crd pursuit_pt[3];
	crd pursuit_heading[3];
	crd pursuit_vel[3];

	float wanderRadius;
	float wanderDist;
	float wanderJitter;
	crd wanderJitterMin[3];
	crd wanderJitterMax[3];
	crd wanderTarget[3];

	float separation_scale;
	float alignment_scale;
	float cohesion_scale;
	float seek_scale;
	float arrive_scale;
	float wander_scale;
	float pursuit_scale;
	float path_scale;
	float grav_scale;

	ResDataKan* path_key_data;
	ResDataKanKey* path_keys;
	uns32 path_waypoint;
	float path_dist_to_next;
	crd path_offset[3];
	float path_start_slow_speed;
	float path_finish_slow_speed;

	float gravity;

	float forceScale;

	struct SteeringAgent* offset_lead;
	crd offset_lead_mat[VM_SIZE];
	crd offset_lead_tvel[3];
	crd offset_lead_max_speed;
	crd offset[3];

	float nradius;	// neighbors
} SteeringData;

struct SteeringAgent;	// crazy c forward declaration
typedef struct SteeringAgent* (*Agent_Iterator)( int indx, void* data, uns32* flags );	// iterator function type

// stuff to have a moving thing with orientation
typedef struct SteeringAgent
{
	ObjVel* ov;
	crd heading[3];
	crd heading_smooth[3];
	float heading_smooth_blend;
	crd heading_smoother[3*AGENT_SMOOTH_CNT];
	crd force_smooth[3];
	float force_smooth_blend;
	int smooth_cnt;
	int smooth_full;
	crd side[3];
	crd up[3];
	float bradius;
	crd clr[3];

	float maxSteerForce;
	float maxSpeed;

	SteeringData steering;
	Agent_Iterator agent_iterator;
	void* agent_iterator_data;

	int tag;
} SteeringAgent;


void Agent_Init( SteeringAgent* pAgent, ObjVel* ov );
void Agent_SetMaxSpeed( SteeringAgent* pAgent, float max_speed );
void Agent_SetMaxSteerForce( SteeringAgent* pAgent, float max_steer_force );
void Agent_SetIterator( SteeringAgent* pAgent, Agent_Iterator agent_iterator, void* data, float neighbor_radius );
void Agent_SetNeighborsRadius( SteeringAgent* pAgent, float radius );
void Agent_SetBoundingRadius( SteeringAgent* pAgent, float radius );
void Agent_DisableAll( SteeringAgent* pAgent );
void Agent_PursuitOn( SteeringAgent* pAgent );
void Agent_PursuitOff( SteeringAgent* pAgent );
void Agent_PursuitUpdate( SteeringAgent* pAgent, crd* pt, crd* vel );
void Agent_OffsetPursuitOn( SteeringAgent* pAgent, SteeringAgent* pLead, crd* offset );
void Agent_OffsetPursuitOff( SteeringAgent* pAgent );
void Agent_OffsetPursuitSplineOn( SteeringAgent* pAgent, crd* offset );
void Agent_OffsetPursuitSplineOff( SteeringAgent* pAgent );
void Agent_OffsetPursuitSplineUpdate( SteeringAgent* pAgent, crd* mat, crd* tvel, float maxspeed );
void Agent_WanderOn( SteeringAgent* pAgent, float radius, float dist, float jitter );
void Agent_WanderOff( SteeringAgent* pAgent );
void Agent_ArriveOn( SteeringAgent* pAgent, crd* pt, float deceleration );
void Agent_ArriveOff( SteeringAgent* pAgent );
void Agent_ArriveUpdate( SteeringAgent* pAgent, crd* pt, float deceleration );
void Agent_SeekOn( SteeringAgent* pAgent, crd* pt );
void Agent_SeekOff( SteeringAgent* pAgent );
void Agent_SeekUpdate( SteeringAgent* pAgent, crd* new_pos );
void Agent_SeparationOn( SteeringAgent* pAgent );
void Agent_SeparationOff( SteeringAgent* pAgent );
void Agent_CohesionOn( SteeringAgent* pAgent );
void Agent_CohesionOff( SteeringAgent* pAgent );
void Agent_AlignmentOn( SteeringAgent* pAgent );
void Agent_AlignmentOff( SteeringAgent* pAgent );
void Agent_SetHeadingFromRot( SteeringAgent* pAgent, crd* rot );
void Agent_Update( SteeringAgent* pAgent );
void Agent_RenderDebug( SteeringAgent* pAgent );
void Agent_PathFollowOn( SteeringAgent* pAgent, ResDataKan* key_data, ResDataKanKey* keys, uns32 cur_waypoint, float dist_to_next );
void Agent_PathFollow_NoSpeed( SteeringAgent* pAgent );
void Agent_PathFollow_Offset( SteeringAgent* pAgent, crd* offset );
void Agent_PathFollow_Stop( SteeringAgent* pAgent );
void Agent_PathFollow_Slow( SteeringAgent* pAgent, float new_max_speed );
void Agent_PathFollowOff( SteeringAgent* pAgent );
int Agent_IsPathFollowOn( SteeringAgent* pAgent );
void Agent_SetCalc_Priority( SteeringAgent* pAgent );
void Agent_SetCalc_Blend( SteeringAgent* pAgent );
void Agent_SetJitterBounds( SteeringAgent* pAgent, crd* jMin, crd* jMax );

void Agent_GravityOn( SteeringAgent* pAgent, float gravity );
void Agent_GravityOff( SteeringAgent* pAgent );

void Agent_SetForceScale( SteeringAgent* pAgent, float scale );
// 0 is no smoothing (use 100% of the force from the current frame,
// 1 uses 100% of the last frame
void Agent_SetForceSmoothing( SteeringAgent* pAgent, float blend );
void Agent_SetHeadingSmoothing( SteeringAgent* pAgent, float blend );

void Agent_SetSeparationScale( SteeringAgent* pAgent, float scale );
void Agent_SetAlignmentScale( SteeringAgent* pAgent, float scale );
void Agent_SetCohesionScale( SteeringAgent* pAgent, float scale );
void Agent_SetSeekScale( SteeringAgent* pAgent, float scale );
void Agent_SetArriveScale( SteeringAgent* pAgent, float scale );
void Agent_SetWanderScale( SteeringAgent* pAgent, float scale );
void Agent_SetPursuitScale( SteeringAgent* pAgent, float scale );
void Agent_SetPathScale( SteeringAgent* pAgent, float scale );
void Agent_SetGravityScale( SteeringAgent* pAgent, float scale );

float GetLaunchVec( crd* target_pos, crd* launch_pos, float launch_ang, float gravity, crd* out_vel );
void GetLaunchVecT( crd* target_pos, crd* launch_pos, float time, float gravity, crd* out_vel );
float tvelToMPH( crd* tvel );
float mphToTvel( float mph );
void plane_point_normal( crd* pt, crd* n, crd* out_plane );
float plane_point_dist( crd* plane, crd* point );

#endif
