// game_2D.h
// Play Mechanix - Video Game System
// Copyright (c) 2010 Play Mechanix Inc. All Rights Reserved
//

#ifndef GAME_2D_H
#define GAME_2D_H

#include "game.h"


#define UI_EVENT_SELECT			(1 << 0)
#define UI_EVENT_HOVER_SCALE	(1 << 1)
#define UI_EVENT_HOVER_IMAGE	(1 << 2)
#define UI_EVENT_HOVER_COLOR	(1 << 3)
#define UI_EVENT_HOVER_TRIGGER	(1 << 4)


PSM_StateMachine gGame2DStateMachine;

enum Game2DStateMachineStates
{
	SM_GAME2D_SETAPPEAR = SM_GAMESTATES_MAX,
	SM_GAME2D_APPEAR,
	SM_GAME2D_DISAPPEAR,
	SM_GAME2D_SHOTREP_3DDISAPPEAR,
	SM_GAME2D_SETPLAYERCOLOR,
	SM_GAME2D_CALLDISAPPEAR,
	SM_GAME2D_CAMSEQ,

	SM_GAME2D_ENABLE_EVENTS,
	SM_GAME2D_DISABLE_EVENTS,
	SM_GAME2D_REGISTER_SELECT,
	SM_GAME2D_REGISTER_HOVER_SCALE,
	SM_GAME2D_REGISTER_HOVER_IMAGE,
	SM_GAME2D_REGISTER_HOVER_COLOR,
	SM_GAME2D_REGISTER_HOVER_TRIGGER,
	SM_GAME2D_ENABLE_ALL_EVENTS,
	SM_GAME2D_DISABLE_ALL_EVENTS,
	SM_GAME2D_REMOVE_ALL_EVENTS,
	SM_GAME2D_REMOVE_SELECT,
	SM_GAME2D_REMOVE_HOVER_SCALE,
	SM_GAME2D_REMOVE_HOVER_IMAGE,
	SM_GAME2D_REMOVE_HOVER_COLOR,
	SM_GAME2D_REMOVE_HOVER_TRIGGER,
	SM_GAME2D_TIMER_START,
	SM_GAME2D_TIMER_WAIT,
	SM_GAME2D_TIMER_WATCH,

	SM_GAME2D_MAX,
	NUM_SM_GAME2DSTATES = (SM_GAME2D_MAX - SM_GAMESTATES_MAX),
	
};

PSM_State gGame2DMachineStates[NUM_SM_GAME2DSTATES];

void Game2D_SetAppear( uns32 frames, float xdist, float ydist );
void Game2DCallDisappear();


void Game2DAddTap( int gnum );
void Game2DRemoveTap( int gnum );


void Game2DSetActiveGun( int gnum );

void Game2D_Enable_Events();
void Game2D_Disable_Events();
int Game2D_Check_Events_Enabled();

int Game2D_UI_RegisterSelect(Obj* obj);
int Game2D_UI_RegisterHoverScale(Obj* obj, float minScale, float maxScale, int shrink_ms, int grow_ms);
int Game2D_UI_RegisterHoverImage(Obj* obj);
int Game2D_UI_RegisterHoverColor(Obj* obj, float* rgba, uns8* active_flags, int operation, int newColor_ms, int oldColor_ms);
int Game2D_UI_RegisterHoverTrigger(Obj* obj, char* hover_enter_trigger, char* hover_exit_trigger);
int Game2D_UI_RemoveEvent(Obj* obj, int ui_event);
int Game2D_UI_RemoveAllEvents(Obj* obj);
int Game2D_UI_DisableAllEvents(Obj* obj);
int Game2D_UI_EnableAllEvents(Obj* obj);

void Game2D_TimerUpdate();
int Game2D_TimerStart(int timeout, int turnOffTapout ); // starts global timer counting down
int Game2D_TimerCheck();  // returns number of ms left before timeout (0 if timed out)
void Game2D_Timer_TextCallback( wchar_t* input, wchar_t* output, int outSize, TextParams* tp ); // will splot in num secs left in timer


void* MemAllocGame2D_UI_Data(size_t size);
int Game2D_UI_Init();
void Game2D_UI_Reset();
int Game2D_UI_Loop();


// sm funcs


int SM_G2D_Set2DAppear( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_G2D_Appear( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_G2D_CallDisappear( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_G2D_Disappear( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_G2D_ShotRep_Disappear( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_G2D_SetPlayerColor( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_G2D_CamSeq( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );


int SM_G2D_EnableEvents( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_DisableEvents( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RegisterSelect( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RegisterHoverScale( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RegisterHoverImage( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RegisterHoverColor( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RegisterHoverTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RemoveAllEvents( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_EnableObjEvents( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_DisableObjEvents( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RemoveSelect( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RemoveHoverScale( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RemoveHoverImage( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RemoveHoverColor( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_G2D_RemoveHoverTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );


int SM_G2D_TimerStart( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_G2D_TimerWait( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_G2D_TimerWatch( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );


#endif // GAME_2D_H

