// game_score.h
// Play Mechanix - Video Game System
// Copyright (c) 2010 Play Mechanix Inc. All Rights Reserved
//


#ifndef GAME_SCORE_H
#define GAME_SCORE_H


#include "game.h"

// dino enums
enum
{
	DINO_RAPTOR,
	DINO_MRAPTOR,
	DINO_DILO,
	DINO_COMPY,
	DINO_PTRDON,
	DINO_WASP,
	DINO_UBUG,
	DINO_TRICERA,
	DINO_TREX,
	DINO_BRACHIO,	
	DINO_SPINO,
	DINO_NUM_DINO
};

// bonus items enums
enum
{
	BONUS_DAMAGE,
	BONUS_STREAK,
	BONUS_SURVIVOR,
	BONUS_POWERUP,
	BONUS_NUM_BONUS
};

// entries into level count table
enum
{
	LEVEL_COUNT_DAMAGE=0,
	LEVEL_COUNT_POWERUP,
	LEVEL_COUNT_SURVIVOR,
	LEVEL_COUNT_NUM_COUNT
};

// count this player's bullet as hitting an enemy
void GameScore_CountHit(uns32 pnum, int dino_type, ObjDispRes* odr);

// count one bullet miss
void GameScore_CountMiss(uns32 pnum);

// count one enemy kill
void GameScore_CountKill(uns32 pnum, int type, ObjDispRes* odr);

// shooting bonus items
int GameScore_BonusReadType(char* bonusName);
void GameScore_BonusShot(int player, int bonusType, ObjDispRes* odr);

// add in a bonus
void GameScore_BonusAdd(int player, int bonusType);

// shooting destructables
void GameScore_DestructShot( ObjDispRes* odr, ObjColAct* cola );

// accuracy between 0.0 and 1.0
float GameScore_GetAccuracy(int pnum);

int GameScore_GetNumShotsFired(int pnum);

// current total score
int GameScore_GetTotalPoints( int pnum );
int GameScore_GetInGamePoints( int pnum );

// counts for level items results scores
int GameScore_GetLevelCount( int count_type );
void GameScore_SetLevelCount( int count_type, int count );
void GameScore_ResetLevelCounts();

// circle score
void GameScore_CircleStartBossSequence( char* boss );
void GameScore_CircleEndBossSequence();
void GameScore_CircleKilled( int pnum );

// point value accessors
int GameScore_GetPointValueDino( int dino );
int GameScore_GetPointValueBonus( int bonus );
int GameScore_GetPointValuePickupBonus();
int GameScore_GetPointValueAccuracy();

// icon names
char* GameScore_GetIconNameDino( int dino );
char* GameScore_GetIconNameBonus( int bonus );

void GameScore_SaveCheckpoint();
void GameScore_RestoreCheckpoint();


#endif
