// game_statemachines.h
// Play Mechanix - Video Game System
// Copyright (c) 2008 Play Mechanix Inc. All Rights Reserved
//
// init routintes and statemachines for programmable statemachine engine 
#ifndef GAMESTATEMACHINES_H
#define GAMESTATEMACHINES_H

int GameStateMachinesInit();
int GameInitSingleStateMachine( PSM_State* states, int numStates, PSM_StateMachine *statemachine, char *name, char *params_type, RTDataTypeParser *prtdtp, void* (*malloc_func)(size_t), void  (*free_func) (void *)  );

void ShakeFootstepSuspend( uns32 durationMS );

// auto fast forward interface
void AutoFF_KillObjFunc();
int AutoFF_IsActive();


enum GameCommonStateMachineStates
{
	// these are game related states that are used in enemy and other actor related statemachines
	// this as opposed to the above which are all dispatcher-centric
	SM_GAMESTATES_GCONF_SLEEP = MAX_CORESTATES, // sleep based on a value from game conf
	SM_GAMESTATES_GCONF_AWATCH, // anim watch based on a value from game conf
	SM_GAMESTATES_GCONF_SWATCH, //  spline watch based on a value from game conf

	SM_GAMESITE_INIT,
	SM_GAMESITE_END,
	SM_GAMESITE_GUNS,
	SM_GAMESITE_FLASHLIGHT,
	SM_GAMESITE_LIGHTNING_ON,
	SM_GAMESITE_LIGHTNING_OFF,
	SM_GAMESITE_GETGUNCLR,
	SM_RECAP_ANN_SPEECH,

	SM_GAMEFLOW_BR_ON_MULTIPLAYER,


	SM_GAMESEL_TOOLSETUP,
	SM_GAMESEL_CAM_FWD,
	SM_GAMESEL_CAM_BACK,
	SM_GAMESEL_LOAD_ADVSEL,

	SM_GAMESHOTREP_ADD_ACCBONUS,
	SM_GAMESHOTREP_ADD_CRITBONUS,
	SM_GAMESHOTREP_ADDTORACK,
	SM_GAMESHOTREP_CALC_SITEACCY,
	SM_GAMESHOTREP_CHK_DEERTROPHY,
	SM_GAMESHOTREP_CHK_DEERTROPHY_2P,
	SM_GAMESHOTREP_CREATE_DEERTROPHY,
	SM_GAMESHOTREP_CREATE_CRITTER,
	SM_GAMESHOTREP_BR_NUMDEERKILLED,
	SM_GAMESHOTREP_BR_NUMCRITSKILLED,
	SM_GAMESHOTREP_ADDSCORE,
	SM_GAMESHOTREP_SETLAYOUTCOLOR,
	SM_GAMESHOTREP_SETLAYOUTCOLOR_2P,
	SM_GAMESHOTREP_DECR_RACKSCORE,
	SM_GAMESHOTREP_ADDDOTS,
	SM_GAMESHOTREP_SETTROPHYTRIGGER,
	SM_GAMESHOTREP_3DDISAPPEAR,
	SM_GAMESHOTREP_BR_ONCRITTERMISS,
	SM_GAMESHOTREP_BR_ONPERFSTREAK,
	SM_GAMESHOTREP_BR_ONHHERO,
	SM_GAMESHOTREP_BR_ONCRITSCORE,
	SM_GAMESHOTREP_BR_ONDEERNUMPNUM,
	SM_GAMESHOTREP_BR_ONTROPHYPNUM,
	SM_GAMESHOTREP_SPELLCOUNT,
	SM_GAMESHOTREP_ROLLUPSCORE,
	SM_GAMESHOTREP_LETTERBITS,
	SM_GAMESHOTREP_CRITTERIDLEANIM,
	SM_GAMESHOTREP_CRITTERDIEANIM,
	SM_GAMESHOTREP_CRITTERSCALE,
	SM_GAMESHOTREP_CRITTERYOFF,
	SM_GAMESHOTREP_ANN_SPEECH,
	SM_GAMESHOTREP_RACKUP_SETTRIGGER,
	SM_GAMESHOTREP_SET_2P_COLOR,
	SM_GAMESHOTREP_SLIDE_SCORE,

	SM_LBOARD_SETLAYOUTCOLOR,
	SM_LBOARD_SETCOLOR,

	SM_TIP_SETLAYOUTCOLOR,
	SM_TIP_SETLAYOUTCOLOR_2P,
	SM_GETREADY_SETLAYOUTCOLOR,
	SM_BONUSTIP_SETLAYOUTCOLOR,
	SM_GAMEEND_SETLAYOUTCOLOR,
	SM_CHARGE_SETLAYOUTCOLOR,

	SM_GAMESITESELECT_INIT, 
	SM_GAMESITESELECT_SETPLAYERCOLOR,
	SM_GAMESITESELECT_SETLAYOUTCOLOR,
	SM_GAMESITESELECT_SETTREKTEXT,
	SM_GAMESITESELECT_BKGRNDBLEND,
	SM_GAMESITESELECT_SETLAYOUTTREKTEXT,
	SM_GAMESITESELECT_END,


	SM_INITIALS_KEYBOARD_START,
	SM_INITIALS_KEYBOARD_CHKDONE,

	SM_LOGIN_TOOLSETUP,
	SM_LOGIN_SETLAYOUTCOLOR,

	SM_CHARGE_TIMER_UPDATE,
	SM_CHARGE_TIMER_TAP,

	SM_ADVSEL_PLAYALLSCENES,
	SM_ADVSEL_RECORDGROUPOID,
	SM_ADVSEL_GETSTARTSPLINE,
	SM_ADVSEL_MOVE_LEFT,
	SM_ADVSEL_MOVE_RIGHT,
	SM_ADVSEL_PICK_ADV,

	SM_WARPSEL_SHOOT,

	SM_GUNCAL_MOVETONEXTPOINT,
	SM_GUNCAL_GETGUNINPUT,
	SM_GAME_WAITFORINPUT,

	SM_CHALLENGEPRESENT_SWAPRES,

	SM_BOLDER_EFFECTS,
	SM_CLIFF_COLLAPSE_EFFECT,
	SM_SHOW_HUD_IN_TOOL,
	SM_AWARD_BOSS_POINTS,
	SM_CREATE_GGG_SCORE_TRACKER,
	SM_HUD_HIDE,
	SM_HUD_SHOW_RETICLES,
	SM_HUD_HIDE_RETICLES,

	SM_ENEMY_ENABLE_ATTACKS,

	SM_SPLAT_POO,
	SM_SPLAT_BUG_EXP,
	SM_SPLAT_WATER_DROPS,
	SM_SPLAT_WATER_DROPS_REP,
	SM_SPLAT_WATER_DROPS_ROAR,
	SM_SPLAT_MUD,
	SM_SPLAT_RAIN_DROPS,

	SM_CIRCLE_BOSS_START,
	SM_CIRCLE_BOSS_END,

	SM_AUTO_FF,
	SM_LEVEL_COMPLETE,
	SM_LEVEL_COMPLETE_END,
	SM_MID_GAME_BANGUP,
	SM_MID_GAME_BANGUP_END,

	SM_LEVEL_SUCCESS_MSG,
	SM_LEVEL_FAIL_MSG,
	SM_GAME_FINISH_MSG,
	SM_CIRCLES_TUTORIAL,

	SM_SAVE_CHECKPOINT,

	SM_PICKUP_MINIGUN,

	SM_GAMESTATES_MAX,
	NUM_SM_GAMESTATES = (SM_GAMESTATES_MAX - MAX_CORESTATES),
};

enum SndFootStepStateMachineStates
{
	SM_SNDFOOTSTEP_INIT = MAX_CORESTATES,
	SM_SNDFOOTSTEP_PLAY,
	SM_SNDFOOTSTEP_MAX,
	NUM_SNDFOOTSTEP_STATES = (SM_GAMESTATES_MAX - MAX_CORESTATES)
};

enum ShakeFootStepStateMachineStates
{
	SM_SHAKEFOOTSTEP_INIT = MAX_CORESTATES,
	SM_SHAKEFOOTSTEP_PLAY,
	SM_SHAKEFOOTSTEP_MAX,
	NUM_SHAKEFOOTSTEP_STATES = (SM_GAMESTATES_MAX - MAX_CORESTATES)
};

enum AnimSFXStateMachineStates
{
	SM_ANIM_SFX_INIT,
	SM_ANIM_SFX_EXTRAFRAME,
	SM_ANIM_SFX_PLAY,
	SM_ANIM_SFX_MUTE_ANIACTION,
	SM_ANIM_SFX_MAX,
	NUM_ANIM_SFX_STATES = (SM_GAMESTATES_MAX - MAX_CORESTATES)
};


int SM_GuncalMoveToNextPoint( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_GuncalGetGunInput( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_GameWaitForInput( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_GConfSleep( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_GConfSwatch( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_GConfAnimWatch( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_GunsEnable( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_FlashlightEnable( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_LightningOn( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_LightningOff( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_BolderEffects( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_CliffCollapseEffect( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_ShowHudInTool( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_AwardBossPoints( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_CreateGGGScoreTracker( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_HudHide( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_HudShowReticles( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_HudHideReticles( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );

int SM_EnemyEnableAttacks( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SplatPoo( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SplatBugExp( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SplatWaterDrops( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SplatWaterDropsRep( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SplatWaterDropsRoar( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SplatWaterDropsRain( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SplatMud( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_CircleBossStart( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_CircleBossEnd( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_AutoFF( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_LevelComplete( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_LevelCompleteEnd( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_MidGameBangup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_MidGameBangupEnd( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SuccessMessage( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_FailMessage( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_GameFinishMsg( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_CirclesTutorial( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SaveCheckpoint( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_PickupMinigun( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );

int SM_SndFootstep_Init( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SndFootstep_Play( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );

int SM_ShakeFootstep_Init( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_ShakeFootstep_Play( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );

int SM_AnimSFX_Init( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_AnimSFX_ExtraFrames( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_AnimSFX_Play( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_AnimSFX_MuteAniAction( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );


// scene states
// int SM_BrOnGameType( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );


#endif // GAMESTATEMACHINES_H

