// high score table
#ifndef HST_H_
#define HST_H_


#include "game.h"


typedef enum HSTType
{
	HST_SCORE,
	HST_ACCURACY,
	HST_STREAK,

	NUM_HST_TYPES
	
} HSTType;

#define HST_GLOBAL	GAME_NUM_LEVELS


GameScoreTable* HST_GetTable(uns tableIdx, uns level);
GameScoreTable* HST_GetTableFrom(GameHighScores* hiScores, uns tableIdx, uns level);

// insert into the table of this type. return 1 if inserted, 0 if not.
int HST_Insert_Type(HSTType tableType, uns level, char* plyrName, double newScore);
// insert into this table.
int HST_Insert(GameScoreTable* table, char* plyrName, double newScore);

void HST_CopyTable(HSTType tableType, uns level, GameScoreTable* tableOut);
void HST_SetDefault(GameHighScores* hiScores);

// get the player's current score corresponding to this table type
int HST_GetScoreForTbl(HSTType tableType, uns pnum, double* scoreOut);

// return true if this table type with this level should be shown and updated in the game
int HST_UsingType(HSTType tableType, uns level);


#define HST_SCREEN_NUM_ENTRIES	GAME_HST_MAX_SCORES
// visible entries (slots per page)
#define HST_SCREEN_NUM_SLOTS	6
#define HST_SCREEN_NUM_PAGES	( (HST_SCREEN_NUM_ENTRIES-1) / HST_SCREEN_NUM_SLOTS + 1 )


typedef enum HSTScreenExitState
{
	// screen is still running
	HST_SCREEN_EXIT_NONE,
	// screen timed out
	HST_SCREEN_EXIT_TIMEOUT,
	// user quit screen
	HST_SCREEN_EXIT_INPUT,

} HSTScreenExitState;

void HSTScreen_Create();
void HSTScreen_Destroy();

void HSTScreen_Run(HSTType tableType, uns level, int* highlightPlyrs, int doFadeAll);
HSTScreenExitState HSTScreen_Update();


#endif