#ifndef MICRORAPTOR_STATEMACHINE_H
#define MICRORAPTOR_STATEMACHINE_H


#include "dino_agent.h"


enum MicroraptorStateMachineStates
{
	SM_MICRORAPTORSTATES_INIT = SM_ENEMYSTATES_MAX,
	SM_MICRORAPTORSTATES_INIT_ATTACK,
	SM_MICRORAPTORSTATES_DESTROY,
	SM_MICRORAPTORSTATES_SHOT,
	SM_MICRORAPTORSTATES_FREEZE,
	SM_MICRORAPTORSTATES_WAIT_UNTIL_LAUNCH,
	SM_MICRORAPTORSTATES_WAIT_UNTIL_ATTACK,
	SM_MICRORAPTORSTATES_WAIT_UNTIL_DONE,

	SM_MICRORAPTORSTATES_MAX,
	NUM_SM_MICRORAPTORSTATES = (SM_MICRORAPTORSTATES_MAX - SM_ENEMYSTATES_MAX + NUM_SM_ENEMYSTATES),
};


typedef enum MicroraptorAttackState
{
	MRAPTOR_ATK_IDLE,
	// start attack sequence
	MRAPTOR_ATK_LAUNCH,
	// jump into the air
	MRAPTOR_ATK_JUMP,
	// fly toward target
	MRAPTOR_ATK_PURSUE,
	// fly toward a point offset from target
	MRAPTOR_ATK_PURSUE_OFFSET,
	// slow down and arrive at offset
	MRAPTOR_ATK_ARRIVE,
	// hover in front of screen
	MRAPTOR_ATK_HOLD,
	// fly at and hit target
	MRAPTOR_ATK_HIT_TGT,
	// damage player
	MRAPTOR_ATK_DAMAGE,
	// get off the screen
	MRAPTOR_ATK_FLEE,
	MRAPTOR_ATK_DONE,
	// dead in the air
	MRAPTOR_ATK_DEAD,
	// freeze ray death
	MRAPTOR_ATK_FROZEN,
	MRAPTOR_ATK_FROZEN_SHATTER,
	// dead and lying on the ground
	MRAPTOR_ATK_DEAD_ON_GROUND,
	// fly straight, ignoring player position
	MRAPTOR_ATK_FLY_IGNORE,

} MicroraptorAttackState;


typedef struct MicroraptorAttackParams
{
	// this mraptor's statemachine params
	SM_Microraptor_Params* params;
	// steering agent for this mraptor
	SteeringAgent agent;
	// mraptor's position at the start of the attack
	crd pos0[3];
	crd dir0[3];
	// direction mraptor is facing
	crd dir[3];
	crd rollAng;
	crd deathAng;
	crd deathAngSpeed;
	crd deathAxis[3];
	// random roll offset
	crd rollOffset;
	MicroraptorAttackState state;
	// local transform
	ObjTransform* otf;
	// angles to rotate the seek vector
	crd seekRot[3];
	// final camera-space target offset vector
	crd camOffs[3];
	// current target in camera space
	crd camTgt[3];
	float atkAng;
	// amount of time left to spend in hold state
	float holdTimeLeft;
	// distance from target when transitioning to arrive state
	float arriveDist;

	// target object
	Obj* otgt;
	// elapsed time in milliseconds since start of attack
	uns32 elapsedTime;
	// time of jump after start of attack
	uns32 jumpTime;
	uns32 deathTime;
	// minimum distance to target to end attack
	float endDist;
	// distance to target after last update
	float curDist;

	// true if decided for sure that the microraptor will show up for the
	// current number of players.
	int willLaunch;

	// true if mraptor was in view of the camera.
	// used just for no-attack guys for now.
	int wasSeen;

	float relativity;

	// wing flapping sound
	uns32 wing_flap_sound_id;

	ObjFunc* destructor;

} MicroraptorAttackParams;


extern PSM_State gMicroraptorStateMachineStates[NUM_SM_MICRORAPTORSTATES];
extern PSM_StateMachine gMicroraptorStateMachine;


int SM_Microraptor_Init( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Microraptor_Init_Attack( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Microraptor_Destroy( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Microraptor_Shot( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Microraptor_Freeze( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Microraptor_RunSpline( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
// wait until microraptor should start its attack
int SM_Microraptor_WaitUntilLaunch( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
// wait until player is close enough to damage
int SM_Microraptor_WaitUntilAttack( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Microraptor_WaitUntilDone( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );


#endif
