
#ifndef RAPTOR_GENERAL_H
#define RAPTOR_GENERAL_H

#include "dino_group.h"


//////////////////////////////////////////////////////////////////////////
// helpful defines
//////////////////////////////////////////////////////////////////////////

#define GROUND_GRAVITY_TWEAK			1.0f
#define CLIMB_GRAVITY_TWEAK				5.0f
#define RAPTOR_COMPRESS_FRAMES			20.0f
#define RAPTOR_LAND_LOOKAHEAD_FRAMES	26.0f
#define RAPTOR_SLOW_TIME				250.0f

#define RAPTOR_PERCH2JUMP_OFF_X			0.0f
#define RAPTOR_PERCH2JUMP_OFF_Y			3.689f
#define RAPTOR_PERCH2JUMP_OFF_Z			2.728f

#define RAPTOR_CAM_LEAD					12.0f
#define RAPTOR_STRIKE_FRAME				26.0f

#define RAPTOR_SCALE					1.25f


//////////////////////////////////////////////////////////////////////////
// raptor enums
//////////////////////////////////////////////////////////////////////////

enum RAPTOR_MODELS
{
	RAPTOR_MODEL_RAPTOR=500,
	RAPTOR_MODEL_DILO,
};

// raptor behavior types
enum RAPTOR_TYPES
{
	RAPTOR_TYPE_NONE=1000,
	RAPTOR_TYPE_LOCKDOWN,
	RAPTOR_TYPE_CHASE_BEHIND,
	RAPTOR_TYPE_MOUNT,
	RAPTOR_TYPE_AUTO_GRAPPLE,
	RAPTOR_TYPE_TANK,
};

// raptor climb states
enum RAPTOR_CLIMB_STATES
{
	RAPTOR_CLIMB_NONE=2000,
	RAPTOR_CLIMB_MOUNT,
	RAPTOR_CLIMB_UP,
	RAPTOR_CLIMB_2_PERCH,
	RAPTOR_CLIMB_PERCH_2_JUMP,
	RAPTOR_CLIMB_JUMP_2_FALL,
	RAPTOR_CLIMB_LAND,
	RAPTOR_CLIMB_LAND_2_RUN,
};

// raptor attack types
enum RAPTOR_ATTACK_TYPES
{
	RAPTOR_ATTACK_TYPE_CLOSE=7000,
	RAPTOR_ATTACK_TYPE_GRAPPLE,
	RAPTOR_ATTACK_TYPE_SPIT,
	RAPTOR_ATTACK_TYPE_NONE
};

// raptor attack states
enum RAPTOR_ATTACK_STATES
{
	RAPTOR_ATTACK_NONE=8000,
	RAPTOR_ATTACK_SLOW,
	RAPTOR_ATTACK_COMPRESS,
	RAPTOR_ATTACK_COMPRESS_RUNNING,
	RAPTOR_ATTACK_COMPRESS2JUMP,
	RAPTOR_ATTACK_JUMP,
	RAPTOR_ATTACK_JUMP2MAX,
	RAPTOR_ATTACK_JUMP2LAND,
	RAPTOR_ATTACK_LAND,
	RAPTOR_PATTACK,
	RAPTOR_ATTACK_DONE
};

enum RAPTOR_PATTACK_STATES
{
	RAPTOR_PATTACK_NONE=9000,
	RAPTOR_PATTACK_SLOW,
	RAPTOR_PATTACK_FIND_LPOS,
	RAPTOR_PATTACK_SEEK_LPOS_ZIG,
	RAPTOR_PATTACK_SEEK_LPOS_ZAG,
	RAPTOR_PATTACK_TRUN,
	RAPTOR_PATTACK_DONE
};

enum RAPTOR_CLOSE_ATTACK_STATES
{
	RAPTOR_CLOSE_ATTACK_NONE=10000,
	RAPTOR_CLOSE_ATTACK_SLOW,
	RAPTOR_CLOSE_ATTACK_TAUNT,
	RAPTOR_CLOSE_ATTACK_SWIPE,
	RAPTOR_CLOSE_ATTACK_DONE
};

enum RAPTOR_SPIT_ATTACK_STATES
{
	RAPTOR_SPIT_ATTACK_NONE=10500,
	RAPTOR_SPIT_ATTACK_SLOW,
	RAPTOR_SPIT_ATTACK_TAUNT,
	RAPTOR_SPIT_ATTACK_SPIT,
	RAPTOR_SPIT_ATTACK_DONE
};

// raptor death states
enum RAPTOR_DEATH
{
	RAPTOR_DEATH_NONE=11000,
	RAPTOR_DEATH_START,
	RAPTOR_DEATH_AIR_HIT,
	RAPTOR_DEATH_AIR,
	RAPTOR_DEATH_LAND,
	RAPTOR_DEATH_FACEPLANT_START,
	RAPTOR_DEATH_FACEPLANT,
	RAPTOR_DEATH_EXPLOSION,
	RAPTOR_DEATH_FREEZE_HOLD,
	RAPTOR_DEATH_FREEZE_SHATTER,
	RAPTOR_DEATH_ELECTRO
};

enum RAPTOR_LOCKDOWN_STATES
{
	RAPTOR_LOCKDOWN_NONE=12000,
	RAPTOR_LOCKDOWN_INIT,
	RAPTOR_LOCKDOWN_WAIT,
	RAPTOR_LOCKDOWN_IDLE,
	RAPTOR_LOCKDOWN_SEEK,
	RAPTOR_LOCKDOWN_ATTACK,
	RAPTOR_LOCKDOWN_JUMP,
	RAPTOR_LOCKDOWN_CLIMB,
	RAPTOR_LOCKDOWN_TANK,
};

enum RAPTOR_CHASE_BEHIND
{
	RAPTOR_CB_INIT=13000,
	RAPTOR_CB_WAIT_TRIGGER,
	RAPTOR_CB_WAIT_CAM,
	RAPTOR_CB_START,
	RAPTOR_CB_PICK_POS,
	RAPTOR_CB_GOTO_POS,
	RAPTOR_CB_SLOW_POS,
	RAPTOR_CB_MAINTAIN_POS,
	RAPTOR_CB_JUMP,
	RAPTOR_CB_ATTACK_CHARGE,
	RAPTOR_CB_ATTACK,
	RAPTOR_CB_RUN,
};

enum RAPTOR_MOUNT_STATES
{
	RAPTOR_MOUNT_NONE=12000,
	RAPTOR_MOUNT_INIT,
	RAPTOR_MOUNT_WAIT,
	RAPTOR_MOUNT_ATTACK,
};

enum RAPTOR_AUTOGRAPPLE_STATES
{
	RAPTOR_AUTOGRAPPLE_NONE=16000,
	RAPTOR_AUTOGRAPPLE_WAIT,
	RAPTOR_AUTOGRAPPLE_LAUNCH,
	RAPTOR_AUTOGRAPPLE_ATTACK,
	RAPTOR_AUTOGRAPPLE_FALL_OFF,
};

enum RAPTOR_GRAPPLE_ATTACK_STATES
{
	RAPTOR_GRAPPLE_ATTACK_NONE=15000,
	RAPTOR_GRAPPLE_ATTACK_SLOW,
	RAPTOR_GRAPPLE_ATTACK_TAUNT,
	RAPTOR_GRAPPLE_ATTACK_LAUNCH,
	RAPTOR_GRAPPLE_ATTACK_BITE_LOOP,
	RAPTOR_GRAPPLE_ATTACK_DONE
};

enum RAPTOR_LOOK_STATES
{
	RAPTOR_LOOK_NONE=0,
	RAPTOR_LOOK_RIGHT,
	RAPTOR_LOOK_LEFT,
};

enum RAPTOR_ANIM_CHANNELS
{
	RAPTOR_CHANNEL_NORMAL=0,
	RAPTOR_CHANNEL_LOOK,
	RAPTOR_CHANNEL_BITE, // snapping teeth at objects they hate
	RAPTOR_CHANNEL_DAMAGED,
	RAPTOR_CHANNEL_HOOD, // for dilo frilled hood
	RAPTOR_CHANNEL_FREEZE
};

enum RAPTOR_INTRO_TYPES
{
	RAPTOR_INTRO_NORMAL=0,
	RAPTOR_INTRO_FALL,	// Fall exactly x meters away from camera
	RAPTOR_INTRO_FALL_POS, // put him right where he's placed in maya
	RAPTOR_INTRO_TANK,
	RAPTOR_INTRO_NUM_TYPES
};

enum RAPTOR_DEATH_TYPES
{
	RAPTOR_DEATHTYPE_AIR=0, // The default -> is no one sets a death type, it will remain this one
	RAPTOR_DEATHTYPE_FACEPLANT,
	RAPTOR_DEATHTYPE_FALL,
	RAPTOR_DEATHTYPE_HIGH_HIT,
	RAPTOR_DEATHTYPE_THROWN,
	RAPTOR_DEATHTYPE_SCREEN,
	RAPTOR_DEATHTYPE_FREEZE,
	RAPTOR_DEATHTYPE_ELECTRO
};

//////////////////////////////////////////////////////////////////////////
// raptor complex data types
//////////////////////////////////////////////////////////////////////////

// obstacle jump data

enum RAPTOR_JUMP_TYPE
{
	RAPTOR_JUMP_STOPPED,
	RAPTOR_JUMP_RUN
};

typedef struct RaptorObsJump
{
	crd jump_to[3];
	float jump_time;
	int jump_type;
}RaptorObsJump;

typedef struct RaptorCB	// chase behind data
{
	crd offset[3];
	uns32 attack_stick;
	uns32 attack_time;
}RaptorCB;


typedef struct RaptorAttackAnim
{
	char* anim;
	float attackFrame;
}RaptorAttackAnim;




typedef struct RaptorModelData
{
	int num_skins;
	char** skin_names;
}RaptorModelData;


enum RaptorFlankType
{
	RAPTOR_FLANK_TRICERA,
	RAPTOR_FLANK_JEEP,
	RAPTOR_FLANK_TRICERA_OVERRIDE,
};



// primary data struct
typedef struct RaptorData
{
	RaptorModelData * rmd;	// model specific info
	int model;				// raptor or dilo
	int type;				// type of raptor behavior
	int state;				// main raptor state
	float start_time;
	char start_trigger[SM_NAME_SIZE];
	char end_trigger[SM_NAME_SIZE];
	char killed_trigger[SM_NAME_SIZE]; // when raptor is killed, send this out
	float start_cam_time;	// if we keep same camera while waiting to start - use this to start raptor
	uns32 start_cam_oid;		// this is the camera we are watching to decide to start
	float start_delay;		// If start camera changes, just use a normal timer to start the raptor
	float idle_delay;
	float end_delay; // delay before killing with end trigger
	int min_players; // how many players need to be in game for this guy to be allowed to spawn
	int max_players;
	int auto_noattack; // if this raptor isn't allowed to go because not enough/too many players -> make him a no attack one
	int intro_type; // how will raptor come in to scene

	float actual_start_time;		// camera time that the raptor started running on. 
	float estimated_attack_time;	// camera time the raptor is expected to attack on.
	
	// Appearance
	float scale;
	float color_rgba[4];

	// climbing
	float jump_time;

	// new climb
	float climb_height;

	// jumping
	int allow_jump;
	int jump_down;
	crd jump_target[3];
	float attack_jump_time;
	float jump_last_y;
	crd chomp_offset[3];
	float gravity;
	float land_time_left;
	RaptorObsJump obs_jump;

	RaptorCB cb_data;

	uns32 tankOID; // for when they are waiting for raptor tank to be destroyed
	float tankDelay;

	// animations
	uns32 run2climb_track;
	uns32 climb2perch_track;
	uns32 perch2jump_track;
	uns32 jump_in_air_track;
	uns32 jump_look_track;
	uns32 jump_land_track;
	uns32 jump_ground_track;
	uns32 death_ground_track;
	uns32 death_inair_hit_track;
	uns32 death_inair_track;
	uns32 death_land_track;
	uns32 jump_attack_track;
	uns32 charge_attack_track;
	uns32 taunt_track;
	uns32 look_track;
	uns32 attack_track;
	uns32 hit_reaction_track;
	uns32 dilo_hood_track;
	uns32 freeze_track;
	uns32 electro_track;

	// Looking
	uns32 look_state;

	float last_mount_time; // last anim time of mount target anim

	// turn smoothing
	float last_dir_quat[4];

	// ground type
	float max_speed;
	float seek_force;
	float shot_slowdown;

	// spline to agent
	int agent_ready;

	// attack
	int attack_state;		// state for attack sequence
	int pattack_state;		// position for attack
	int attack_type;
	float aradius;
	float pradius;
	float p1radius;
	float slow_stvel_mag;	// org mag to slow from
	uns32 slow_stick;		// timer for slowing
	int delt_damage;
	int objkill_timer;
	crd pattack_offset[3];
	crd zig_pt[3];	
	float taunt_time;
	int kill_on_offscreen_attack;
	int run_to_cam_frame; // they will try to run in front of camera by default, or they can just run straight to camera regardless of where its facing
	float attack_timer;
	int attack_cancels; // number of times their attack may be canceled by being shot
	float attack_charge_speed;

	int attack_anim;
	float grapple_chance;
	float grapple_halt_timer;
	float grapple_halt_delay; // slight pause before halting so they don't all sync up
	float auto_grapple_angle;
	float fake_attack_target_chance; // chance to leap at one of the jeeps its chasing

	crd no_attack_target[3];
	float no_attack_time;

	uns32 alt_target; // if they are targetting someone that isn't player-cam
	int flank_type;
	float flank_override;
	

	// death
	int death_type;
	uns32 death_state;
	float health;
	float mp_health_multiplier;
	float hit_blink_timer;
	float hit_anim_time;
	int hit_anim_grapple;
	int death_anim;
	float lifetime;

	crd screen_pos[3];
	crd screen_vel[3];

	// sound
	uns32 snd_screech_id;

	// explosion reaction
	uns32 exp_id;

	// freeze gun reaction
	ObjDispRes* freeze_destruct;
	int freeze_jumping;
	int freeze_climb;
	int freeze_pose;
	float electro_pose_time;

	SteeringAgent agent;
	SM_TravelFunc_Params travelParams;

	// commonly used stuff
	crd target_pos[3];
	crd target_last_pos[3];
	crd target_rot[3];
	crd target_last_rot[3];
	Obj* parent;
	ObjKeyAnim* okan;
	ObjVel* ov;
	ObjTrackAnim* otan;
	ObjKeyAnim* target_okan;
	ObjKeyAnim* camera_okan;
}RaptorData;



//////////////////////////////////////////////////////////////////////////
// functions
//////////////////////////////////////////////////////////////////////////

// functions
void Raptor_ModelInit_Rptr(RaptorData* rd);
void Raptor_ModelInit_Dilo(RaptorData* rd);

// vel controller
int Raptor_Update( ObjVel* objVel, float deltaTime );
int Raptor_Free( ObjVelController* velCtrl );

// types
int Raptor_Lockdown( RaptorData* rd, float deltaTime );
int Raptor_Mount( RaptorData* rd, float deltaTime );
int Raptor_AutoGrapple( RaptorData* rd, float deltaTime );

// attack
void Raptor_StartPAttack( RaptorData* rd );
void Raptor_PAttack( RaptorData* rd, float deltaTime );

// Close Attack
void Raptor_PrepCloseAttack( RaptorData* rd );
void Raptor_Close_Attack( RaptorData* rd, float deltaTime );


//Grapple Attack
void Raptor_Grapple_Attack( RaptorData* rd, float deltaTime );

//Spit
void Raptor_Spit_Attack( RaptorData* rd, float deltaTime );


// death
void Raptor_GetInAirShotReaction( RaptorData* rd, crd* out_vel );
void Raptor_HandleDeath_Screen( RaptorData *rd, float deltatime);
void Raptor_HandleDeath_Freeze( RaptorData *rd, float deltatime);
void Raptor_HandleDeath_Electro( RaptorData *rd, float deltatime);
void Raptor_MakeFreezeDestruct( RaptorData* rd );
int Raptor_HandleDeath( RaptorData* rd, float deltaTime );

int Raptor_LookForKillCol(RaptorData* rd);

// lockdown
int Raptor_Jump(RaptorData* rd, float deltaTime);
int Raptor_LookForJump(RaptorData* rd);

void Raptor_Lockdown_Climb(RaptorData* rd, float deltaTime);
int Raptor_Lockdown_LookForClimb(RaptorData* rd);

void Raptor_Halt(RaptorData* rd, float deltaTime);

// chase behind
int Raptor_Chase_Behind( RaptorData* rd, float deltaTime );
void Raptor_CB_GetWorldOffset( RaptorData* rd, crd* wpos, crd* tvel );
void Raptor_FakeAttack(RaptorData *rd, crd* target_pos);

// helpers
int Raptor_Start( RaptorData* rd );
void Raptor_Kill(RaptorData* rd);
float Raptor_JumpSlow( RaptorData* rd, float deltaTime );
int Raptor_NextFrameGroundHit( RaptorData* rd, float offset );
int Raptor_Travel_ObjVel( RaptorData* rd, float deltaTime, float pitch );
int Raptor_SmoothTurn( RaptorData* rd, crd* target_pos, float* ang_diff );
uns32 Raptor_GetBoneID( RaptorData* rd, const char* name );
void Raptor_ClearLanding( RaptorData* rd, float deltaTime );
void Raptor_ScreechSFX( RaptorData* rd );
void Raptor_ChaseBehind_ScreechSFX( RaptorData* rd );
void Raptor_DeathSFX( RaptorData* rd );
void Raptor_SpitSFX( RaptorData* rd );
void Raptor_FreezeSFX( RaptorData* rd );
void Raptor_ShatterSFX( RaptorData* rd );
void Raptor_LandSFX( RaptorData* rd );


float GetAngleFromCam(RaptorData* rd, float time);
float GetAngleFromPosRot(crd* targetPos, crd* camPos, crd *camRot );
int RaptorIsInCamFrame(RaptorData* rd, float toleranceDegrees);
float GetAngleToPos(RaptorData* rd, crd* pos);
int RaptorIsInCamFrame(RaptorData* rd, float toleranceDegrees);
int RaptorIsInCamFrameAtTime(RaptorData* rd, float toleranceDegrees, float time);
int IsRaptorRunning(RaptorData* rd);


void Raptor_FacePlayer( RaptorData* rd, crd* target);


void Raptor_LookAt(RaptorData *rd, crd* lookAtPos);


int GetRaptorAuditAttackType(RaptorData* rd);
void RaptorRecordKillAudit(RaptorData *rd, int pnum);
void RaptorRecordDamageAudit(RaptorData *rd);


#define MAX_FLANK_POSITIONS	10
typedef struct FlankGroup
{
	int num_positions;							// Number of flank positions
	float reservations[MAX_FLANK_POSITIONS];	// Reservation times dinos have taken (array of size positions)
	float zDist;								// How far away they will flank toward
	float width;								// Total angle around center of camera they can flank
	crd (*positions)[3];						// If there are hardcoded positions, store them here (pointer to array of 3d coordinatess)
} FlankGroup;

// Flank Position
void ResetFlankGroups();
void GetFlankPosition(RaptorData* rd, FlankGroup* group, float reservationTime, crd* resultPos);



#define MAX_RAPTOR_GRAPPLE_COUNT 1
#define MAX_RAPTOR_ATTACK_COUNT 2 
#define NUM_RAPTOR_TRICERA_MOUNTS 5

#define MAX_CHASE_RAPTOR_ATTACK_COUNT 1

typedef struct RaptorManager
{
	int AttackEnabled;

	uns32 Grapple_oids[MAX_RAPTOR_GRAPPLE_COUNT];
	uns32 Attack_oids[MAX_RAPTOR_ATTACK_COUNT];	
	uns32 TriceraMount_oids[NUM_RAPTOR_TRICERA_MOUNTS];


	FlankGroup FlankLockdown;
	FlankGroup FlankChase;
	FlankGroup FlankTriceraChase;
	FlankGroup FlankJeepChase;

	uns32 lastDeathSFXTime;

} RaptorManager;


void RaptorManager_Reset(RaptorManager* rm);

RaptorManager g_RaptorManager;



void RegisterOID(uns32 oid, uns32* oid_list, int list_size, int* out_registerIndex, int* out_prevCount);
int UnregisterOID(uns32 oid, uns32* oid_list, int list_size, int* out_activeLeft);
int GetOIDCount(uns32* oid_list, int list_size);


void RegisterAttack(RaptorData* rd);
void UnregisterAttack(RaptorData* rd);
int GetAttackCount();

void RegisterGrapple(RaptorData* rd, int pause_target);
void UnregisterGrapple(RaptorData* rd);
int GetGrappleCount();

int RegisterMount(RaptorData* rd);
void UnregisterMount(RaptorData* rd);
int GetMountCount();

void Raptor_ExplosionCallback( ExplosionData* exp_data, void* data );


#endif


