//

#ifndef FLYER_H
#define FLYER_H

#include "g5.h"

#define FLYERFLAG_TRANSFORM_DIRTY	0x01
#define FLYERFLAG_HAS_BEEN_HIT		0x02
#define FLYERFLAG_HIDE_NEXT_FRAME	0x04

typedef void (FlyerUpdateFunc)(void* flyer);
typedef struct FlyerElements
{
	void* group;
	uns32 flags;
	float transform[VM_SIZE];

	float time;	

	float worldpos[3];
	float worldrot[3];
	float pos[3];
	float vel[3];
	float acc[3];
	
	float rot[3];
	float rotVel[3];

	float radiusSq;
} FlyerElements;


#define FLYERGROUPFLAG_KILL_WHEN_DONE	0x01
#define FLYERGROUPFLAG_ANIMATED			0x02
#define FLYERGROUPFLAG_ONE_FX			0x04

typedef struct FlyerGroup
{
	FlyerElements* elements;
	int numElements;
	int numTransformFloats;
	float offset;

	void* parentOdr;
	FlyerUpdateFunc* updateFunc;
	char exploder[RES_NAME_SIZE];
	char fx[RES_NAME_SIZE];
	uns32 groupFlags;
} FlyerGroup;

void*	FlyerAddNew(char* instanceName, N2* link, char* shader, int lgroup, char* resName, float* pos, float* rot);
void*	FlyerAddRes(void* pObj, Res* res);
void		FlyerDestroy(FlyerGroup* flyerGroup);
void		FlyerAddElements(Res* res, void* parentOdr, FlyerGroup* flyerGroup, ResDataDisp5* resdisp5, char* type);
// set the positions of the wind-effected elements
void		FlyerSetPositions(FlyerGroup* flyerGroup, float* positions, int count);
// get the transforms (and stride [in # of floats] between transforms) of wind elements
int			FlyerGetTransforms(FlyerGroup* flyerGroup, float** transforms, uns32* stride);

int			FlyerGetNumElements(FlyerGroup* flyerGroup);

void		FlyerUpdate(void* objFlyer, uns32 deltaTime);
int			FlyerLoop(uns32 deltaTime);



#endif

