#ifndef PHYSX
#define PHYSX

// #define PX_DISABLE_PHYSX




// #define USE_PX_TRACK 1
#ifdef USE_PX_TRACK
EXTERN_C void PXTrackInit();
EXTERN_C void PXTrackAdd( char *type, uns32 gen, char *descr );
EXTERN_C void PXTrackFree( char *type, uns32 gen );
EXTERN_C void PXTrackLog( );
#else
#define PXTrackInit(...)
#define PXTrackAdd(...)
#define PXTrackFree(...)
#define PXTrackLog(...)
#endif

typedef struct PhysxExtendedData
{
	uns32 oid; // NOTE: this member should remain first, to maintain compatibility
			   //	with the other physx objects that only store oid.
	int meshIndex;
	int windIndex;
	void (*triggerFunc)(struct PhysxExtendedData* trigger, struct PhysxExtendedData* otherObj);
	int dynamicReverseUpdate; // if 1, then this actor will act as a kinematic object, despite not being one
} PhysxExtendedData;

// public defines
#define PHYSX_CATCHALL_PLANE_HEIGHT -500.0

// physxInit() - initializes entire physics engine and
//		establishes a scene with default gravity, friction, etc.
EXTERN_C int physxInit();

// deletes all physx objects and re-initializes PhsyX scene
// and resets the gravity, friction & restitution.
// call during wipeout
EXTERN_C int physxReset();

// deletes all physx objects, shuts physx engine down
EXTERN_C int physxFinal();

// physxLoop() - to be called once per frame.
//		First, it grabs some data from the game engine
//		Then, it simulates the next timestep,
//		and finally, sends the new data back to the game engine.
EXTERN_C void physxLoop();

// physxSimPause() - call this to freeze the simulation.
//		The simulation can then be continued at any time
//		via physxSimResume(). Returns 0 on success, -1
//		if the simulation is already paused.
EXTERN_C int physxSimPause();

// physxSimResume() - call this to resume simulation after
//		pausing with physxSimPause(). Returns 0 on success,
//		and -1 on failure (if sim isn't paused).
EXTERN_C int physxSimResume();

// physxSetGravity() - sets scene gravity to any vector value.
//		Default gravity = (0.0, -9.8, 0.0)
EXTERN_C void physxSetGravity(float grav_x, float grav_y, float grav_z);
EXTERN_C void physxGetGravity(float* x, float* y, float* z);

// physxSetLinearDampening() - sets 'air resistance' of object
//		Higher value = more deceleration
//		Default dampening = 0
EXTERN_C int physxSetLinearDampening(Obj* obj, float dampening);

// physxSetVelocity() - sets the velocity of any object
//		to any vector.
EXTERN_C int physxSetVelocity(Obj* obj, float vel_x, float vel_y, float vel_z);
EXTERN_C int physxSetRotVel(Obj* obj, float rotvel_x, float rotvel_y, float rotvel_z);

// physxGetVelocity() - gets the velocity of the object and pokes
//		those values into the float pointers.
EXTERN_C int physxGetVelocity(Obj* obj, float* vel_x, float* vel_y, float* vel_z);
EXTERN_C int physxGetRotVel(Obj* obj, float* rotvel_x, float* rotvel_y, float* rotvel_z);

// physxGetVelMagnitude() - gets the magnitude of the object's velocity.
//		Essentially returns the speed of the object. Returns FLOAT_NAN
//		if the object cannot be found.
EXTERN_C float physxGetVelMagnitude(Obj* obj);

// physxSetVelMagnitude() - sets the velocity magnitude of
//		any object to any value. This essentially rescales the
//		current velocity vector of the object. I'm not sure what
//		occurs when you set the magnitude of an object that wasn't moving.
EXTERN_C int physxSetVelMagnitude(Obj* obj, float velMag);

// physxSetCatchAllPlane() - sets the height of the catch-all plane
//		that sits underneath the world.
EXTERN_C void physxSetCatchAllPlane(float height);

// physxSetMaterials() - sets the material surface values for any object.
//		restitution = collision elasticity (i.e. "bounciness")
//		staticFriction = coefficient of friction to put an object in motion from rest
//		dynamicFriction = coefficient of friction for objects already in motion
//		All three values should GENERALLY be between 0.0 and 1.0
//		Returns 0 on success, -1 on object not found
EXTERN_C int physxSetMaterials(Obj* obj, float restitution, float staticFriction, float dynamicFriction);

// physxGetBoundingBoxDimensions() - gets the dimensions of the object's axis-aligned
//		bounding box in each axis. Dimensions will be put into data pointed to by
//		xDim, yDim, and zDim. Returns 0 on success and -1 on failure.
EXTERN_C int physxGetBoundingBoxDimensions(Obj* obj, float* xDim, float* yDim, float* zDim);

// physxAddPartSys() - add a particle system.
/*EXTERN_C void* physxAddPartSys(char* name,										// Name this particle system
							  float pos_x, float pos_y, float pos_z,			// Position of particle emission in global space
							  float posVar_x, float posVar_y, float posVar_z,	// Each particle will spawn within a box of these dimensions around the initial pos
							  float vel_x, float vel_y, float vel_z,			// Initial velocity of each particle
							  float velVar_x, float velVar_y, float velVar_z,	// Each particle's initial vel will be within +/- these values of the initial vel
							  float* rotationMatrix,							// Matrix to rotate all positions and velocities with
							  float scaleMin, float scaleMax,					// Range of possible radii for each particle
							  int numParticles,									// Number of particles to spawn
							  ResDataDisp2* res,								// Resource to display (if NULL, white spheres are used)
							  int selfCollide,									// 1 if particles can collide with others in the same system, 0 otherwise (should probably be 0 in general cases)
							  int isBillboarded,								// 1 if 'res' should be billborded, 0 if not
							  int ignoreOID);									// Object to ignore collisions with (used when spawning particles very close to another)
*/
// physxAddStaticObj() - adds the object to the physics simulation as
//		an unmoving solid object.
EXTERN_C int physxAddStaticObj(Obj* obj);

// physxAddStaticObj() - adds the object to the simulation as a sphere
//		with a radius determined by the culling sphere of that object
EXTERN_C int physxAddSimpleStatic(Obj* obj, float radius, PhysxExtendedData* extData);

// physxAddDynamicObj() - adds the object to the simulation as a
//		solid, movable object subject to gravity, friction, etc.
//		"density" merely needs to be correct relative to other
//		objects in the scene
EXTERN_C int physxAddDynamicObj(Obj* obj,float density);
EXTERN_C int physxAddDynamicObjPriority(Obj* obj,float density, uns32 priority, int setPivot);

// physicsAddSimpleDynamic() - adds the object to the simulation
//		as a solid, dynamic sphere who's radius is the average
//		of the distances between the verts and the object's origin.
EXTERN_C int physxAddSimpleDynamicObj(Obj* obj, float density);
EXTERN_C int physxAddSimpleDynamicObjPriority(Obj* obj, float density, uns32 priority, int setPivot);

// physxAddSkelObj() - adds a disp3 object with a parts list
//		and collision meshes & skeleton to the simulation.
//		It's updated every frame based on the latest bindMats
EXTERN_C int physxAddSkelObj(Obj* obj,			// disp3 to add
							 int physxStatus, // see physxStatus enum
							 float density);	// only used for dynamic

// physxSetKinematic() - used to turn kinematic motion (game-driven motion)
//		on or off for a given object. Set 'onOff' to 1 to turn it on, 0 to
//		turn it off. Returns 0 on success, -1 on failure (obj not found).
EXTERN_C int physxSetKinematic(Obj* obj, int onOff);

EXTERN_C int physxSetCollisionResponse(Obj* obj, int enable);

// physxAddSphericalTrigger() - used to add a trigger callback for this object.
//		the trigger callback function will be called every time something else in
//		physx passes into the trigger object.
EXTERN_C int physxAddSphericalTrigger(Obj* obj, void (*callback)(PhysxExtendedData* trigger, PhysxExtendedData* otherObj), int whenToTrigger);

// physxAddJoint() - add a joint that links two PhysX actors together.
//		the position is the point in world space at which both of the
//		actors are joined together. Strength is the breaking force of the
//		joint. Set to zero for infinitely strong joint. maxDist & minDist
//		are the max/min distances for the rope joint. Set to zero for no limit.
//		Variable allowConjoinedObjCollision decides whether the two objects
//		connected by the joint should still collide with each other.
EXTERN_C int physxAddJoint(Obj* obj1, Obj* obj2, int jointType,
						   float pos_x, float pos_y, float pos_z,
						   float strength, float maxDist, float minDist,
						   int allowConjoinedObjCollision);

// physxSetJointStrength() - sets the maximum force/torque that a joint can
//		withstand before it breaks. This value is expressed in amount of
//		physx force (which is relative to the masses in the simulation).
EXTERN_C int physxSetJointStrength(Obj* joint, float str);

// physxIsJointValid() - returns 1 if the joint is valid and 0 if it is
//		broken or not found.
EXTERN_C int physxIsJointValid(Obj* joint);

// physxRemoveJoint() - removes the joint specified by 'joint'. Note that
//		this will also invalidate the game object pointed to by 'joint', so
//		that game object is also deleted. Returns 0 on success, -1 on failure.
EXTERN_C int physxRemoveJoint(Obj* joint);

// physxSetNonCollideGroup() - puts the actor in a physx group that
//		will not collide with other actors in the same group
//		NOTE: Group must be between 1 and 31.
EXTERN_C int physxSetNonCollideGroup(Obj* obj, int group);

EXTERN_C int physxTestSphereOverlap(float* sphereCenter, float sphereRadius, PhysxExtendedData** collidingObjs);

// physxRemove() - removes that specific object from the simulation
//		NOTE: it does NOT remove it from the Obj list, but only 
//		from the simulation. Also note that objects can be re-added
//		at any time. Thus, an object can be added as dynamic, allowed
//		to come to rest, then removed and re-added as static.
EXTERN_C int physxRemove(Obj* obj);

// physxResetScene() - removes everything and resets the gravity,
//		friction & restitution.
// EXTERN_C void physxResetScene();

// physxRenderScene() - to be called once per frame. This function renders
//		the physx particles as well as collision mesh debug wireframes (activated with dbg(px=1))
EXTERN_C void physxRenderScene();

// physxSetLocRot() - sets any object's location and rotation.
//		NOTE: not functional for disp3 objects. In most cases,
//		it's better to move the object in the game engine using
//		the obj() cf call, which then automatically calls this.
EXTERN_C void physxSetLocRot(Obj* obj,
							 float pos_x, float pos_y, float pos_z,
							 float rot_x, float rot_y, float rot_z);

// physxSetCollisionGroup() - sets the object into a group
//		that can be flagged to not collide with 
// physxGetNumStaticShapes() - retrieves the number of static
//		shapes in the current scene.
EXTERN_C int physxGetNumStaticShapes();

// physxGetNumDynamicShapes() - retrieves the number of dynamic
//		shapes in the current scene.
EXTERN_C int physxGetNumDynamicShapes();

// physxGetNumShapes() - retrieves the total number of
//		shapes in the current scene.
EXTERN_C int physxGetNumShapes();

/////////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\////////////////////////////////////

EXTERN_C int physxAddDynamicNOW(Obj* obj, float density, uns32 priority, int kinematic);
EXTERN_C int physxAddSimpleNOW(Obj* obj, float density, uns32 priority, int kinematic);

// physxWalkObjects() - calls the parameter function on each actor
//		in the current scene. Used by various functions above.
EXTERN_C void physxWalkObjects( void (*function)(void* actor) );
EXTERN_C void physxWalkObjectsParam( void (*function)(void* actor, float parameter), float parameter );

// physxWalkJoints() - calls the parameter function on each joint
//		in the current scene.
EXTERN_C void physxWalkJoints( void (*function)(void* joint) );

// physxAddParticle() - creates a single particle as a dynamic actor
//		Used by physxAddPartSys()
/*EXTERN_C void* physxAddParticle(float radius,
								float pos_x, float pos_y, float pos_z,
								float vel_x, float vel_y, float vel_z,
								void* data);
*/
// physxRender() - if the parameter actor is a particle, it is rendered,
//		otherwise it returns immediately. Usually passed to physxWalkObjects()
//EXTERN_C void physxRender(void* actor);

// physxPrintActors() - prints a list of actors in the scene to the specified
//		buffer. Used to print gcon physx list. Returns 0 on success and -1
//		on failure.
EXTERN_C int physxPrintActors(char* buf, int start, int numToPrint);

// This function caps an object's velocity, simulating terminal
//		velocity. Set gTerminalVelocity to the capping value.
//		Returns 0 on success, -1 on failure.
EXTERN_C void physxCapVelocity(void* actor, float maxVel);

// physxDeleteObject() - removes the actor from the simulation. Usually either
//		called by physxRemove() or passed to physxWalkObjects to clear everything.
// will also purge actor from internal hash tables and lists...
EXTERN_C void physxDeleteObject(void* actor);
EXTERN_C void physxRemoveFromList(uns32 OID);
// same thing but for joint objs
EXTERN_C void physxDeleteJoint(void* joint);


// physxDataOutput() - updates object positions in the game engine based on the
//		latest positions in the physx engine. Called by physxLoop() after simulating.
EXTERN_C void physxDataOutput(void* actor);

// physxDataInput() - updates all skelcol objects in the physx engine based on the
//		latest positions in the game engine using bindMats. Called by physxLoop()
//		just before simulating.
EXTERN_C void physxDataInput(void* actor);

// getRotXYZ() - determines the X, Y, and Z angle rotations of the 3x3 matrix pointed to
//		by "float* matrix", and puts the angles into "angles[3]"
EXTERN_C void getRotXYZ(float *matrix, float *angles);

// physxSetContinuousCollisionCallback() - switches the object to continuous
//		collision callback mode. The callback will be called every frame that an
//		object is colliding with another.
EXTERN_C void physxSetContinuousCollisionCallback(Obj* obj);

// physxCollisionCallbackTest() - used to test collision callbacks.
//		simply prints out the OIDs of the colliding objects and the
//		point of collision.
EXTERN_C void physxCollisionCallbackTest(void* obj1, void* obj2, void* collision);

EXTERN_C void physxGrassTriggerFunc(void* obj1, void* obj2, void* collision);

// phsyxGunFunc() - used to test physx objects as projectiles.
//		Shoots a "ball" out of the camera in the direction the gun was pointed.
//		The ball disappears after a certain amount of time.
//EXTERN_C int PhysxGunFunc(int op,CabGun *gun,void *d2,void *d3);

// physxSparkCollisionCallback() - creates "sparks" (currently just default spheres)
//		at the point of collision that disappear after a certain amount of time.
//EXTERN_C void physxSparkCollisionCallback(void* obj1, void* obj2, void* collision);

// physxSkelAddPart() - adds an individual skeleton part as a "kinematic" dynamic object.
//		(see Physx Documentation for details on "kinematic" dynamic objects).
//		Called by physxAddSkelObj()
EXTERN_C int physxSkelAddPart(Obj* obj,
							  int physxState,	// see physxState enum
							  float density);	// only used for dynamic

// physxGetAngleFromAxis() - determines what the angle is (IN DEGREES) between the 
//		object's current rotation and a given axis (see axis enum in this file). 
//		Returns FLOAT_NAN if the object is not found.
EXTERN_C float physxGetAngleFromAxis(Obj* obj, int axis);

struct PhysxData;
// physxDequeueAdd() - will add any objects waiting in the queue, if there's room
//		within the current simulation.
EXTERN_C void physxDequeueAdd();

// physxCheckJointList() - determines if any joints that are waiting to be created
//		can be created now, or can't be created at all
EXTERN_C void physxCheckJointList();

// physxCollision - structure that is passed to a collision callback function.
//		To add a collision callback func to an object, use
//		AddPhysCollisionCallbackFunc() in obj.h/obj.c
typedef struct{
	float pointVec[3];
	float normVec[3];
	float contactForce;
	float separation;
	float obj1vel[3];
	float obj2vel[3];
	int numPoints;
} physxCollision;

typedef struct {
	char parent[32];
	char child[32];
	int boneParent, boneChild;
	float pos[3];
} physxActorObjLinkType;

enum physxStates {
	PHYSXSTATE_OFF = 0,
	PHYSXSTATE_STATIC,
	PHYSXSTATE_DYNAMIC,
	PHYSXSTATE_SKELPART,
	PHYSXSTATE_SIMPLE,

	NUM_PHYSXSTATES
};

enum physxPriority {
	PHYSX_PRIORITY_LOW = 5,
	PHYSX_PRIORITY_MID = 50,
	PHYSX_PRIORITY_HIGH = 100
};

enum physxJoints {
	PHYSX_JOINT_FIXED,
	PHYSX_JOINT_ROPE
};

enum{
	X_AXIS,
	Y_AXIS,
	Z_AXIS,
	NEG_X_AXIS,
	NEG_Y_AXIS,
	NEG_Z_AXIS
};

enum{
	PHYSX_TRIGGER_ONENTER,
	PHYSX_TRIGGER_ONLEAVE,
	PHYSX_TRIGGER_ONSTAY
};


#endif
