// collision.h
// Copyright 2010 Play Mechanix Inc. All Rights Reserved
//

#ifndef COLLISION_H
#define COLLISION_H

// collisionQueueUpSphere() - queue up a sphere for a sphere tree
// returns success/fail
EXTERN_C int collisionQueueUpSphere(float* center, float radius, PhysxExtendedData* extData);

// collisionCreateSphereTreeFromQueuedSpheres() - create a collision tree from the queued spheres
// returns col tree id or zero (for failure)
EXTERN_C unsigned int collisionCreateSphereTreeFromQueuedSpheres();

// collisionGetNumShapesInTree() - get the number of shapes in a given collision tree
// returns number of shapes in tree 'treeID'
EXTERN_C unsigned int collisionGetNumShapesInTree(unsigned int treeID);

// collisionTestTreeAgainstAABB() - test an axis-aligned bounding box against
//	an existing collision tree
// returns numCollisions, and populates 'oidsCollided' with up to maxToReturn oids that collided with the box
EXTERN_C int collisionTestTreeAgainstAABB(unsigned int treeID, float* center, float* extents, PhysxExtendedData** collidingObjs, int maxToReturn);

// collisionClearAll() - removes all collision info from system
//	(useful for resetting scene)
EXTERN_C void collisionClearAll();

// collisionIsValidTree() - check to see if this treeID is still valid
EXTERN_C int collisionIsValidTree(unsigned int treeID);

// STAND-ALONE COLLISION FUNCTIONS
EXTERN_C int collisionRayVSBox(float* rayOrigin, float* rayDirection, float* boxMins, float* boxMaxs);

EXTERN_C List sphereUserData;
EXTERN_C void* collisionSphereAlloc(void);
EXTERN_C void collisionSphereFreeAll(void);
EXTERN_C void collisionFreeCurrentTreesMemory(void);

EXTERN_C void rtColl_ClearCols();
EXTERN_C void Game_CollisionClearAll();

#endif
