#ifndef COREINPUT_H
#define COREINPUT_H
// core_input.h
// Copyright 1998-2010 Play Mechanix Inc. All Rights Reserved
//

#include "core_input_enums.h"



#define INP_MAX_MOUSE_BUTTONS 8

#define INP_USE_GUN_SMOOTHING	0x0001

typedef struct MouseInput
{
		io_input_analog x;
		io_input_analog y;

		io_input_digital buttons[INP_MAX_MOUSE_BUTTONS];
		io_input_digital wheel_up;
		io_input_digital wheel_down;

} MouseInput;


typedef struct KeyboardInput
{
	char buffer[256];
	io_input_digital keys[NUM_INP_KEYS];

} KeyboardInput;

#define INP_MAX_GUN_TRIGGERS 8
typedef struct GunInput
{
		io_input_analog x;
		io_input_analog y;
		io_input_digital triggers[INP_MAX_GUN_TRIGGERS];
} GunInput;

typedef struct PaddleInput
{
		io_input_analog x;
		io_input_analog tilt;
} PaddleInput;

#define INP_MAX_JOY_BUTTONS 32
typedef struct JoystickInput
{
		io_input_analog x;
		io_input_analog y;
		io_input_analog z;
		io_input_analog rx;
		io_input_analog ry;
		io_input_analog rz;

		io_input_digital buttons[INP_MAX_JOY_BUTTONS];

} JoystickInput;

#define INP_MAX_COINS 4
#define INP_MAX_STARTS 4
typedef struct CabinetInput
{
	io_input_digital coins[INP_MAX_COINS];

	io_input_digital diag;
	io_input_digital service;
	io_input_digital volup;
	io_input_digital voldwn;

	io_input_digital start[INP_MAX_STARTS];

} CabinetInput;

typedef struct MenuInput
{
	io_input_digital menu_up;
	io_input_digital menu_down;
	io_input_digital menu_select;
	io_input_digital menu_cancel;

	io_input_analog hover_x[2];
	io_input_analog hover_y[2];
	io_input_digital analog_select;
	io_input_digital analog_select_p[2];
	io_input_digital analog_cancel;
} MenuInput;

#define INP_MAX_GUNS 4
typedef struct InputRecord
{
	MouseInput mouse;
	KeyboardInput keyboard;
	GunInput guns[INP_MAX_GUNS];
	CabinetInput cab;
	MenuInput menu;

	io_input_digital quit;

} InputRecord;

typedef struct InputMap
{
	int src;
	int dst;
} InputMap;

InputRecord *InputGet();
io_input_joystick *InputGetJoystick(int joystickNumber);

int InputSetup(int IOLibFlags, int inpFlags);
int InputInit();
int InputFinal();
int InputReset(int force_rm_cbs);
int InputLoop();

io_input_digital *InputGetDigitalInput( int input );
io_input_analog *InputGetAnalogInput( int input );
int InputGetPressed( int input );
int InputGetHeld( int input );
int InputGetReleased( int input );
float InputGetPos( int input );


int InputAddMap( int src, int dst );
int InputDelMap( int src, int dst );
int InputUnMapSrc( int src );
int InputUnMapDst( int src );

#define INPCB_FLAGS_SURVIVE_WIPEOUT  0x00000001 // dflt is to remove all input callbacks on wipeout

typedef struct InputCallbackRecord
{
	uns32 id;
	uns32 flags;
	core_input_enum input;
	int priority;
	uns32 states;
	void *data;
	int (*func)( int, uns32, io_input_analog *, io_input_digital *, void * );

} InputCallbackRecord;

uns32 InputAddCallback( core_input_enum input, int priority, uns32 states, uns32 flags, void *data, int (*func)( int, uns32, io_input_analog *, io_input_digital *, void * ) );
InputCallbackRecord *InputGetCallback( uns32 id ); // use with care!
int InputDelCallback( uns32 id );
int InputDelAllCallbacks( int input );

typedef void (*InputAutoplayUpdate_Func)();

InputAutoplayUpdate_Func InputGetAutoplayUpdateCB();
void InputSetAutoplayUpdateCB(InputAutoplayUpdate_Func func);

extern int gCoreInputUseSPF; // motion platform debug sensor

extern void GunVibrateSetup();

#endif // ndef COREINPUT_H
