// core_statemachines_datatypes.h
// Play Mechanix - Video Game System
// Copyright (c) 2010 Play Mechanix Inc. All Rights Reserved
//
// data types for core use

// NOTES:
// only simple defines are supported by RTDataParser
// only local defines are supported by RTDataParser 
// only 'typdef struct's are supported by RTDataParser
// only single dimension arrays are supported
//
// NOTE:
// use core_statemachine_datatypes.h for structs related to state machines


#ifndef CORERTDTDATATYPES_H
#define CORERTDTDATATYPES_H

#pragma pack(4)

// stuff goes here

typedef struct WaterWaveParams
{
	float rot[3];
	float scale[3];
	float speed[3];
	float pos[3];
} WaterWaveParams;

typedef struct WaterParams
{
	char  name[24];
	float uNormalMapAmp;
	float uDistortionAmp;
	float uVertAmp;
	float uMinReflectivity;
	float uReflectionScale;
	float uReflectionDistortion;
	float uVertSpeed[2];
	float uVertFreq[2];

	WaterWaveParams wave[2];
	uns32 LastUpdate;

} WaterParams;



typedef struct _WindParams
{
	uns32 flags;
	char  name[32];
	float dir; // angle in rads
	float dirVec[3]; // sin(dir), 0, cos(dir)
	float ndirVec[3]; // -1 *( sin(dir), 0, cos(dir) )
	float speed;
	float strength;
	float noiseCoordMult;
	float tangentialRatio;
	float tangentialCoordMult;
	float currentOffset[3];
	uns32 simResolution;  // how many frames between re-sampling noise texture 
	uns32 simInterpolation; // linear or cos interp between frames

} WindParams;

#define WIND_NUM_PRESETS 64

typedef struct _WindParamList
{
	WindParams list[WIND_NUM_PRESETS];
} WindParamList;


typedef struct FurParams
{
	char name[32];
	int maxLayers;
	float maxDistance;
	float len;
	float tilt;
	float angle;
	float dir[3];
	float baseVal;
	float tipVal;
	float density;
	float specShift;

} FurParams;


typedef struct _CarParams
{
	char name[32];
	float color1[3];
	float color2[3];
	float colorFresnelPow;
	float fresnelBias;
	float fresnelPow;
	float specIntensity;
	float specPow;

} CarParams;


#define LOCALIZATION_NAME_LEN		64
#define LOCALIZATION_MAX_STR_LEN	256

typedef struct _textParams
{
	char		identifier[LOCALIZATION_MAX_STR_LEN];
	wchar_t		string[LOCALIZATION_MAX_STR_LEN];
	char		font[32];
	char		justifyHorizontal;
	char		justifyVertical;
	float		scale[2];
	char		group[32];
	char		textCallback_name[LOCALIZATION_MAX_STR_LEN]; // Optional text callback name so that callback can be specified in localization file vs. being hardcoded
} TextParams;

typedef struct _addressTabOrder
{
	int			addresseeName;
	int			address1;
	int			address2;
	int			city;
	int			state;
	int			province;
	int			zipCode;
	int			postalCode;
} AddressTabOrder;

#define POSTALCODE_MAX_LEN	16

typedef struct _addressFormat
{
	int			stateLen;
	int			provinceLen;
	int			postalCodeLen;
	int			postalNumbersOnly;
	int			phoneLen;
	int			cellPhone;
	char		cellFormat[4];
	int			monthFirst;
} AddressFormat;

typedef struct _locale
{
	char		name[LOCALIZATION_NAME_LEN];
	int			countryCode;
	int			onlineStatus;		// boolean
	char		language[LOCALIZATION_NAME_LEN];

	char		dbQueryServer[LOCALIZATION_NAME_LEN];
	char		dbQueryIP[LOCALIZATION_NAME_LEN];
	int			dbQueryPort;
	char		dbQueryCert[LOCALIZATION_MAX_STR_LEN];
	
	char		fileTransferServer[LOCALIZATION_NAME_LEN];
	char		fileTransferIP[LOCALIZATION_NAME_LEN];
	int			fileTransferPort;
	char		fileTransferCert[LOCALIZATION_MAX_STR_LEN];

	char		train[LOCALIZATION_NAME_LEN];	// should be ISO 3166-1 alpha-2 code (US, AU, CA, CN, UK...)
	int			payCardVerifySys;				// AVS for US...

	AddressTabOrder	addressTabOrder;
	AddressFormat	addressFormat;

	char		dateFormat[LOCALIZATION_NAME_LEN]; //"BigEndian", "MiddleEndian", or "LittleEndian"
} Locale;


// stuff for our travel system
typedef struct TravelAnim
{
	char name[32];
	float units_traveled_per_cycle;
	float cycle_length;
	float turn_radius;
	float speed; 
	float inv_tradius;
	float pitch;
} TravelAnim;

typedef struct SM_Travel_Params
{
	// visible from the tool
	float e_n_blendTime;
	float e_n_frames;

	float e_n_SpeedLookBack;
	float e_n_SpeedLookForward;

	float e_n_TurnLookBack;
	float e_n_TurnLookForward;

	int e_cb_PitchAndTurn;


	// hidden from tool (internal)
	uns32 i_track_ids[8];
	float i_apercent;


} SM_Travel_Params;

typedef struct TravelAnimCollection
{
	int count;
	TravelAnim tanims[256];
} TravelAnimCollection;

typedef struct _envColParams
{
	char name[32];
	float leanMagnitude;
	float wobbleTime;
	float wobbleMagFrac;
	float wobbleCycles;
} EnvColParams;

// Die Anims
typedef struct DieAnim
{
	char colbox[32];
	char anim[32];
	float distance;
	float units_traveled_per_cycle;
	float cycle_length;
	float anim_speed;
	float max_anim_speed;
	float max_units_traveled_per_cycle;
	float max_cycle_length;

	float stop_frames;
	float blend_frames;
	float speed;
	float max_speed;
	float speed_range;
	char side;

} DieAnim;

typedef struct DieAnimCollection
{
	int count;
	int no_stop;
	char dieFX[64];
	float FXVelMul;
	int  dieFX_do_rot;
	int  dieFX_do_scale;
	int  dieFX_use_objpos; // dflt is to use the cola pos
	int dieFX_use_bonepos;

	char dieFX_bonepos_name[64];
	float dieFX_bonepos_xoff;
	float dieFX_bonepos_yoff;
	float dieFX_bonepos_zoff;

	int  keep_alive; // keep object alive even if there is no anim for him
	DieAnim danims[128];
} DieAnimCollection;




// Adjustments
#define ADJ_NAME_SIZE 64

typedef struct AdjDef
{
	char name[ADJ_NAME_SIZE];
	int version;

	// flags
	uns8 default_on_version_change;		// Whether to default value if def version does not match saved value version
	uns8 online_reconcile;				// Whether to reconcile with online db
	uns8 hidden;						// Do not show in diag

	int adminOnly;						// 1 = Only can be adjusted by admin on website (e.g. CoinUp fees)

	int default_value;					// Factory default adjustment value
	int min;							// Minimum adjustment value
	int max;							// Maximum adjustment value
	int step_value;						// Step value (increment/decrement value)
	int divisor;						// Used for Get/SetByFloat commands

	int displayAsTime;					// 1 = Display adjustment in a time format
	int displayAsDate;					// 1 = Display adjustment in a date format
	int displayAsCurrency;				// 1 = Display adjustment in a currency format
	int displayAsPercentage;			// 1 = Display adjustment in a percentage format

	int displayPriority;				// Priority that is used to create a display order when displaying in diagnostics (used to sort the list)

	char lookup_table[ADJ_NAME_SIZE];	// Name of lookup table, if adjustment is to use one
	char sound_to_play[ADJ_NAME_SIZE];	// If this adjustment requires a sound to be played while adjusting, name of the sound
	char rumble_to_play[ADJ_NAME_SIZE];	// If this adjustment requires a rumble to be played while adjusting
	int sound_no_loop;

} AdjDef;


typedef struct AdjGroupDef
{
	char name[ADJ_NAME_SIZE];
	uns32 flags;

	char name_text_identifier[ADJ_NAME_SIZE];          // ADJ_GRP_name_NAME if not given
	char help_text_identifier[ADJ_NAME_SIZE];          // ADJ_GRP_name_HELP if not given

} AdjGroupDef;

typedef struct AdjValStore
{
	char name[ADJ_NAME_SIZE];
	int version;
	time_t time_set;
	int val;
} AdjValStore;


// Audits
#define AUD_NAME_SIZE 32

typedef struct AudDef
{
	char name[AUD_NAME_SIZE];
	char datatype[AUD_NAME_SIZE];
	int version;

	// flags
	int hidden; // do not show in diag


	char help_text_identifier[AUD_NAME_SIZE];          // AUD_name_HELP if not given

} AudDef;



// CashBox Audits

// TO DO - put enum (and typedef enum?) into RTDT
/*
typedef enum _cointype_in_e
{
	COINTYPE_IN_NONE = -1,
	COINTYPE_IN_SLOT0,			// starting with g5, starts at 0
	COINTYPE_IN_SLOT1,
	COINTYPE_IN_SLOT2,
	COINTYPE_IN_DBV,
	COINTYPE_IN_SVCCRED,
	COINTYPE_IN_BONUSCRED,
	COINTYPE_IN_PLAYER_XFER,
	COINTYPE_IN_PROMOCARD,
	COINTYPE_IN_GIFTCARD,
	COINTYPE_IN_CREDITCARD,
	NUM_COINTYPE_IN
} cointype_in_t;
*/

#define COINTYPE_IN_NONE			-1
#define COINTYPE_IN_SLOT0			0
#define COINTYPE_IN_SLOT1			1
#define COINTYPE_IN_SLOT2			2
#define COINTYPE_IN_DBV				3
#define COINTYPE_IN_SVCCRED			4
#define COINTYPE_IN_BONUSCRED		5
#define COINTYPE_IN_PLAYER_XFER		6
#define COINTYPE_IN_PROMOCARD		7
#define COINTYPE_IN_GIFTCARD		8
#define COINTYPE_IN_CREDITCARD		9
#define COINTYPE_IN_CCREFUND		10
#define NUM_COINTYPE_IN				11	// WARNING - if you change NUM_COINTYPE_IN, make sure you change NUM_ONLINE_COINTYPE_IN too

typedef struct CashBoxSlot
{
	float total;
	float totalSpent;		// in case some future audit func does reconcilation after an extended period (e.g. "grey" game)
	
	// doesn't get reset when coin audits are reset
	float unspent;

	int	coin_clicks;	// clicks for cash types, button presses for service credit, swipes for pay cards
} CashBoxSlot;

typedef struct CashBoxAudits
{
	// totals that don't get deleted when credits are reset
	float total;
	float totalSpent;

	// doesn't get reset when coin audits are reset
	float unspent;

	CashBoxSlot coin_type[NUM_COINTYPE_IN];

	// next 3 items are billing flag hacks, initially done for promo cards
	int lastTakeFullCash;
	int lastTakeFullType;
	int lastAltPayId;

	int lastCoinInType;
	int outOfSync;
} CashBoxAudits;


#define GAME_LOCATION_STR_SIZE	19
#define GAME_VENUE_STR_SIZE	19

typedef struct SystemAudit
{
	// total number of milliseconds the game has been running
	uns64 timeOn;

	int boots;
	int diagEntries;
	int maintReboots;
	int noIOBoardInits;
	int IODisconnects;
	int IOReconnects;

	int watchdogs;
	int exceptions;
	int traps;
	
	uns32 motionButtonStops;
	uns32 motionButtonResumes;
	uns32 motionButtonManualResumes;
	uns32 motionFanErrors;
	uns32 motionLowPSI;
	uns32 motionDisconnects;



	uns32 motionMatStops;
	
	char location[GAME_LOCATION_STR_SIZE];
	char venue[GAME_VENUE_STR_SIZE];

	time_t startTime;

} SystemAudit;

// --------------------------------------------------------------------------
// Menu system rtdt
// --------------------------------------------------------------------------

// too bad we aren't using C++... 
// The base member must be first in any derived diag button type!
typedef struct _MenuScrollingMenuItemBase_s
{
	int prev_hover;						// were we highlighted last frame?
	int menu_level_number;				// used to determine if page is active
	int focus_number;					// where in the taborder are we?
	void *hover_func;					// should be 'void (*hover_func)(void *me)' but not supported yet by RTDT
} MenuScrollingMenuItemBase;

typedef struct _MenuScrollingMenuItemData_s
{
	MenuScrollingMenuItemBase menu_item_base;	// base diag button members, must be first
	void *menu_item_ptr;							// it was just easier than pulling diag_item_t here.
} MenuScrollingMenuItemData;

// --------------------------------------------------------------------------

// keyboard RTDT

typedef struct _keyboard_setup_data_s
{
	char	keyboard_layout[LOCALIZATION_NAME_LEN];
	uns32	keyboard_flags;
	wchar_t	*inbuf_ptr;				// if non null, must point to a buffer less than max_len
	int		max_len;				// max input length, including ending null
	void	*validate_func;
	void	*validate_params;
	void    *exit_func;            // run on keyboard exit
} keyboard_setup_data_t;

typedef struct _KeyboardButton_s
{
	char	sound[LOCALIZATION_NAME_LEN];		// name of sound file
	float	hover_xscale;						// x scale for hover grow
	float	hover_yscale;						// y scale for hover grow
} KeyboardButton;


#define AUD_GUN0_CALIBRATION  "gun0Calibration"
#define AUD_GUN1_CALIBRATION  "gun1Calibration"
#define AUD_GUN2_CALIBRATION  "gun2Calibration"
#define AUD_GUN3_CALIBRATION  "gun3Calibration"
typedef struct GunCalibrationAudit
{
	float XgunPos[2];			
	float YgunPos[2];			
	int   loadedFromDefault;	// Designates whether the gun cal audit was loaded from its default (i.e. this gets set to 1 in the default, and 0 elsewhere)
} GunCalibrationAudit;


#pragma pack()

#endif // CORERTDTDATATYPES_H
