// core_scene.h
// Play Mechanix - Video Game System
// Copyright (c) 2009 Play Mechanix Inc. All Rights Reserved
//
#ifndef CORE_SCENE_H
#define CORE_SCENE_H

#include "g5.h"

// save all fields in particle files
// or just those that have changed from the dflt (0)
//#define SCENE_FULL_PARTICLE_SAVE 1

#define SCENE_PRECREATE_ACTORS 0
#define SCENE_PRECREATE_ACTORS_COUNT 50 // number of actors to pre-create per frame (0=create all on first frame)
#define SCENE_PRECREATE_ACTORS_DURING_LOAD 1 // do pre-creates as part of the load process (otherwise they are done during game starting from when scene play is called)



// states our scenes step through on their way to being done...
enum
{
	SCENE_STATE_INIT,
	SCENE_STATE_LOADING,
	SCENE_STATE_LOADED,
	SCENE_STATE_PLAYING,
	SCENE_STATE_ENDING, // in the process of ending the scene...
	SCENE_STATE_DONE,

};

// define scene types
// NOTE:
// all these scene types contain some gui data for them
// originally all the scene stuff lived the tool
// moved it out so that scenes could exists sans the tool
// and to have a cleaner separation of parts
// but it is still pretty handy for the tool to keep
// track of gui stuff right in these structures
// it probly violates some CS theorums
// but really doing any other way is just making work
// for ourselves...
typedef struct {

	char name[32];			// - scene name
	uns32 res_mark;			// - bit to mark loaded/used resources with...
	uns32 dispatchers_group_oid;
	uns32 cam_group_oids[NUM_OBJCAMS];  // where to link objs for each camera

	List resources;			// - list of resources to load
	List actors;			// - list of scene actors
	List actor_groups;		// - list of actor groups (only for organization purposes)
	List dispatchers;		// - list of dispatchers (programs that wait on triggers to run)
	List triggers;			// - list of scene triggers (added only for organization purposes)
	List programList; // list of programs required by this scene...
	List objects;			// - a list of objects created by the scene (used for tracking obj's by actor name, resource, etc...)

/*  moved to particle collection
	List partEmitters;		// - list of particle emitters
	List fxGroups;			// - list of effects groups (groups of particle emitters and sound calls)

	HashTable partEmittersHash; // - hash of particle emitters by name, for quick finds (points to nodes in partEmitters list)
	HashTable fxGroupsHash;     // - hash of particle fxgroups by name, for quick finds (points to nodes in fxGroups list)
*/
	int state;
	uns64 load_time; // how long in ms did the most recent scene load take?? (hold load start time when currently loading...)
	float load_progress;    
	int play_on_load;		// - run the scene as soon as we are finished loading!

#if (SCENE_PRECREATE_ACTORS==1)
	// things used for pre-creating actors
	ListNode *CurPreCreateActor; // pointer to actors listnode, use assumes actors list does not change during scene...
	HashTable PreCreatedActors; // keys are actor names, nodes are obj pointers that are waiting to be linked onto obj list
	HashTable CreatedActors; // keep track of which actors have been created...
#endif

//	List progResList;		// - store a pointer to all the program resources

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	// these are gui items the tool tracks for the scene...
	char group_name[32];	// - what group is displayed in the actor window
	int gui_id_parent;		// - gui id of the scene edit window
	int gui_id_window;		// - gui id of the resource list window
	int gui_id_proglabel;	// - gui id of the text label (program name)
	int gui_id_loadresbtn;	// - gui id of the "load resources" button
	int gui_id_runscenebtn;	// - gui id of the "run scene" button
//	int gui_id_emitterbtn;	// - "restart selected" particle emitters
//	int gui_id_emitter_sort;
//	int gui_id_fxgroups_sort;
	int gui_id_locatetriggers;	// - "locate triggers" button
	int gui_id_actorgroup[2];// - gui id of the "actor group" text input box (label, then textbox)
	int gui_id_programdir;	// - gui id of the "program directory" text input
	int gui_id_tabs[5];		// - tab buttons for the edit window (resources, actors, dispatchers, triggers, settings)
	int tab_current;		// - which is our active tab?
	// this one is used to set off the 'SCENE_START_TOOL' trigger
	int do_scene_start_tool_trigger;
	// these hash tables are used by the tool gui to track what emitter/fxgroup groups to show...
//	HashTable partEmittersShowGroup;
//	HashTable fxGroupsShowGroup;
//	int dirtyFxGroups;		// - used to tell the fx group window to refresh
//	int dirtyEmitters;		// - used to tell the fx group window to refresh
#endif // NO_TOOL
} SceneType;

typedef struct {
	char filename[64];
	int isLoaded;
	int isAdded;
	int isAppended;
	int loadRequestID;
	int camera; // which camera to add the res under
	int isScene; // is this a scene or g5 file to load?

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	int gui_id_text;
	int gui_id_killbox;
	int gui_id_addbtn;
#endif // NO_TOOL

} SceneResource;


enum {
	SCENE_ACTOR_OBJTYPE_DISPGROUP = 0,
	SCENE_ACTOR_OBJTYPE_DISP3,
	SCENE_ACTOR_OBJTYPE_DISP2,
	SCENE_ACTOR_OBJTYPE_LIGHT,
	SCENE_ACTOR_OBJTYPE_FXGROUP,
	SCENE_ACTOR_OBJTYPE_OBJGROUP,
	SCENE_ACTOR_OBJTYPE_TEXT,
	SCENE_ACTOR_OBJTYPE_LAYOUT,
	SCENE_ACTOR_OBJTYPE_NUM,
};

enum {
	SCENE_ACTOR_VIOLENCEMODE_ALL,
	SCENE_ACTOR_VIOLENCEMODE_LOW_VIOLENCE_ONLY, // NOBLOOD
	SCENE_ACTOR_VIOLENCEMODE_HIGH_VIOLENCE_ONLY, // DEFAULT
};

enum {
	SCENE_ACTOR_COLMODE_3D,
	SCENE_ACTOR_COLMODE_2D,
	SCENE_ACTOR_COLMODE_OFF,
};


typedef struct {	
	char name[32];
	char trigger[TRIGGER_NAME_SIZE];
	char obj_name[PM_NAME_SIZE];
	char col_name[PM_NAME_SIZE];
	char prog_name[PM_NAME_SIZE];
	char shader_name[PM_NAME_SIZE];
	char group_name[PM_NAME_SIZE];
	int obj_type;
	int lightgroup;
	
	int fog;
	int shad_recieve;
	int shad_cast;
	int forcedg;
	int nophysx;
	int noanisfx;
	int playermode;		// 0 = 1P & 2P, 1 = 1P only, 2 = 2P only
	int violencemode;		
	int camera; // which camera to create the object under
	int colmode; // 3d, 2d or off
	int repeat_create; // create once or every time create trigger is called?
	void *scene;			// <- a link back to the scene to whom this actor belongs

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	int hide; // note: as another concession to our gui, we save whether the actor is hidden in the gui or not...
	int gui_id_editwin;
	int gui_id_playermode[3];
	int gui_id_violencemode[3];
	int gui_id_colmode[3];
	int gui_id_camera[5];
	int gui_id_name;
	int gui_id_trigger;
	int gui_id_objname;
	int gui_id_colname;
	int gui_id_progname;
	int gui_id_shadername;
	int gui_id_lightgroup;
	int gui_id_groupname;
	int gui_id_crect;
	int gui_id_fog;
	int gui_id_shad_recieve, gui_id_shad_cast;
	int gui_id_forcedg;
	int gui_id_nophysx;
	int gui_id_noanisfx;
	int gui_id_repeat_create;
	int index;				// <- the index that this actor occupies in the scene actor list
	int helper_pos[2];		// <- store away the xy coords of the edit window, just in case we
							//    need to recreate the window
#endif // NO_TOOL
} SceneActor;


typedef struct {
	char name[32];
//	char resname[32];
	uns32 id;  // oid of the object
//	int actorindex;			// - if this object is an actor, what is the index in the scene actor list?

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	int mouse_axis[3];		// - is the mouse currently over any axis?
	float mx, my;			// - previous mouse pos
	crd pos2d[12];			// - 2d coordinates of the 4 points describing the axis
							//   (origin, x, y, z)
							//   generated everytime the object is aux rendered

	int notWipeable;		// - can this object be "wiped" by the tool via F3?
	int draggable;			// - can this object be "dragged" around the scene?
#endif //NO_TOOL

} SceneObject;

typedef struct {
	char trigger[TRIGGER_NAME_SIZE];
	char program[32];
	int repeat_create;
	void *scene;			// <- a link back to the scene to whom this actor belongs

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	int gui_id_trigger;
	int gui_id_progname;
	int gui_id_run_once_btn;
#endif //NO_TOOL

} SceneDispatcher;

typedef struct {
	char name[32];
	// BEGIN GUI STUFF
} SceneTrigger;

/*
enum {
	SCENE_FXGROUP_EMITTER = 0,
	SCENE_DATA_FXGROUP_SOUND,
};

typedef struct {
	char emitter[64];
	char soundname[64];
	int soundid;
	int delay;
	float scale;
	float pos[3];
	float pvar[3];
	float vel[3];
	float vvar[3];
	float acc[3];
	float rvel[3];
	int forceveldir;

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	int show_details;
	int gui_id_show_details;
	int gui_id_delay;
	int gui_id_scale;
	int gui_id_pos[3];
	int gui_id_pvar[3];
	int gui_id_vel[3];
	int gui_id_vvar[3];
	int gui_id_acc[3];
	int gui_id_rvel[3];
	int gui_id_forceveldir[NUM_PARTICLE_FORCEVELDIR_MODES];
#endif // NO_TOOL

} SceneFXGroupList;

typedef struct {
	char name[32];
	char group[32];
	List effectList;
	void *scene;

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	int gui_id_fxwindow;
	int gui_id_window;
	int gui_id_name;
	int gui_id_group;
	int gui_id_scale;
#endif // NO_TOOL
} SceneFXGroup;

typedef struct {
	
	char name[32];
	char trigger[TRIGGER_NAME_SIZE];
	char offTrigger[TRIGGER_NAME_SIZE];
	ParticleEmitter data;
	void *scene;

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	int selected;
	int gui_id_window;
	int gui_id_editwindow;
	int gui_id_tabs[6];
	int tab_current;
	int gui_id_name;
	int gui_id_group;
	int gui_id_res;
	int gui_id_nres;
	int gui_id_shader;
	int gui_id_trigger, gui_id_trigger_off;
	int gui_id_lightgroup;
	int gui_id_selected;
	int gui_id_pos[3];
	int gui_id_posvar[3];

	int gui_id_vel[3];
	int gui_id_velvar[3];

	int gui_id_acc[3];
	int gui_id_accvar[3];

	int gui_id_color;
	int gui_id_colorb;

	int gui_id_size, gui_id_sizevar;
	int gui_id_growth, gui_id_growthvar, gui_id_growthend;
	int gui_id_rotvel, gui_id_rotvelvar;
	int gui_id_life, gui_id_lifevar;
	int gui_id_xsize, gui_id_ysize;

	int gui_id_alphain;
	int gui_id_alphaout;

	int gui_id_disp2_rot[3];
	int gui_id_disp2_rotvar[3];

	int gui_id_scale, gui_id_scaleAttack, gui_id_scaleDecay;

	int gui_id_imgmode[2];			// - buttons: 0 = static, 1 = animating
	int gui_id_animating_kill[2];	// - buttons: 0 = loop, 1 = kill on done
	int gui_id_looping[2];			// - buttons: 0 = looping, 1 = non-looping
	int gui_id_particletype[4];		// - buttons: 0 = standard, 1 = yscale, 2 = streak, 3 = disp2
	int gui_id_orientation[PEMITTER_ORIENTATION_NUM]; // - buttons: 0 = billboard, 1 = y axis, 2 = yz billboard, 3 = decal
	int gui_id_flipx;
	int gui_id_flipy;

	int gui_id_dispersion[NUM_PARTICLE_DISPERSION_MODES];
	int gui_id_forceveldir[NUM_PARTICLE_FORCEVELDIR_MODES];
	int gui_id_usecameraivm;

	int gui_id_numframes;
	int gui_id_framedim[2];
	int gui_id_frametime;
	int gui_id_frames;

	int gui_id_eject_over[NUM_PARTICLE_EJECT_MODES];
	int gui_id_rate;
	int gui_id_limit;

	int gui_id_total;
	int gui_id_streakLen;

	int gui_id_randtheta;
	int gui_id_randthetavelsign;

	int gui_id_gcol[2];
	int gui_id_shaderfx[NUM_PARTICLE_SHADERFXS];

	int gui_id_blend[NUM_PARTICLE_BLEND_MODES];
	int gui_id_blendColor[NUM_PARTICLE_COLOR_BLEND_MODES];

	int gui_id_vm_mode[NUM_PARTICLE_VMMODES];

	uns32 ope_oid;
#endif //  NO_TOOL

} SceneParticleEmitter;
*/


/*
int CoreSceneInit();
int CoreSceneReset();
int CoreSceneFinal();
int CoreSceneLoop();
*/

int SceneInit( SceneType *scene );
int SceneDeInit( SceneType *scene );
int SceneCreate( SceneType **scene ); // wrapper for init that will malloc for scene
int SceneDestroy( SceneType *scene ); // wrapper for DeInit that will free scene ptr



int SceneLoadSceneFile( SceneType *scene, char *name );
int SceneLoadResourcesStart( SceneType *scene );
int SceneLoadResourcesUpdate( SceneType *scene, int progs_to_load );
int SceneUnLoadResources( SceneType *scene );
int ScenePlay( SceneType *scene, int do_scene_start_trigger, char* custom_trigger);
int SceneWipe( SceneType *scene ); // this will destroy all but 'non-wipeable' scene objects (backgrounds)
int SceneEnd( SceneType *scene, int do_scene_end_trigger ); // this nukes all scene objects and programs...
int SceneGetState( SceneType *scene );

#if(SCENE_PRECREATE_ACTORS==1)
int ScenePreCreateActorsLoop( SceneType *scene, int count ); // pre-creates up to count actor objects...
int ScenePreCreateActorsFree( SceneType *scene ); // frees any pre-created actors...
#endif

int SceneSaveSceneFile( SceneType *scene );
//int SceneSavePEmitters( SceneType *scene );

int SceneRebuildHashTables(SceneType *scene);

// scene keeps an internal list of objects that are associated with the scene, this allows one to add objs to that list...
int SceneAddSceneObj(SceneType *scene, uns32 oid, char *name, SceneObject **sret);

int SceneResourceAdd(SceneType *scene, SceneResource *res);
int SceneResourceRemove(SceneType *scene, SceneResource *res);

void SceneGetActorByName(SceneType *scene, char *name, uns32 *oid);
void SceneActorsCountByRes(SceneType *scene, char *resname, int *count, int kill, char *eaction );

int SceneCreateActor( SceneType *scene, char *actor, Obj **o, Obj* overrideObj );
int SceneKillActor( SceneType *scene, char *actor );
int SceneKillActorsByRes( SceneType *scene, char *resname );
int SceneGetActorsInGroup( SceneType *scene, char *group_name, Obj **objs, int maxObjs );

// these two funcs are here so that one can stash a ptr
// to the currently running scene
// onus is on the application to manage this ptr
// meaning the above scene functions do no set/clear this variable
// changed so current scene is set in SceneManagerLoad as well
void SceneSetCurScene( SceneType *scene );
SceneType *SceneGetCurScene();

int scene_add_pemitters_fromfile(SceneType *scene, char *filename );


int scene_actorlist_callback(Trigger *t, TriggerEvent *te); // from scene_core.c
int scene_dispatcherlist_callback(Trigger *t, TriggerEvent *te); // from scene_core.c

N2 *scene_actor_get_link_point( SceneType *scene, SceneActor *actor );

/*
void t2cb_scene_edit_loadprograms(T2_DATA_SCENE_TYPE *scene); // initiates scene program loads, use t2cb_scene_edit_loadprograms_loop to complete load...
int t2cb_scene_edit_loadprograms_loop(T2_DATA_SCENE_TYPE *scene, int count); // count is number of progs to load right now, spin on this guy to load all scene programs...
*/

Obj* ObjMakeActor(Res* res, char* pbuf, Obj* overrideObj);
void ObjUseMa(Res* res, char* pbuf);
ObjDispRes* ObjAddProp(ObjDispRes* odr, char* bone, char* maName, char* objName, char* shader, int hideWithPrime);


typedef void (*GameRenderDebugInfo_Func)( );
extern GameRenderDebugInfo_Func gGameRenderDebugInfoFunc;

typedef void (*GameRenderDebugCol_Func)( );
extern GameRenderDebugCol_Func gGameRenderDebugColFunc;

typedef int (*GameMakeActor_Callback)(Res*, char*, Obj*);
extern GameMakeActor_Callback gGameMakeActor;
void SetCallback_GameMakeActor(GameMakeActor_Callback func);

typedef int (*GameScenePlay_Callback)(SceneType*);
extern GameScenePlay_Callback gGameScenePlay;
void SetCallback_GameScenePlay(GameScenePlay_Callback func);

typedef void (*UsemaCustomCode_Callback)(Res*, ObjDispRes*);
extern UsemaCustomCode_Callback gUsemaCustomCode;

typedef void (*UsemaShot_Callback)( ObjDispRes* odr, ObjColAct* cola );
extern UsemaShot_Callback gUsemaShotFunc;

typedef int (*PickupReadType_Func)(char*);
extern PickupReadType_Func gPickupReadTypeFunc;

typedef void (*PickupShot_Func)(int, int, ObjDispRes* odr);
extern PickupShot_Func gPickupShotFunc;

typedef void (*PickupCreate_Func)(int);
extern PickupCreate_Func gPickupCreateFunc;

typedef int (*BonusReadType_Func)(char*);
extern BonusReadType_Func gBonusReadTypeFunc;

typedef void (*BonusShot_Func)(int, int, ObjDispRes*);
extern BonusShot_Func gBonusShotFunc;

typedef void (*DestructShot_Func)( ObjDispRes*, ObjColAct* cola );
extern DestructShot_Func gDestructShotFunc;

typedef void (*SetGeneralNum_Func)( int num );
extern SetGeneralNum_Func gSetGeneralNumFunc;

typedef void (*FlashlightUpdate_Func)( ObjLightRes* olr, float* scaleOut );
extern FlashlightUpdate_Func gFlashlightUpdateFunc;

typedef void (*MakeFlyer_Func)( Res* res, FlyerElements* flyer, char* type );
extern MakeFlyer_Func gMakeFlyerFunc;


extern int gHideHumanDeaths;

#endif

