// core_statemachines.h
// Play Mechanix - Video Game System
// Copyright (c) 2008 Play Mechanix Inc. All Rights Reserved
//
// init routintes and statemachines for programmable statemachine engine 
#ifndef CORESTATEMACHINES_H
#define CORESTATEMACHINES_H


#include "g5.h"



typedef struct SM_TravelFunc_Params
{
	int state_updates;
	uns32 state_elapsed_time;
	uns32 frame_elapsed_time;
	

	float blendTime;
	float frames;

	uns32 track_ids[8];
	float apercent;

	ObjDispRes* prime;
	ObjTrackAnim* otan;

} SM_TravelFunc_Params;


void HideByGroup(char* name);
void ShowByGroup(char* name);

int CoreStateMachinesInit();

int SM_AddGeneralPurposeStates( PSM_StateMachine *statemachine );
int SM_AddCoreSceneStates( PSM_StateMachine *statemachine );
int SM_AddStateMachineRes( PSM_StateMachine *statemachine );


void UpdateObjCommonSMParams( ObjExeContext *oec );

int ResLoadSMProgram( Res **resp, char *program_name, char *statemachine_name, uns32 mark );
int ResLoadSMProgramPath( Res **resp, char *program_name, char *path, char *statemachine_name, uns32 mark, N2 *link );
Res *ResFindOrLoadSMProgramRes( char *program_name, char *statemachine_name, uns32 mark );
// loads all the sm programs in a given folder
int ResLoadSMPrograms( char *subfolder, uns32 mark, N2 *link );


EXTERN_C int ObjAddExeContext( ObjExeContext **oec, Obj *parent, char *program_name, char *statemachine_name, void *statemachine_params );
int ObjExeContextRegisterEventActions( ObjExeContext *oec );
int ObjExeContextUnRegisterEventActions( ObjExeContext *oec );

void SetEventActionState(PSM_ExeContext *context,char *eventName, char *stateName );


int ObjDoEventActions( Obj *obj, char *event_name, void *event_data, int recursive ); // handles event for all ExeContext kids, if recursive will do all kids kids kids kids...
int SM_RunProgramSub( PSM_ExeContext *context, char *progName );
int SM_RunProgramWithParamPtrSub( PSM_ExeContext *context, char *progName, int free, int wait  );
void SMKillChildPrograms(PSM_ExeContext *context, Obj *prime);
void SMKillAllPrograms(PSM_ExeContext *context, Obj *prime);
void MountedGunsOnSub(int gunType);
void MountedGunsOffSub(void);




// General Purpose State Machine Functions
enum {
	SM_GPSTATES_MOVE	 = NUM_PSM_BUILTIN_STATES,
	SM_GPSTATES_MOVETO,
	SM_GPSTATES_HIDE,
	SM_GPSTATES_SHOW,
	SM_GPSTATES_SETOUTLINE,

	//	SM_GPSTATES_ROTATE,
	// SM_GPSTATES_ANISEQ,
	SM_GPSTATES_RUN_PROGRAM,
	SM_GPSTATES_RUN_FREE_PROGRAM,
	SM_GPSTATES_RUN_PROGRAM_WITH_PARAM_PTR,
	SM_GPSTATES_SWITCH_PROGRAM,

	SM_GPSTATES_SLEEP,
	SM_GPSTATES_SLEEP_RAW,
	SM_GPSTATES_SLEEP_RND,
	SM_GPSTATES_GOSUB_RND,
	SM_GPSTATES_GOTO_RND,
	SM_GPSTATES_SET_RECOVERY_POINT,
	SM_GPSTATES_GOTO_RECOVERY_POINT,
	SM_GPSTATES_GOTO_CB,
	SM_GPSTATES_BR_ONFLAG,
	SM_GPSTATES_BR_ON_TRIGGER,
	SM_GPSTATES_BR_ON_NOT_TRIGGER,
	SM_GPSTATES_BR_ON_ADJUSTMENT,
	SM_GPSTATES_BR_ON_SPLINEFRAME,
	SM_GPSTATES_SET_NAME,
	SM_GPSTATES_SEND_LOCAL_EVENT,
	SM_GPSTATES_SEND_ACTOR_LOCAL_EVENT,
	SM_GPSTATES_DROP_EACTION,
	SM_GPSTATES_SETTRIGGER,
	SM_GPSTATES_CLRTRIGGER,
	SM_GPSTATES_DELTRIGGER,
	SM_GPSTATES_SETTRIGGER_ON_TRIGGER,
	SM_GPSTATES_SETTRIGGER_ON_NOT_TRIGGER,
	SM_GPSTATES_CAM_TRIGGER,
	SM_GPSTATES_WAITFOR_TRIGGER,
	SM_GPSTATES_WAITFOR_ACTOR,
	SM_GPSTATES_WAITFOR_BR,
	SM_GPSTATES_WAITFOR_ACTOR_BR,
	SM_GPSTATES_WAITFOR_OBJ,
	SM_GPSTATES_WAITFOR_TRAVEL,
	SM_GPSTATES_CAMERA_SELECT,
	SM_GPSTATES_CAMERA,
	SM_GPSTATES_CAMERA_RESET,
	SM_GPSTATES_CAMERA_LOOP,
	SM_GPSTATES_CAMERA_SWATCH,
	SM_GPSTATES_CAMERA_SWATCH_ABS,
	SM_GPSTATES_CAMERA_HOLD,
	SM_GPSTATES_CAMERA_RUN,
	SM_GPSTATES_CAMERA_SPEED,
	SM_GPSTATES_CAMERA_SKIPTO,
	SM_GPSTATES_CAMERA_CONDITIONAL,
	SM_GPSTATES_CAMERA_SPLINE2,
	SM_GPSTATES_CAMERA_WATCH_ACT,
	SM_GPSTATES_CAMERA_LOOK_AT,
	SM_GPSTATES_CAMERA_SET_FOV,
	SM_GPSTATES_CAMERA_SHAKE,
	SM_GPSTATES_CAMERA_SHAKE_SIMPLE,
	SM_GPSTATES_GAME_SPEED,
	SM_GPSTATES_GUN_SPEED,


	SM_GPSTATES_SPLINE,
	SM_GPSTATES_SPLINE_SPEED,
	SM_GPSTATES_SPLINE_GCOL_PARAMS,
/*
	SM_GPSTATES_SPLINE_NAME,
	SM_GPSTATES_SPLINE_LOOP,
	SM_GPSTATES_SPLINE_PKILL,
	SM_GPSTATES_SPLINE_GCOL,
*/
	SM_GPSTATES_SPLINE_SKIP,
	SM_GPSTATES_SPLINE_SKIPTO,
	SM_GPSTATES_SPLINE_HOLD,
	SM_GPSTATES_SPLINE_RUN,
	SM_GPSTATES_SPLINE_STEP,
	SM_GPSTATES_SWATCH,
	SM_GPSTATES_SPLINE_TO_VEL,

	SM_GPSTATES_ANIMCHANNEL,
	SM_GPSTATES_ANIMCHANNEL_BLEND,
	SM_GPSTATES_ANIMCHANNEL_BLENDMODE,
	SM_GPSTATES_ANIMCHANNEL_ISOLATESUBSKEL,

	SM_GPSTATES_BAN,
//	SM_GPSTATES_BAN_TIMED,
	SM_GPSTATES_BAN_SPEED,
/*
	SM_GPSTATES_BAN_NAME,
	SM_GPSTATES_BAN_BLEND,
	SM_GPSTATES_BAN_LOOP,
*/
	SM_GPSTATES_BWATCH,
	SM_GPSTATES_BWATCH_TRAVEL,
	SM_GPSTATES_BAN_SKIPTO,
	SM_GPSTATES_BAN_SYNC,

	SM_GPSTATES_TRANS,
	SM_GPSTATES_TVEL,
	SM_GPSTATES_TACC,
	SM_GPSTATES_ROT,
	SM_GPSTATES_RVEL,
	SM_GPSTATES_RACC,
	SM_GPSTATES_SCALE,
	SM_GPSTATES_SVEL,
	SM_GPSTATES_SACC,

	SM_GPSTATES_TRANS_BOUNDS,
	SM_GPSTATES_TVEL_BOUNDS,
	SM_GPSTATES_ROT_BOUNDS,
	SM_GPSTATES_RVEL_BOUNDS,
	SM_GPSTATES_SCALE_BOUNDS,
	SM_GPSTATES_SVEL_BOUNDS,
	SM_GPSTATES_CLEAR_BOUNDS,

	SM_GPSTATES_TRANS_RAND,
	SM_GPSTATES_ROT_RAND,
	SM_GPSTATES_TVEL_RAND,
	SM_GPSTATES_RVEL_RAND,

	SM_GPSTATES_TRANS_WAIT,
	SM_GPSTATES_TVEL_WAIT,
	SM_GPSTATES_ROT_WAIT,
	SM_GPSTATES_RVEL_WAIT,
	SM_GPSTATES_SCALE_WAIT,

	SM_GPSTATES_TRANS_BR,
	SM_GPSTATES_TVEL_BR,
	SM_GPSTATES_ROT_BR,
	SM_GPSTATES_RVEL_BR,
	SM_GPSTATES_SCALE_BR,
	SM_GPSTATES_SCALE_FRAMES,


	SM_GPSTATES_BLEND,
	SM_GPSTATES_ZBUFFER,
	SM_GPSTATES_SWAPRES,
	SM_GPSTATES_SWAPSHADER,
	SM_GPSTATES_SWAPTEX,
	SM_GPSTATES_USESCREENGRAB,
	SM_GPSTATES_USEBLANKTEX,
	SM_GPSTATES_USEVERTCOLORS,
	SM_GPSTATES_USEBILLBOARDY,
	SM_GPSTATES_SETLIGHTGROUP,
	SM_GPSTATES_SETENVMAP,
	SM_GPSTATES_SETREFLECTIVITY,

	SM_GPSTATES_SETVSYNC,

	//SM_GPSTATES_COLOR_CHANGE,
	SM_GPSTATES_TIMED_COLOR_CHANGE,
	SM_GPSTATES_KILLPRIME,
	SM_GPSTATES_KILLPROG,
	SM_GPSTATES_KILLACTOR,
	SM_GPSTATES_KILL_BY_NAME,
	SM_GPSTATES_KILL_BY_RESNAME,
	SM_GPSTATES_ADVANCE,
	SM_GPSTATES_KILL_BY_GROUP,
	SM_GPSTATES_KILL_OBJ_GROUP,
	SM_GPSTATES_HIDE_BY_GROUP,
	SM_GPSTATES_SHOW_BY_GROUP,

	SM_GPSTATES_SOUNDPLAY,
	SM_GPSTATES_SOUNDPLAY_BGM,
	SM_GPSTATES_SOUNDPLAY_REP,
	SM_GPSTATES_SOUNDPLAY_VO,
	SM_GPSTATES_SOUNDRUMBLEPLAY,
	SM_GPSTATES_SOUNDRUMBLEPLAY_REP,
	SM_GPSTATES_SOUNDSTOP,
	SM_GPSTATES_SOUNDEXPIRE,
	SM_GPSTATES_SOUNDSETVOL,
	SM_GPSTATES_SOUNDDUCK,
	SM_GPSTATES_SOUNDDUCKALL,
	SM_GPSTATES_SOUNDSETPAN,
	SM_GPSTATES_SOUNDSETRANGE,
	SM_GPSTATES_SOUNDSETPOS,
	SM_GPSTATES_SOUNDPAUSE,
	SM_GPSTATES_SOUNDRESUME,
	SM_GPSTATES_SOUND_ATTACH_TO_PRIME,
	SM_GPSTATES_SOUND_ATTACH_TO_OBJ,
	SM_GPSTATES_SOUND_LOAD_BANK,

//	SM_GPSTATES_ADD_AS_GRASS_EFFECTOR,

// stuff for attaching things to an obj
	SM_GPSTATES_ATTACH_LIGHT,
	SM_GPSTATES_ATTACH_TO_ACTOR,
	SM_GPSTATES_ATTACH_FXGROUP,
	SM_GPSTATES_ATTACH_ACTOR_TO_ME,
	SM_GPSTATES_ATTACHMENT_TO_VEL,

	// FX group stuff
	SM_GPSTATES_KILL_FXGROUP,
	SM_GPSTATES_CREATE_FXGROUP,
	SM_GPSTATES_KILL_FXGROUP_FROM_PRIME,

	SM_GPSTATES_COL_OFF,
	SM_GPSTATES_FUR_PARAMS_SET,

	SM_GPSTATES_POSTFX,

	SM_GPSTATES_TRAVEL,
	SM_GPSTATES_TRAVEL_OVERRIDE,
	SM_GPSTATES_POP_BONE_ACTION,

	SM_GPSTATES_CAMTOTEX,
	SM_GPSTATES_CAMTOTEXOFF,
	SM_GPSTATES_USECAMTEX,

	SM_GPSTATES_CLIPBOX,

	SM_GPSTATES_VIDEO_PLAY,
	SM_GPSTATES_VIDEO_STOP,
	SM_GPSTATES_VIDEO_READY,
	SM_GPSTATES_VIDEO_WATCH,
	SM_GPSTATES_VIDEO_GOTO_FRAME,
	SM_GPSTATES_VIDEO_SET_SPEED,

	SM_GPSTATES_BLOWER,
	SM_GPSTATES_BLOWERTEMP,

	SM_GPSTATES_SWIPE,

	// for dealing with multiple splines or vels
	SM_GPSTATES_VEL_ADD,
	SM_GPSTATES_VEL_SELECT,
	SM_GPSTATES_SPLINE_ADD,
	SM_GPSTATES_SPLINE_SELECT,



	// for manipulating layouts
	SM_GPSTATES_LAYOUT_SWAPITEMRES,
	SM_GPSTATES_LAYOUT_SETITEMALPHA,
	SM_GPSTATES_LAYOUT_SETITEMCOLOR,

	SM_GPSTATES_RESTART_LEVEL,

// specialized event action states
	SM_GPSTATES_EA_GET_SM_PARAM,


	// debug
	SM_GPSTATES_SETLOGFILE,


//	SM_GPSTATES_SWATCH,
//	SM_GPSTATES_SETBAN,
	NUM_SM_GPSTATES,
};

// Scene Dispatch State Machine Functions
enum {
	SM_SCENEDISPSTATES_INIT	 = NUM_SM_GPSTATES,
	SM_SCENEDISPSTATES_CREATE,
	SM_SCENEDISPSTATES_LOAD,
	SM_SCENEDISPSTATES_ADD,
	SM_SCENEDISPSTATES_PLAYSCENE,
	SM_SCENEDISPSTATES_WIPELIGHTS,

//	SM_SCENEDISPSTATES_CAMERA,
	SM_SCENEDISPSTATES_LOADSHADER,
// 	SM_SCENEDISPSTATES_RUNCF,
	SM_SCENEDISPSTATES_WAITFOR,
	SM_SCENEDISPSTATES_ZEROTIMER,
	SM_SCENEDISPSTATES_TIMERWATCH,
	SM_SCENEDISPSTATES_CAM_GMOTION,
//	SM_SCENEDISPSTATES_POSTFX,
	SM_SCENEDISPSTATES_CLEARCOLOR,
	SM_SCENEDISPSTATES_WIND,
	SM_SCENEDISPSTATES_FOG,
	SM_SCENEDISPSTATES_WATERPLANE,
	SM_SCENEDISPSTATES_SHADSPLIT,
	SM_SCENEDISPSTATES_SHAD_LGROUP,
	SM_SCENEDISPSTATES_SCREENGRAB,
	SM_SCENEDISPSTATES_PARTICLE_LGROUP,

	SM_SCENEDISPSTATES_MOVECAMERA,
	SM_SCENEDISPSTATES_SET_PROJ_LGROUP,
	SM_SCENEDISPSTATES_GLOBAL_AMBIENT,
	SM_SCENEDISPSTATES_SCENELIGHTS_ON,
	SM_SCENEDISPSTATES_SCENELIGHTS_OFF,
	SM_SCENEDISPSTATES_SWAPLIGHTGROUPS,
	SM_SCENEDISPSTATES_GUNLIGHTS_ON,
	SM_SCENEDISPSTATES_GUNLIGHTS_OFF,
	SM_SCENEDISPSTATES_MOUNTEDGUNS_ON,
	SM_SCENEDISPSTATES_MOUNTEDGUNS_OFF,
	SM_SCENEDISPSTATES_SETTAPOUT,
	SM_SCENEDISPSTATES_LETTERBOX,
	SM_SCENEDISPSTATES_MOTION,
	SM_SCENEDISPSTATES_MOTION_BASE_HEIGHT_PCT,
	SM_SCENEDISPSTATES_MOTION_ANGLE,
	SM_SCENEDISPSTATES_MOTION_CENTER,
	SM_SCENEDISPSTATES_MOTION_FORWARD,
	SM_SCENEDISPSTATES_MOTION_RIGHT,
	SM_SCENEDISPSTATES_MOTION_BACK,
	SM_SCENEDISPSTATES_MOTION_LEFT,
	SM_SCENEDISPSTATES_MOTION_CUSTOM,

	NUM_SCENEDISPSTATES_MAX,
	MAX_CORESTATES=NUM_SCENEDISPSTATES_MAX,
};

// FX Group state machine functions

enum {
	SM_FXGROUPSTATES_LAUNCH	= NUM_SM_GPSTATES,
	SM_FXGROUPSTATES_AGE,
	SM_FXGROUPSTATES_KILL,
	SM_FXGROUPSTATES_WAIT_FOR_START_TRIG,
	SM_FXGROUPSTATES_REPEAT_UNTIL_KILL_TRIG,
	SM_FXGROUPSTATES_RUN_SPLINE,
};

// State machine for scene objects
enum {
	SM_SCENEOBJ_WAIT_FOR_TRIG = NUM_SM_GPSTATES,
	SM_SCENEOBJ_WAIT_FOR_ENDTRIG, 
	SM_SCENEOBJ_SLEEP,
	SM_SCENEOBJ_WAIT_CAM_COL,
	SM_SCENEOBJ_SET_PARAM_TRIG,
	SM_SCENEOBJ_SPLINE,
	SM_SCENEOBJ_ANIM,
	SM_SCENEOBJ_RUN_PROG,
	SM_SCENEOBJ_ATTACH,
	SM_SCENEOBJ_EXPLODE,
	SM_SCENEOBJ_TIRE_EMIT,
	SM_SCENEOBJ_VEHICLE_SOUND,
	SM_SCENEOBJ_BOLDER_EFFECTS,
	SM_SCENEOBJ_SHOT,
	SM_SCENEOBJ_MAKE_FRIENDLY,

	NUM_SCENEOBJ_STATES
};

extern PSM_State gGPStateMachineStates[];
extern PSM_State gSceneObjStateMachineStates[NUM_SCENEOBJ_STATES];



int SM_Move( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_MoveTo( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Hide( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Show( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SetOutline( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Blend( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_ZBuffer( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SwapRes( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SwapShader( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SwapTex( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_Swipe( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_UseScreenGrabTex( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_UseBlankTex( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_UseVertColors( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_UseBillboardY( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SetLightgroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SetEnvMap( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SetReflectivity( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_SetVSync( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );



// int SM_AniSeq( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RunProgram( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RunFreeProgram( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RunProgramWithParamPtr( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SwitchProgram( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_Sleep( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SleepRaw( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SleepRand( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_GoSubRnd( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_GoToRnd( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SetRecoveryPoint( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_GotoRecoveryPoint( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_GotoCB( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BrOnFlag( PSM_ExeContext *context,  void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SetName( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SendLocalEvent( PSM_ExeContext *context,  void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SendLocalEventByActorName( PSM_ExeContext *context,  void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_DropEventAction( PSM_ExeContext *context,  void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_WaitForTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_WaitForActor( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_WaitForBr( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BrOnTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BrOnNotTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BrOnAdjustment( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BrOnSplineFrame( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_WaitForActorBr( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_TurnColOff( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_WaitForObj( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_WaitForTravel( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_KillPrime( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_KillProg( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_KillByName( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_KillByResName( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_KillActor( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Advance( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_KillByGroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_KillObjGroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_HideByGroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_ShowByGroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_SoundPlay( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundPlayBGM( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundPlayRep( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundPlayVO( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundRumblePlay( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundRumblePlayRep( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundStop( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundExpire( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundSetVol( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundDuck( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundDuckAll( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundSetPan( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundSetRange( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundSetPos( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundPause( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundResume( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundAttachToPrime( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundAttachToObj( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SoundLoadBank( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

//int SM_Add_As_Grass_Effector( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SndVolAdjustName( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SndVolAdjustCMask( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_Spline( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Spline_Set_Gcol_Params( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_Spline_Speed( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Spline_Skip( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Spline_SkipTo( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Spline_Hold( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Spline_Run( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Spline_Step( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Swatch( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Spline_To_Vel( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
/*
int SM_Spline_Name( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Spline_Loop( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Spline_PKill( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Spline_GCol( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
*/

int SM_BAN_Channel( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
/*
int SM_BAN_Channel_Blend( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BAN_Channel_BlendMode( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BAN_Channel_IsolateSubskel( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
*/

int SM_BAN( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
// int SM_BAN_Timed( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BAN_Speed( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Bwatch( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BwatchTravel( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BAN_SkipTo( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BAN_Sync( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
/*
int SM_BAN_Name( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BAN_Loop( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_BAN_Blend( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
*/

int SM_Trans( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Tvel( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Tacc( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Rot( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Rvel( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Racc( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Scale( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Svel( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_Sacc( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_TransBounds( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_TvelBounds( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RotBounds( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RvelBounds( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_ScaleBounds( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SvelBounds( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_ClearBounds( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_TransRand( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RotRand( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_TvelRand( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RvelRand( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_TransWait( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_TvelWait( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RotWait( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RvelWait( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_ScaleWait( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_TransBr( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_TvelBr( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RotBr( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_RvelBr( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_ScaleBr( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_ScaleFrames( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );


int SM_AttachLight( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_AttachToActor( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_AttachFXGroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_AttachActorToMe( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_AttachmentToVel( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_CreateFXGroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_KillFXGroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_KillFXGroupFromPrime( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_FXGroup_WaitForStartTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_FXGroup_RepeatUntilKillTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_FXGroup_RunSpline( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_Restart_Level( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_EA_Get_SM_Param( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SetLogFile( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

// int SM_Move( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );


// int SM_Ban( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_SceneInit( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneCreate( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneLoad( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneAdd( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_ScenePlayScene( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneWipeLights( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneCameraSelect( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneCamera( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneCameraReset( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneCameraLoop( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneLoadShader( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
// int SM_SceneRunCF( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_WaitFor( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_TimerWatch( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_ZeroTimer( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SetTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_ClrTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_DelTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SetTriggerIfTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_SetTriggerIfNotTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_CamTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_CamGMotion( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_PostFx( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_ClearColor( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Wind( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Fog( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_WaterPlane( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_ShadSplit( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_ShadLGroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_ScreenGrab( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_ParticleLightGroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Blower( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_BlowerTemp( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );

int SM_SceneMoveCamera( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraSwatch( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraSwatchAbsolute( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraSplineHold( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraSplineRun( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraSplineSpeed( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraSpline_SkipTo( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraSpline_ConditionalSkipTo( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraSpline2( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraWatchAct( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraLookAt( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraSetFOV( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraShake( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CameraShakeSimple( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_GameSpeed( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_GunSpeed( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

//int SM_SetProjLightGroup( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SwapLightGroups( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_SceneGlobalAmbient( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_SceneLightsOn( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneLightsOff( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

/* belong in game
int SM_GunLightsOn( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_GunLightsOff( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_MountedGunsOn( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_MountedGunsOff( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_SetTapout( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
*/
int SM_Letterbox( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_Motion( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_MotionBaseHeightPct( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_MotionAngle( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_MotionCenter( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_MotionForward( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_MotionRight( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_MotionLeft( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_MotionBack( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_MotionCustom( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_FXGLaunch( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_FXGKill( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_FXGAge( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_ChangeColor( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_TimedColorSet( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );

int SM_FurParamsSet( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );


// Scene Obj States
int SM_SceneObj_WaitForTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_Sleep( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_WaitForCamCollision( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_SetParamTrigger( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_Spline( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_Anim( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_RunProgram( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_Attach( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_Explode( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_TireEmit( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_VehicleSound( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_BolderEffects( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_Shot( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_SceneObj_MakeFriendly( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );



// returns the index of the first string in list that is a match for match
int FindSmallStringInList( RTDataSmallString *match, RTDataSmallString *list, int count );

int SM_Travel( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_TravelOverride( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
void SM_TravelFunc_Apply(float speed, float *pAspeed, float *blends, int blend_count, TravelAnim* anim_slow, float scale_slow, TravelAnim* anim_fast, float scale_fast, SM_TravelFunc_Params* params);
int SM_TravelFunc( float speed, float iradius, float pitch, float scale, float *pAspeed, SM_TravelFunc_Params* params);
int SM_TravelFunc_SpeedOnly(float speed, float iradius, float pitch, float scale, float *pAspeed, SM_TravelFunc_Params* params);
int SM_PopBoneAction( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );


int SM_CamToTex( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_CamToTexOff( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );
int SM_UseCamTex( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time  );

int SM_SetClipBox( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );


int SM_Video_Play( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Video_Stop( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Video_Ready( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Video_Watch( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Video_GotoFrame( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Video_SetSpeed( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );

int SM_Vel_Add( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Vel_Select( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Spline_Add( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Spline_Select( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );


int SM_Layout_SwapItemRes( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Layout_SetItemAlpha( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Layout_SetItemColor( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );


// Misc helper funcs

// this is a helper func for SM_Trans/Vel/Rot
#define SET_OBJVEL_CH_VM_MODE_TFM_VEC 0
#define SET_OBJVEL_CH_VM_MODE_ROT_VEC 1
#define SET_OBJVEL_CH_VM_MODE_ROT_ROT 2
void SetObjVelChannel( SM_OV_Params *vals, Obj *prime, int cam_vm_mode, float *dst );


#endif // CORESTATEMACHINES_H
