// core_statemachines_datatypes.h
// Play Mechanix - Video Game System
// Copyright (c) 2008 Play Mechanix Inc. All Rights Reserved
//
// data types for core statemachines

// NOTES:
// only simple defines are supported by RTDataParser
// only local defines are supported by RTDataParser 
// only 'typdef struct's are supported by RTDataParser
// only single dimension arrays are supported
//

////////////////////
// SM PARMS
//
// Structs used for State Machine Params
// these are the global settings for a state machine
////////////////////

#ifndef CORESTATEMACHINESDATATYPES_H
#define CORESTATEMACHINESDATATYPES_H

#pragma pack(4)

// be sure to keep this define consitent across all datatypes.h files
#ifndef SM_NAME_SIZE
#define SM_NAME_SIZE 64
#endif

// this one should be first item in params struct for all statemachines
typedef struct SM_Common_Params
{
	Obj *prime; // parent object on which SM is acting
	ObjExeContext *oec; // the obj for this SM ExeContext
	ObjVel *vel; // ObjVel of object on which SM is acting
	ObjKeyAnim *spline;  // Spline of object on which SM is acting
	ObjTrackAnim *otan; // track anim (skel anims) of object on which SM is acting
	int active_anim_channel;
	int active_camera;
	char active_vel[SM_NAME_SIZE];
	char active_spline[SM_NAME_SIZE];

	int recovery_point; // instruction number to return to 'recover' from something (getting hit, etc, etc)
	int recovery_point_used; // how many times has the recovery point been used since the last time it was set?

} SM_Common_Params;

// state machine params for our general purpose state machine
typedef struct SM_GP_Params
{
	SM_Common_Params common; // this one should be first item in params struct for all statemachines

} SM_GP_Params;

typedef struct SM_Scene_Params
{
	SM_Common_Params common; // this one should be first item in params struct for all statemachines
//	char e_s_Name[SM_NAME_SIZE];
	int	sceneTimer;		//record GameWavetime when scene starts - used for scene timer dispatch


} SM_Scene_Params;


// state machine params for our destructible obj state machine
typedef struct SM_Destruct_Params
{
	int		e_cb_StayAlive;					// stay alive even after the last damage level has been hit
	char	e_s_BaseName[SM_NAME_SIZE];		// it goes like obj_name_s1, obj_name_s2, obj_name_s...
	char	e_s_DestructFX[SM_NAME_SIZE];	// fx played when dropping a level
	char	e_s_DieFX[SM_NAME_SIZE];		// fx played when drop to last level
	char	e_s_PartFX[SM_NAME_SIZE];		// fx attached to flyers

	float	DestructFXScale;
	float	DieFXScale;

	int		damageThreshold;				// hits before level drops
	int		currentDamageState;				// damage level we are at
	int		numHits;						// current hits at this level
	int		totalNumHits;					// total hits we have taken
	int		lastHitPnum;					// number of the player who hit last

	int		killObj;						// internal no kill flag used to always keep prime alive and instead just hide last level 
	int		useFlyers;						// singled if flyers are present
	int		activeFlyers;					// count of flyers that are still updating, used to remove after flyers are done
	char	trigger1[SM_NAME_SIZE];			// trigger to cause destructible level switch on hit
	char	trigger2[SM_NAME_SIZE];			// trigger to cause destructible level switch on hit
	int		trigger_used;					// flag thats set if the destruct is set off with a trigger
	char	fx_obj[SM_NAME_SIZE];			// optional pos/rot to place fx when destruct fx is played (vs normally coming from bullet collision data)
	float	fx_pos[3];
	float	fx_rot[3];
	char	hit_trigger[SM_NAME_SIZE];		// trigger to set when switching states
	char	start_trigger[SM_NAME_SIZE];	// trigger to say when this guy can get shot
	char	ani_trigger[SM_NAME_SIZE];		// trigger to say when this guy should start animating on spline
	int		killAfterAni;					// if true, kills object after animation completes
	int		died;
	char	hitProgram[SM_NAME_SIZE];		// Run this program when destructing first state

	char	attach_name[SM_NAME_SIZE];
	Obj*	attach_obj;
	uns32	attach_oid;

	int		light_group;

	//Pickup attributes
	int		pickupType;
	int		pickupSeen;
	uns32	pickupHiddenOID;				// if we have a hidden pickup associated with this destruct, this is the oid

	//Bonus attributes
	int		bonusType;

	// ObsShoot
	int		obsShootHitPlayer;

	// any explosion notification properties
	float	exp_min_dist;
	float	exp_max_dist;
	float	exp_rolloff;
	float	exp_force;

	// flyer launch vector params
	int		world;
	float	vec_x_min;
	float	vec_x_max;
	float	vec_y_min;
	float	vec_y_max;
	float	vec_mag_min;
	float	vec_mag_max;

	// other flyer behavior params
	float	rot_vel;
	float	gravity;
	unsigned int lifetime;
	float	flyer_part_chance;

	// some collision info saved for flyers
	float	shot_pos[3];
	float	shot_nrm[3];
	float	shot_dir[3];

	// info about obj movement for flyers
	float	prime_okan_tvel[3];
	float	prime_ov_tvel[3];
	int		use_obj_vel;

	// explosion handling distance
	float	exp_response_dist;				// respond to explosions with in this radius
	unsigned int exp_handle;						// internal handle for registration

	// enable destruction to happen
	int		enabled;

	float pulse;
	float pulseTimer;
	float pulseCycle;


} SM_Destruct_Params;

typedef struct SM_DestructSM_Params
{
	SM_Common_Params common; // this one should be first item in params struct for all statemachines
	SM_Destruct_Params destruct;

} SM_DestructSM_Params;


typedef struct SM_SceneObj_Params
{
	SM_Common_Params common; // this one should be first item in params struct for all statemachines
	char e_s_WaitTriggerName[SM_NAME_SIZE]; // if it waits for a trigger to start, this is what it waits for
	float e_n_Delay;
	char e_s_SendTriggerName[SM_NAME_SIZE]; // if needs to send a trigger, this is the one to send
	char e_s_SplineName[SM_NAME_SIZE];
	int e_cb_IgnoreSpline;
	int e_cb_LoopSpline;
	char e_s_AnimName[SM_NAME_SIZE];
	int e_i_RandomAnimOffset;  // if we are playing an anim, should it advance the anim a random amount at start?
	int e_i_GndCol;
	char e_s_EndTriggerName[SM_NAME_SIZE]; // trigger to kill this object
	char e_s_ProgramName[SM_NAME_SIZE]; // side program to run
	char e_s_AttachToObj[SM_NAME_SIZE]; // obj to attach to
	uns32 e_n_AttachToOID; // store off its OID
	char e_s_AttachToBone[SM_NAME_SIZE];
	int e_cb_HideAttachmentWithPrime;
	char e_s_HitFX[SM_NAME_SIZE];

	float e_n_ExplosionRadius;
	float e_n_ExplosionForce;

} SM_SceneObj_Params;

typedef struct SM_TireEmit_Params
{
	float e_n_WheelRadius;
	unsigned int i_LastProbTime;
	float i_LastProbPos[3];
	float i_LastColNrm[3];
	unsigned int i_CurTerrainType;
	unsigned int i_CurFXGroupOID;

} SM_TireEmit_Params;

typedef struct SM_VehicleSound_Params
{
	char e_s_BaseName[SM_NAME_SIZE];
	float e_n_Height;
	unsigned int e_n_MaxAvgVelSamples;
	unsigned int e_n_MinPlayTimeMS;
	unsigned int e_n_BlendTimeMS;
	float e_n_VelLowThresh;
	float e_n_VelMedThresh;
	float e_n_VelHighThresh;
	float e_n_MaxVolume;
	float e_n_Rolloff;
	
	float i_VelSamples[32];	
	unsigned int i_VelSampleCnt;	
	unsigned int i_VelSampleIndx;
	float i_LastPos[3];
	unsigned int i_LastCheckTime;
	char i_CurSnd[SM_NAME_SIZE];
	unsigned int i_CurSndID;
	unsigned int i_LastChangeTime;
	unsigned int i_SlowMoStarted;
} SM_VehicleSound_Params;

typedef struct SM_FXGroup_Params
{
	SM_Common_Params common; // this one should be first item in params struct for all statemachines
	char e_s_WaitTriggerName[SM_NAME_SIZE]; // if it waits for a trigger to start, this is what it waits for
	float e_n_Delay; // delay after trigger (seconds)
	float e_n_CamDelay; // delay after trigger (seconds) measured with main camera okan
	float e_n_Repeat; // Repeat launch of this emitter every x seconds
	float e_n_RepeatVar; // Variable repeat time amount
	float e_n_RepeatLifetime; // How long to repeat until stopping
	char e_s_KillTrigger[SM_NAME_SIZE];
	float e_n_Age; // Starting age so particles don't look like they just turned on
	char e_s_Attach[SM_NAME_SIZE];
	char e_s_AttachBone[SM_NAME_SIZE];

	int i_triggerTime;
	float i_camTriggerTime;
	int i_nextRepeatTime;
	int i_isTriggered;
	
} SM_FXGroup_Params;

////////////////////
// State PARMS
//
// Structs used for State Params
// these are the local settings/arguments passed into a state
////////////////////

// some generic params first
typedef struct SM_XYZ
{
	float e_n_x;
	float e_n_y;
	float e_n_z;
} SM_XYZ;

typedef struct SM_RGB
{
	float e_n_r;
	float e_n_g;
	float e_n_b;
} SM_RGB;

typedef struct SM_RGBA
{
	float e_n_r;
	float e_n_g;
	float e_n_b;
	float e_n_a;
} SM_RGBA;


typedef struct SM_Name
{
	char  e_s_Name[SM_NAME_SIZE];
} SM_Name;

typedef struct SM_iName
{
	char  i_s_Name[SM_NAME_SIZE];
} SM_iName;

typedef struct SM_Flag
{
	int e_cb_on;
} SM_Flag;

typedef struct SM_OnOff
{
	int e_cbg_on;
	int e_cbg_off;
} SM_OnOff;

typedef struct SM_Frame
{
	int e_n_Frame;
} SM_Frame;

typedef struct SM_Conditional_Frame
{
	char e_s_Label[SM_NAME_SIZE];
	int e_n_Frame;	
	int e_cb_Enabled;
	int e_cb_GotoLabel;
} SM_Conditional_Frame;

typedef struct SM_Trigger_Frame
{
	int e_n_Frame;
	char e_s_Trig[32];
} SM_Trigger_Frame;

typedef struct SM_Time
{
	float e_n_Time;
} SM_Time;

typedef struct SM_Speed
{
	float e_n_Speed;
} SM_Speed;

typedef struct SM_Speed_Change
{
	float e_n_Speed;
	float e_n_Time;
} SM_Speed_Change;


typedef struct SM_Channel
{
	int e_n_Channel;
} SM_Channel;


typedef struct SM_ForwBckwrds
{
	int e_cbg_Forwards;
	int e_cbg_Backwards;

} SM_ForwBckwrds;


typedef struct SM_Label
{
	char e_s_Label[SM_NAME_SIZE];
} SM_Label;

typedef struct SM_CountAndLabel
{
	int  e_n_Count;
	char e_s_Label[SM_NAME_SIZE];
} SM_CountAndLabel;

typedef struct SM_Lambda
{
	float e_n_Lambda;
} SM_Lambda;



typedef struct SM_TwoTrig_Params
{
	char	e_s_If[SM_NAME_SIZE];
	char	e_s_ThenSet[SM_NAME_SIZE];
} SM_TwoTrig_Params;


// some specific ones
typedef struct SM_PlayScene
{
	char  e_s_Name[SM_NAME_SIZE];
	int   e_cb_DoSceneStartTrig;
} SM_PlayScene;


typedef struct SM_GosubRnd_Params
{
	RTDataSmallString labels[10];
} SM_GosubRnd_Params;

typedef struct SM_GoToCB_Params
{
	int e_cb_On;
	RTDataSmallString e_s_Label;

} SM_GoToCB_Params;


typedef struct SM_WaitFor_Params
{
	float	e_n_maxTime;
	RTDataSmallString waitObjNames[10];
	int     e_cb_KillObjsOnTimeout;
	int		i_sent_die_eaction;
} SM_WaitFor_Params;

typedef struct SM_WaitForTrigger_Params
{
	char e_s_Name[SM_NAME_SIZE];
	float	e_n_Timeout;
	int     e_cb_SetTriggerOnTimeout;
} SM_WaitForTrigger_Params;


typedef struct SM_ActorName_Params
{
	RTDataSmallString e_s_actorName;
} SM_ActorName_Params;

typedef struct SM_ActorNames_Params
{
	RTDataSmallString e_s_actorNames[10];
} SM_ActorNames_Params;

typedef struct SM_WaitForActor_Params
{
	RTDataSmallString e_s_actorNames[10];
	float	e_n_Timeout;
	int     e_cb_KillActorsOnTimeout;
	int		i_sent_die_eaction;
} SM_WaitForActor_Params;


typedef struct SM_WaitForBr_Params
{
	float	e_n_timeOut;
	RTDataSmallString waitObjNames[10];
	char e_s_allDeadBr[SM_NAME_SIZE];
	char e_s_exitTrigger[SM_NAME_SIZE];
	char e_s_exitTrigBr[SM_NAME_SIZE];
	int     e_cb_KillObjsOnTimeout;
	int     e_cb_SetTriggerOnTimeout;
	int		i_sent_die_eaction;
} SM_WaitForBr_Params;

typedef struct SM_SoundPlay_Params
{
	char e_s_Name[SM_NAME_SIZE];
	float e_n_Vol;
	int e_cb_Loop;
//	char e_s_Channel[SM_NAME_SIZE];
	int e_cbg_High;
	int e_cbg_Med;
	int e_cbg_Low;
	int e_cb_AttachToPrime;
} SM_SoundPlay_Params;

typedef struct SM_SoundPlayRep_Params
{
	char e_s_Name[SM_NAME_SIZE];
	float e_n_Vol;
	int e_n_Cnt;
	int e_n_InvervalMS;
	//	char e_s_Channel[SM_NAME_SIZE];
	int e_cbg_High;
	int e_cbg_Med;
	int e_cbg_Low;
	int e_cb_AttachToPrime;
	unsigned int i_LastPlay;
} SM_SoundPlayRep_Params;

typedef struct SM_SoundPlayVO_Params
{
	char e_s_Name[SM_NAME_SIZE];
	float e_n_Duration;
	float e_n_Vol;
}SM_SoundPlayVO_Params;


typedef struct SM_SoundSetVol_Params
{
	char e_s_Name[SM_NAME_SIZE];
	float e_n_Vol;
	int e_n_FadeTimeMS;
} SM_SoundSetVol_Params;

typedef struct SM_SoundSetPan_Params
{
	char e_s_Name[SM_NAME_SIZE];
	float e_n_LeftAmt;
} SM_SoundSetPan_Params;

typedef struct SM_SoundSetPos_Params
{
	char e_s_Name[SM_NAME_SIZE];
	float e_n_x;
	float e_n_y;
	float e_n_z;
	char e_s_Spline[SM_NAME_SIZE];
	int e_cbg_3d;
	int e_cbg_cm2;
	int e_cbg_cm3;
	int e_cbg_cm4;
	int e_cbg_cm5;
} SM_SoundSetPos_Params;

typedef struct SM_SoundSetRange_Params
{
	float e_n_DepthStart;
	float e_n_DepthRange;
	float e_n_DepthRolloff;
	float e_n_PanRange;
	float e_n_PanRatio;

} SM_SoundSetRange_Params;

typedef struct SM_SoundAttachObj_Params
{
	char e_s_ObjName[SM_NAME_SIZE];
	char e_s_SoundName[SM_NAME_SIZE];	
} SM_SoundAttachObj_Params;

typedef struct SM_SoundExpire_Params
{
	char e_s_Name[SM_NAME_SIZE];
	unsigned int e_n_TimeMS;
} SM_SoundExpire_Params;

typedef struct SM_SoundDuck_Params
{
	char e_s_Name[SM_NAME_SIZE];
	float e_n_ToVol;
	float e_n_TimeMS;
	float e_n_BlendMS;
} SM_SoundDuck_Params;


typedef struct SM_SleepRaw_Params
{
	float e_n_Frames;
	float i_n_ElapstedMS;
} SM_SleepRaw_Params;

typedef struct SM_SleepRnd_Params
{
	float e_n_minFrames;
	float e_n_maxFrames;
} SM_SleepRnd_Params;


// params for states that alter objvel
typedef struct SM_OV_Params
{
	int e_cb_x;
	float e_n_x;
	int e_cb_y;
	float e_n_y;
	int e_cb_z;
	float e_n_z;
	int e_cb_add;
	int e_cb_multiply;
	int e_cb_cam_space;
	int e_cb_projectile_space;

} SM_OV_Params;


typedef struct SM_OV_Bounds_Params
{
	int e_cbg_min;
	int e_cbg_max;

	float e_n_x;
	float e_n_y;
	float e_n_z;

} SM_OV_Bounds_Params;

typedef struct SM_OVRand_Params
{
	int e_cb_x;
	int e_cb_posNegX;
	float e_n_xmin;
	float e_n_xmax;
	int e_cb_y;
	int e_cb_posNegY;
	float e_n_ymin;
	float e_n_ymax;
	int e_cb_z;
	int e_cb_posNegZ;
	float e_n_zmin;
	float e_n_zmax;
	int e_cb_add;
	int e_cb_multiply;

} SM_OVRand_Params;

typedef struct SM_OV_Cmp_Chan_Params
{
	int e_cb_gt;
	int e_cb_lt;
	int e_cb_eq;
	float e_n_val;
} SM_OV_Cmp_Chan_Params;

typedef struct SM_OV_Wait_Params
{
	int e_cb_x;
	SM_OV_Cmp_Chan_Params e_x_params;
	int e_cb_y;
	SM_OV_Cmp_Chan_Params e_y_params;
	int e_cb_z;
	SM_OV_Cmp_Chan_Params e_z_params;

	int e_n_frames;

} SM_OV_Wait_Params;


typedef struct SM_OV_Br_Params
{
	int e_cb_x;
	SM_OV_Cmp_Chan_Params e_x_params;
	int e_cb_y;
	SM_OV_Cmp_Chan_Params e_y_params;
	int e_cb_z;
	SM_OV_Cmp_Chan_Params e_z_params;

	char e_s_Label[SM_NAME_SIZE];
	int e_cb_GoSub;

} SM_OV_Br_Params;


typedef struct SM_Scale_Frames_Params
{
	float e_n_frames;
	int e_cb_x;
	float e_n_start_x;
	float e_n_end_x;
	int e_cb_y;
	float e_n_start_y;
	float e_n_end_y;
	int e_cb_z;
	float e_n_start_z;
	float e_n_end_z;

	uns32 i_startTick;

} SM_Scale_Frames_Params;


typedef struct SM_BoundsTypes_Params
{
	int e_cb_trans;
	int e_cb_tvel;
	int e_cb_rot;
	int e_cb_rvel;
	int e_cb_scale;
	int e_cb_svel;
	int e_cb_all;
} SM_BoundsTypes_Params;



typedef struct SM_Move_Params
{
	SM_XYZ e_n_dest;
	float e_n_speed;
	int e_cb_absolute;

	float i_dist;

} SM_Move_Params;



typedef struct SM_MoveTo_Params
{
	int e_cb_move;
	SM_XYZ e_n_xyz;
	int e_n_frames;
	int e_n_easein;
	int e_n_easeout;


	int e_cb_rotate;
	float e_n_rot[3];
	int e_n_rot_frames;
	int e_n_rot_easein;
	int e_n_rot_easeout;

	int e_cb_scale;
	float e_n_scale[3];
	int e_n_scale_frames;
	int e_n_scale_easein;
	int e_n_scale_easeout;

	int e_cb_center_pivot;

	unsigned int i_StartTime;
	float i_startxyz[3];
	float i_startxyzr[3];
	float i_startxyzs[3];


} SM_MoveTo_Params;

typedef struct SM_AniSeq_Params
{
	char  e_s_AniSeq[SM_NAME_SIZE];
	int   e_cb_wait_on_seq;

	uns32 i_seqoid;

} SM_AniSeq_Params;


typedef struct SM_RunProgram_Params
{
	char  e_s_Program[SM_NAME_SIZE];
	int   e_cb_wait_on_program;

	uns32 i_progoid;

} SM_RunProgram_Params;

typedef struct SM_RunProgWithParamsPtr_Params
{
	char  e_s_Program[SM_NAME_SIZE];
	int   e_cb_wait_on_program;
	int   e_cb_run_freestanding;
	void  *params;

	uns32 i_progoid;

} SM_RunProgramWithParamsPtr_Params;


typedef struct SM_Spline_Params
{
	char  e_s_Spline[SM_NAME_SIZE];
	float e_n_speed;
	int	  e_cb_loop;
	int	  e_cb_gcol;
	int	  e_cb_pkill;

} SM_Spline_Params;

typedef struct SM_Spline_GCol_Params
{
	float e_n_LookForward;
	float e_n_LookBack;
} SM_Spline_GCol_Params;


typedef struct SM_CamSpline2_Params
{
	char  e_s_Spline[SM_NAME_SIZE];
	float e_n_speed;
	int	  e_cb_loop;
	int	  e_cb_after;

} SM_CamSpline2_Params;

typedef struct SM_CamWatchAct_Params
{
	char	e_s_Actor[SM_NAME_SIZE];
	float	e_n_OffsetX;
	float	e_n_OffsetY;
	float	e_n_OffsetDist;	
	ObjVel*	i_target_vel;

} SM_CamWatchAct_Params;

typedef struct SM_Channel_BlendMode
{
	int e_cbg_Blend;
	int e_cbg_AddRots;
	int e_cbg_AddTrans;
	int e_cbg_AddRotsAndTrans;
} SM_Channel_BlendMode;


typedef struct SM_Channel_Blend
{
//	int e_cbg_Blend;
//	int e_cbg_Add;
	float e_n_val;
	float e_n_time;
} SM_Channel_Blend;


typedef struct SM_BAN_Params
{
	char  e_s_Anim[SM_NAME_SIZE];
	float e_n_speed;
	float e_n_blendFrames;
	int	  e_cb_loop;
	int	  e_cb_blend_out;
	int	  e_cb_snap;
	int	  e_cb_sync;
	int	  e_cb_realtime;
} SM_BAN_Params;

typedef struct SM_BAN_Sync_Params
{
	char e_s_ObjToSync[SM_NAME_SIZE];
	char e_s_ObjTarget[SM_NAME_SIZE];
} SM_BAN_Sync_Params;



typedef struct SM_BanBlend_Params
{
	char  e_s_Anim[SM_NAME_SIZE];
	int	  e_cb_ramp;
	float e_n_blendtime;
} SM_BanBlend_Params;


typedef struct SM_Create_Params
{
	char e_s_ObjName[SM_NAME_SIZE]; // Name of object
	int e_cbg_d2; //type: d2
	int e_cbg_d3; //type: d3 (skel obj)
	int e_cbg_dg; //type: dg display grp
	int e_cbg_lit; //type: light
	char e_s_ProgName[SM_NAME_SIZE]; // Name of program
	char e_s_SeqName[SM_NAME_SIZE]; // Name of sequence
	char e_s_ColName[SM_NAME_SIZE]; // Name of collision geom
	int e_cb_vel;
	int e_cb_skel;
	int e_cb_shadow; //shadow
	char e_s_ShaderName[SM_NAME_SIZE]; //shader

} SM_Create_Params;

typedef struct SM_BrOnFlag_Params
{
	char	e_s_flag[256];
	int		e_n_flagVal;
	char	e_s_label[256];
} SM_BrOnFlag_Params;

typedef struct SM_BrOnTrig_Params
{
	char	e_s_Trigger[SM_NAME_SIZE];
	char	e_s_brLabel[SM_NAME_SIZE];
	int     e_cb_GoSub;
} SM_BrOnTrig_Params;


typedef struct SM_BrOnAdj_Params
{
	char	e_s_Group[SM_NAME_SIZE];
	char	e_s_Adj[SM_NAME_SIZE];
	char	e_s_brLabel[SM_NAME_SIZE];
	int     e_cb_GoSub;
} SM_BrOnAdj_Params;

typedef struct SM_BrOnSplineFrame_Params
{
	int		e_n_Frame;
	char	e_s_brLabel[SM_NAME_SIZE];
} SM_BrOnSplineFrame_Params;


typedef struct SM_CameraSel_Params
{	
	int e_cbg_3d;
	int e_cbg_cm2;
	int e_cbg_cm3;
	int e_cbg_cm4;
	int e_cbg_cm5;

	/*
	int e_cbg_2d;
	int e_cbg_3df;
	int e_cbg_2df;
	int e_cbg_3dt;
	int e_cbg_2dt;
	*/

} SM_CameraSel_Params;


typedef struct SM_Camera_Params
{	
	char e_s_Name[SM_NAME_SIZE]; 
	float e_n_speed;
} SM_Camera_Params;

typedef struct SM_Cam_GMotion_Params
{
	int   e_cb_on;
	float e_n_xRange;
	float e_n_yRange;
	float e_n_speed;

	float e_n_a_xRange;
	float e_n_a_yRange;
	float e_n_a_speed;

} SM_Cam_GMotion_Params;

typedef struct SM_PostFx_Params
{
	char e_s_Name[SM_NAME_SIZE];
	int e_n_blendms;
} SM_PostFx_Params;

typedef struct SM_Wind_Params
{
	char e_s_Name[SM_NAME_SIZE];
} SM_Wind_Params;

typedef struct SM_SendActorEvent_Params
{
	char  e_s_ActorName[SM_NAME_SIZE];
	char  e_s_EventName[SM_NAME_SIZE];
} SM_SendActorEvent_Params;


// for attaching things to the obj
typedef struct SM_AttachLight_Params
{
	char  e_s_light[SM_NAME_SIZE];
	char  e_s_bone[SM_NAME_SIZE];
	float e_n_speed;
} SM_AttachLight_Params;

typedef struct SM_AttachToActor_Params
{
	char  e_s_Actor[SM_NAME_SIZE];
	char  e_s_Joint[SM_NAME_SIZE];
	int   e_cb_bindspace;
	int   e_cb_2dproj;
	float e_n_proj_z;
	int   e_cb_no_trans;
	int   e_cb_no_rots;
	int   e_cb_snap_to_rot;
	char  e_s_offsetspline[SM_NAME_SIZE];
	int   e_cb_offsetvel;
	float e_n_x;
	float e_n_y;
	float e_n_z;
	int   e_cb_kill;
	int   e_cb_bake2vel;
	int   e_cb_hideWithPrime;
} SM_AttachToActor_Params;

typedef struct SM_AttachToVel_Params
{
	char  e_s_AttachmentName[SM_NAME_SIZE];
	int   e_cb_CreateNewVel;
	char  e_s_VelName[SM_NAME_SIZE];

} SM_AttachToVel_Params;

typedef struct SM_AttachFXGroup_Params
{
	char  e_s_fxgroup[SM_NAME_SIZE];
	char  e_s_joint[SM_NAME_SIZE];
	int   e_cb_addoffset;
	int   e_cb_bindspace;
	float e_n_x;
	float e_n_y;
	float e_n_z;
	int   e_cb_kill;
} SM_AttachFXGroup_Params;

typedef struct SM_KillFXGroup_Params
{
	char  e_s_fxgroup[SM_NAME_SIZE];
	int   e_cb_attached;
} SM_KillFXGroup_Params;


typedef struct SM_CreateFXGroup_Params
{
	char e_s_fxgroup[SM_NAME_SIZE];

} SM_CreateFXGroup_Params;




// Event Action Datatypes
// These are used with the states designed to be used w/ Event Action calls
typedef struct SM_EA_Get_Param_Request
{
	char param_name[4*PSM_MAX_NAME_LEN];
	char sm_params_type[PSM_MAX_NAME_LEN];
	void *data_buf;
	int  data_size;
	int error_code;
} SM_EA_Get_Param_Request;


typedef struct SM_MoveCamera_Params
{
	int e_cb_move;
	SM_XYZ e_n_xyz;
	int e_n_frames;
	int e_n_easein;
	int e_n_easeout;


	int e_cb_rotate;
	float e_n_rot[3];
	int e_n_rot_frames;
	int e_n_rot_easein;
	int e_n_rot_easeout;

	int e_cb_lookat;
	float e_n_lxyz[3];
	int e_n_lookat_frames;
	int e_n_lookat_easein;
	int e_n_lookat_easeout;

	unsigned int i_StartTime;
	float i_startxyz[3];
	float i_startxyzr[3];
	float i_startxyzl[3];


} SM_MoveCamera_Params;


typedef struct SM_SetOutline_Params
{
	int e_cb_on;
	float e_n_size;
	float e_n_outline_rgba[4];
	float e_n_inside_rgba[4];

} SM_SetOutline_Params;



typedef struct SM_GlobalAmbient_Params
{
	int  e_cb_on;
	float e_n_rgb[3];
	float e_n_intensity;
} SM_GlobalAmbient_Params;

typedef struct SM_SndVolAdjustName_Params
{
	char e_s_Name[SM_NAME_SIZE];
	int e_n_Vol;
	float e_n_RampTime;
} SM_SndVolAdjustName_Params;



typedef struct SM_Blend_Params
{
	int e_cb_on;
	float e_n_alpha;
	int e_cb_relative;
} SM_Blend_Params;

typedef struct SM_ZBuffer_Params
{
	int e_cb_DepthWrite_on;
	int e_cb_DepthWrite_off;

} SM_ZBuffer_Params;


typedef struct SM_Destruct_SetDam_Params
{
	RTDataSmallString e_s_Part;
	int e_n_Level;
} SM_Destruct_SetDam_Params;

typedef struct SM_intAB
{
	int e_n_A;
	int e_n_B;
} SM_intAB;

typedef struct SM_ChangeColorParams
{
	float e_n_red;
	float e_n_green;
	float e_n_blue;
	float e_n_alpha;

	unsigned int e_n_millisec;


	float i_n_startRed;
	float i_n_startGreen;
	float i_n_startBlue;
	float i_n_startAlpha;
} SM_ChangeColorParams;


typedef struct SM_TimedColorChangeParams
{
	int e_cb_red;
	float e_n_r;

	int e_cb_green;
	float e_n_g;

	int e_cb_blue;
	float e_n_b;

	int e_cb_alpha;
	float e_n_a;

	unsigned int e_n_millisec;
	int   e_cb_Multiply;
	int   e_cb_Add;

	float i_n_startR;
	float i_n_startG;
	float i_n_startB;
	float i_n_startA;

} SM_TimedColorChangeParams;



// water plane settings
typedef struct SM_WaterPlane_Params
{
	char  e_s_Name[SM_NAME_SIZE];
	float e_n_ReflectionMapScale;
	float e_n_ClearR;
	float e_n_ClearG;
	float e_n_ClearB;

} SM_WaterPlane_Params;




typedef struct SM_ScreenGrab_Params
{

	int e_cbg_3d;
	int e_cbg_cm2;
	int e_cbg_cm3;
	int e_cbg_cm4;
	int e_cbg_cm5;

	int e_n_x;
	int e_n_y;

	int e_n_width;
	int e_n_height;

} SM_ScreenGrab_Params;

typedef struct SM_CamToTex_Params
{
	int e_cbg_Slot1;
	int e_cbg_Slot2;
	int e_cbg_Slot3;
	int e_cbg_Slot4;
} SM_CamToTex_Params;

typedef struct SM_LocalEvent_Params
{
	char  e_s_Name[SM_NAME_SIZE];
	int   e_cb_Self;
	int   e_cb_Kids;
	int   e_cb_Siblings;
	int   e_cb_Parent;
} SM_LocalEvent_Params;


typedef struct SM_SetClipBox_Params
{
	SM_XYZ e_n_nw_corner;
	SM_XYZ e_n_se_corner;

} SM_SetClipBox_Params;

typedef struct SM_Video_Play_Params
{
	char  e_s_Name[SM_NAME_SIZE];
	int   e_cb_loop;
	int   e_cb_primekill;
	int   e_cb_ignoretime;

} SM_Video_Play_Params;

typedef struct SM_Video_GotoFrame_Params
{
	int e_n_Frame;
	int e_cb_relative;
} SM_Video_GotoFrame_Params;


typedef struct SM_Swipe_Params
{
	int e_n_Frames;
	float e_n_Border;
	int e_cb_Vert;
	int e_cb_SwipeOff;
	float e_n_Distance;

} SM_Swipe_Params;


typedef struct SM_VelSpl_Add_Params
{
	char e_s_Name[SM_NAME_SIZE];
	int  e_cb_LinkBefore;
	char e_s_LinkPoint[SM_NAME_SIZE];
} SM_VelSpl_Add_Params;


typedef struct SM_LayoutSwapRes_Params
{
	char  e_s_Item[SM_NAME_SIZE];
	char  e_s_Subitem[SM_NAME_SIZE];
	char  e_s_Res[SM_NAME_SIZE];
} SM_LayoutSwapRes_Params;

typedef struct SM_LayoutSetAlpha_Params
{
	char  e_s_Item[SM_NAME_SIZE];
	char  e_s_Subitem[SM_NAME_SIZE];

	float e_n_Alpha;

} SM_LayoutSetAlpha_Params;


typedef struct SM_LayoutSetColor_Params
{
	char  e_s_Item[SM_NAME_SIZE];
	char  e_s_Subitem[SM_NAME_SIZE];

	float e_n_r;
	float e_n_g;
	float e_n_b;

} SM_LayoutSetColor_Params;

typedef struct SM_Camera_LookAt_Params
{
	RTDataSmallString e_s_actorName;
	char e_s_bone[SM_NAME_SIZE];
	float e_n_slerpTime;
	float e_n_smoothing;
} SM_Camera_LookAt_Params;

typedef struct SM_Camera_SetFOV_Params
{
	float e_n_FOV;
	unsigned int e_n_frames;
	float i_n_OriginalFOV;
} SM_Camera_SetFOV_Params;

typedef struct SM_Camera_Shake_Params
{
	// external parameters

	// shake parameters
	int e_cb_Shake;
	float e_n_Intensity;
	float e_n_Seconds;
	int e_n_Period;
	int e_cb_RotOnly;
	int e_cb_TransOnly;

	// spring parameters
	int e_cb_Spring;
	float e_n_DispX;
	float e_n_DispY;
	float e_n_DispZ;
	float e_n_ForceX;
	float e_n_ForceY;
	float e_n_ForceZ;
	float e_n_SmoothX;
	float e_n_SmoothY;
	float e_n_SmoothZ;
	float e_n_DirX;
	int e_cb_RndDirX;
	float e_n_DirY;
	int e_cb_RndDirY;
	float e_n_DirZ;
	int e_cb_RndDirZ;
	float e_n_MaxOscillations;

	// foot step stuff
	int e_cb_FootStep;
	char e_s_Name[64];
	float e_n_StartFrame;
	float e_n_EndFrame;
	float e_n_MinDist;
	float e_n_MaxDist;
	char e_s_SFX[64];
	float e_n_SFX_Vol;

	// internal stuff
	unsigned int i_n_oid;
	unsigned int i_n_foot_state[4];	// four feet max
	unsigned int i_n_last_sfx_tick;
} SM_Camera_Shake_Params;

typedef struct SM_Camera_Shake_Simple_Params
{
	// shake parameters
	float e_n_Intensity;
	float e_n_Seconds;
	int e_n_Period;
} SM_Camera_Shake_Simple_Params;

typedef struct SM_Blower_Params
{
	float e_n_Intensity;
} SM_Blower_Params;

typedef struct SM_BlowerTemp_Params
{
	float e_n_Intensity;
	float e_n_Seconds;
} SM_BlowerTemp_Params;

typedef struct SM_Motion_Params
{
	float e_n_frame;
	float e_n_duration;
	float e_n_tilt;
	float e_n_pitch;
	float e_n_height;
	float e_n_jerk;
} SM_Motion_Params;

typedef struct SM_MotionAngle_Params
{
	float e_n_frame;
	float e_n_duration;
	float e_n_angle;
	float e_n_radius;
} SM_MotionAngle_Params;

typedef struct SM_MotionDirection_Params
{
	float e_n_frame;
	float e_n_duration;
	float e_cb_hitstopper;
} SM_MotionDirection_Params;

typedef struct SM_MotionCustom_Params
{
	float e_n_frame;
	char e_s_name[64];
} SM_MotionCustom_Params;

#pragma pack()

#endif // CORESTATEMACHINES_DATATYPES_H


