// cpmgui.c
// Play Mechanix - Video Game System
// Copyright (c) 2010 Play Mechanix Inc. All Rights Reserved
// Crazy Play Mechanix GUI

#ifndef _CPMGUI_H_
#define _CPMGUI_H_

#define CPMG_NAME_SIZE		PM_NAME_SIZE
#define CPMG_PATH_SIZE		256
#define CPMG_ATTRIB_SIZE	256

enum
{
	CPMG_ITEM,
	CPMG_SUBITEM,
	CPMG_LAYOUT,
};

enum
{
	CPMG_NONE_IMPORT,
	CPMG_TEMPLATE_IMPORT,
	CPMG_ITEM_IMPORT,
	CPMG_LAYOUT_IMPORT,
};

// Flags for allowed imports
#define	CPMG_NO_IMPORT_ALLOWED			0x00000000
#define	CPMG_TEMPLATE_IMPORT_ALLOWED	0x00000001
#define	CPMG_ITEM_IMPORT_ALLOWED		0x00000002
#define	CPMG_LAYOUT_IMPORT_ALLOWED		0x00000004

enum
{
  CPMG_IMG_C,
  CPMG_IMG_L,
  CPMG_IMG_R,
  CPMG_IMG_T,
  CPMG_IMG_B,
  CPMG_IMG_TL,
  CPMG_IMG_TR,
  CPMG_IMG_BL,
  CPMG_IMG_BR,
  NUM_CPMG_IMG_SECTIONS,
};

enum
{
  CPMG_IMG_UV_T,
  CPMG_IMG_UV_L,
  CPMG_IMG_UV_B,
  CPMG_IMG_UV_R,
  NUM_CPMG_IMG_UVS,
};

typedef struct cpmg_template_def
{
	char name[CPMG_NAME_SIZE];
	List attribs;
} cpmg_template_def;

typedef struct templateAttrib
{
    int     index;
    xmlChar name[CPMG_NAME_SIZE];
    xmlChar value[CPMG_NAME_SIZE];
} templateAttrib;

typedef struct cpmg_import_def
{
	int  type;
	char sourceLayout[CPMG_NAME_SIZE];	// only used if import_type is CPMG_LAYOUT_IMPORT
	char targetLayout[CPMG_NAME_SIZE];	// only used if import_type is CPMG_LAYOUT_IMPORT
	List namespace_map;
} cpmg_import_def;

typedef struct cpmg_namespace_map
{
	char source[CPMG_NAME_SIZE];
	char target[CPMG_NAME_SIZE];
} cpmg_namespace_map;

typedef struct cpmg_img_section
{
  int defined;
  float size[2];
  float uvs[NUM_CPMG_IMG_UVS];
} cpmg_img_section;

typedef struct cpmg_img
{
	int defined;
	cpmg_img_section sections[NUM_CPMG_IMG_SECTIONS];
} cpmg_img;

enum
{
  CPMG_IMG_STATE_NORMAL,
  CPMG_IMG_STATE_HOVER,
  CPMG_IMG_STATE_PRESSED,
  CPMG_IMG_STATE_DISABLED,
  NUM_CPMG_IMG_STATES,
};

typedef struct cpmg_imgset
{
	char name[PM_NAME_SIZE];
	char texture[PM_NAME_SIZE];
	cpmg_img images[NUM_CPMG_IMG_STATES];

} cpmg_imgset;

typedef struct cpmg_vert
{
	float trans[3];
	float uv[2];
} cpmg_vert;

#define CPMG_IMG_MAX_VERTS 4*NUM_CPMG_IMG_SECTIONS
#define CPMG_IMG_MAX_INDICES 6*NUM_CPMG_IMG_SECTIONS

typedef struct cpmg_img_verts
{
	cpmg_vert verts[CPMG_IMG_MAX_VERTS];
	uns32     indices[CPMG_IMG_MAX_INDICES];
	int num_verts;
	int num_indices;
} cpmg_img_verts;

typedef struct cpmg_img_obj
{
	char imageset[PM_NAME_SIZE];
	char texture[PM_NAME_SIZE];
	float width;
	float height;
	int state;
	cpmg_img_verts verts;
	uns32 flags;
} cpmg_img_obj;

void *MemAllocCPMG(size_t size);

enum 
{
	CPMG_SUBITEM_TYPE_IMAGESET,
	CPMG_SUBITEM_TYPE_DISP5,
	CPMG_SUBITEM_TYPE_TEXT,
	CPMG_SUBITEM_TYPE_CLIP,
};

// Flags for justifying subitems
#define	CPMG_ITEM_FLAG_HJUSTIFY_CENTER	0x00000001
#define	CPMG_ITEM_FLAG_HJUSTIFY_LEFT	0x00000002
#define	CPMG_ITEM_FLAG_HJUSTIFY_RIGHT	0x00000004
#define	CPMG_ITEM_FLAG_VJUSTIFY_CENTER	0x00000008
#define	CPMG_ITEM_FLAG_VJUSTIFY_TOP		0x00000010
#define	CPMG_ITEM_FLAG_VJUSTIFY_BOTTOM	0x00000020

typedef struct cpmg_layout_subitem
{
	int type;
	char name[CPMG_NAME_SIZE];
	char res[CPMG_NAME_SIZE];
	char shader[CPMG_NAME_SIZE];
	char program[CPMG_NAME_SIZE];
	float trans[3];
	float rot[3];
	float scale[3];
	float size[3];
	float color[4];
	int	col;
	int light_group;
	int skeleton;
	int scroll_text;
	float scroll_text_speed;
	float scroll_text_border;
	int scroll_text_pause;

	uns32 flags;

} cpmg_layout_subitem;

typedef struct cpmg_layout_item_def
{
	char name[CPMG_NAME_SIZE];
	List subitems;
} cpmg_layout_item_def;

typedef struct cpmg_layout_item
{
	char name[CPMG_NAME_SIZE];
	char ns_name[CPMG_NAME_SIZE];
	char updatefunc[CPMG_NAME_SIZE];
	char program[CPMG_NAME_SIZE];
	char datatype[CPMG_NAME_SIZE];
	float trans[3];
	float rot[3];
	float scale[3];
	float size[3];
	size_t data_size;	// if you want to pass data from a layout item to a
	void *data;			// created object, use these.


} cpmg_layout_item;

typedef struct cpmg_layout
{
	char name[CPMG_NAME_SIZE];
	char updatefunc[CPMG_NAME_SIZE];
	char datatype[CPMG_NAME_SIZE];
	char program[CPMG_NAME_SIZE];
	float size[3];
	size_t data_size;	// if you want to pass data from a layout item to a
	void *data;			// created object, use these.
	List items;

} cpmg_layout;


#ifdef __cplusplus
extern "C" {
#endif

int cpmgui_Init(void);
int cpmgui_Reload();
int cpmgui_Final(void);
int cpmgui_Loop(void);

int cpmgui_LoadImageSet( char *filename );
int cpmgui_LoadImageSetFile( char *filename, void *data );
int cpmgui_LoadLayout( char *filename );
int cpmgui_LoadLayoutFile( char *filename, void *data );
int cpmgui_LayoutExists( char *layout_name );
int cpmgui_PurgeLayout( char *layout_name );

int cpmgui_LayoutItemAddSubItem( char *item_name, cpmg_layout_subitem *subitem );
int cpmgui_CloneLayoutItem( char *item_name, char *new_item_name );
int cpmgui_CloneLayoutItemWithNameSpace( char *name_space, char *item_name, char *new_namespace, char *new_item_name );
int cpmgui_LayoutAddItem( char *layout_name, cpmg_layout_item *item );

int cpmgui_SetSubitemResourceWithNameSpace( char *name_space, char *item_name, char *subitem_name, char *value );
int cpmgui_SetSubitemResource( char *item_name, char *subitem_name, char *value );

int cpmgui_BuildImgObj( char *imageset, int state, uns32 flags, float width, float height, cpmg_img_obj *dst );
int cpmgui_BuildResDataDisp5( cpmg_img_obj *src, ResDataDisp5 *disp5 );

int cpmgui_SetUpdateFunc( char *name, int (*UpdateFunc)(ObjCPMGuiRes *o) );

int cpmgui_BuildLayoutItem( char *name_space, char *item_name, char *name, int (*UpdateFunc)(ObjCPMGuiRes *o), N2 *link, ObjCPMGuiRes **pocpmg );
int cpmgui_BuildLayout( char *layout_name, char *name, int (*UpdateFunc)(ObjCPMGuiRes *o), uns32 create_flags, N2 *link, ObjCPMGuiRes **pocpmg );

cpmg_layout *cpmgui_LayoutGet( char *layout_name, int force_create );
List *cpmgui_LayoutItemGetSubItems( char *item_name );
List *cpmgui_LayoutItemGetSubItemsWithNameSpace( char *name_space, char *item_name );

// helper funcs to be used by gui callbacks and such
int ObjCPMG_GunColTest( ObjCPMGuiRes *ocpmg, int gnum, int camnum );

int Obj_GunColTest( Obj *o, int gnum, ObjCam *ocam );

int ObjCPMG_SetImgStates( ObjCPMGuiRes *ocpmg, int state );

int ObjCPMG_SetAlpha( ObjCPMGuiRes *ocpmg, float val );
ObjCPMGuiRes *ObjCPMG_GetChildItem( ObjCPMGuiRes *ocpmg, char *name );
ObjDispRes *ObjCPMG_GetChildODR( ObjCPMGuiRes *ocpmg, char *name );
HashTable *CPMG_GetLayoutHashTable(void);

ObjDispRes *ObjCPMG_GetSubitemODR( ObjCPMGuiRes *ocpmg, char *item, char *subitem );
int ObjCPMG_SwapSubitemRes( ObjCPMGuiRes *ocpmg, char *item, char *subitem, char *resname );
int ObjCPMG_SetSubitemColor( ObjCPMGuiRes *ocpmg, char *item, char *subitem, float r, float g, float b );
int ObjCPMG_SetSubitemAlpha( ObjCPMGuiRes *ocpmg, char *item, char *subitem, float a );
int ObjCPMG_SubitemRunProgram( ObjCPMGuiRes *ocpmg, char *item, char *subitem, char *progname );


int CPMG_TestUpdateFunc( ObjCPMGuiRes *ocpmg );

void *ObjCPMG_GetData( ObjCPMGuiRes *ocpmg );

#ifdef __cplusplus
}
#endif

#endif
