// currency.h
// Play Mechanix - Video Game System
// Copyright (c) 2011 Play Mechanix Inc. All Rights Reserved
//
// JAL 25-MAY-2011: Implementing new currency system

#ifndef CURRENCY_H
#define CURRENCY_H

#include "g5.h"

typedef struct currency_t {
	char name_textID[LOCALIZATION_MAX_STR_LEN];				// Text identifier for the name of the currency
	char value_textID_singular[LOCALIZATION_MAX_STR_LEN];	// Text identifier for the value of the currency (this will contain proper formatting, with the sign) in singular grammatical form
	char value_textID_plural[LOCALIZATION_MAX_STR_LEN];		// Text identifier for the value of the currency (this will contain proper formatting, with the sign) in plural grammatical form
	int defBaseIncrementValue;								// Default amount that coin input values will be stepped up or down by (value will be converted to a float, divided by 100)
	int defCoin0Value;										// Default amount that coin input 0 is worth (value will be converted to a float, divided by 100)
	int defCoin1Value;										// Default amount that coin input 1 is worth (value will be converted to a float, divided by 100)
	int defCoin2Value;										// Default amount that coin input 2 is worth (value will be converted to a float, divided by 100)
	int defCoin3Value;										// Default amount that coin input 3 is worth (value will be converted to a float, divided by 100)
	int defPlayerStartCost;	// default for one credit (value will be converted to a float, divided by 100)
	int defContinueCost; // default for one continue (value will be converted to a float, divided by 100)
} currency_t;


// If adding or removing items here, remember that this MUST match CurrencyTab in currency.c
// AND you will need to adjust the min and max in the "currencyType" adjustment file!!!
typedef enum currency_type_e {	
	CURR_AUS_DOLLAR,
	CURR_POUND,
	CURR_RAND,
	CURR_NZ_DOLLAR,
	CURR_EURO,
	CURR_CAN_DOLLAR,
	CURR_FRANC,
	CURR_GUILDER,
	CURR_KRONA,
	CURR_PESO,
	CURR_REAL,
	CURR_RUBLE,
	CURR_WON,
	CURR_YEN,
	CURR_YUAN,
	CURR_COINS,
	CURR_TOKENS,
	CURR_SWIPE,
	CURR_US_DOLLAR,
	NUM_CURRENCY_TYPES,
} currency_type_e;

int  Currency_Init();
int Currency_RegisterPlayerCostChangeCB( int (*playerCostChangeCurrencyCallback)(currency_type_e new_curr_type, currency_type_e prev_curr_type) );

void FloatToCurrencyString(float amount, char *output, int outSize);
void FloatToCurrencyWideString(float currency, wchar_t *output, int outSize);
void CurrencyChangeType(currency_type_e new_curr_type, currency_type_e prev_curr_type);
void CurrencyChangeCoinDefs(currency_type_e new_curr_type, currency_type_e prev_curr_type);
void CurrencyChangeCoinBaseIncUnit(int new_base_unit, int old_base_unit);
void GetCurrencyName(wchar_t *output, int outSize);

#endif // CURRENCY_H

// EOF

