#ifndef FILE_ACCESS_H
#define FILE_ACCESS_H
// file_access.h
// Copyright 2011 Play Mechanix Inc. All Rights Reserved
//

#include "g5.h"

#define FILE_CRYPTO_ALG  CRYPTO_BLOWFISH 

// full filenames can be no longer than this
#define FILEACCESS_MAX_FILENAME_LEN	  256

#define FILEACCESS_FLAGS_PATHS_ARE_FILENAMES		0x00000001  // treat paths as full filenames, not dirs to be searched for files
#define FILEACCESS_FLAGS_NO_USE_ROOT_PATHS			0x00000002  // do not preprend root paths to type paths for this type
#define FILEACCESS_FLAGS_MERGE_METHOD_REPLACE		0x00000008  // when searching, files/dirs with same name replace each other, this is dflt
#define FILEACCESS_FLAGS_MERGE_METHOD_ADD			0x00000010  // when searching, files/dirs with same name are all added to results
#define FILEACCESS_FLAGS_DO_LOCALE_SEARCH			0x00000020  // search local subdir for files, does not stach with lang search
#define FILEACCESS_FLAGS_DO_LANGUAGE_SEARCH			0x00000040  // search lang subdir for files, does not stack with locale search
#define FILEACCESS_FLAGS_LOCALE_ONLY				0x00000080  // only use files from the locale specific folders
#define FILEACCESS_FLAGS_LANGUAGE_ONLY				0x00000100  // only use files from the language specific folders
// flags for loading/saving files, dflt is no checksum, compression or encruption
#define FILEACCESS_FLAGS_FILE_CHECKSUM				0x00010000  // load/save files w/ SHA1 checksum 
#define FILEACCESS_FLAGS_FILE_ENCRYPT				0x00020000  // load/save files w/ encryption
#define FILEACCESS_FLAGS_FILE_COMPRESS				0x00040000  // load/save files w/ compression
#define FILEACCESS_FLAGS_FILE_FORMAT_FLAGS			0x00070000  // needs to cover all the above bits 

#define	FILEACCESS_TYPE_G5				"G5"
#define FILEACCESS_TYPE_GLCGZ			"GLCGZ"
#define FILEACCESS_TYPE_GLCGZ2			"GLCGZ"
#define	FILEACCESS_TYPE_SND				"SND"
#define	FILEACCESS_TYPE_SHADERS			"SHADERS"
#define	FILEACCESS_TYPE_SHADERCACHE		"SHADERCACHE"
#define	FILEACCESS_TYPE_LOCALES			"LOCALES"
#define	FILEACCESS_TYPE_TRANSLATIONS	"TRANSLATIONS"

#define	FILEACCESS_TYPE_GAMELOG			"GAMELOG_FILE"
#define	FILEACCESS_TYPE_VERSION			"VERSION_FILE"

#define	FILEACCESS_TYPE_RTDT_HEADER_FILES	"RTDT_HEADER_FILES"
#define	FILEACCESS_TYPE_RTDT_HEADER_DIRS	"RTDT_HEADER_DIRS"


#define	FILEACCESS_TYPE_ADJ_DEF			"ADJ_DEF"
#define	FILEACCESS_TYPE_ADJ_LUT			"ADJ_LUT"
#define	FILEACCESS_TYPE_ADJ_SAVE		"ADJ_SAVE"

#define	FILEACCESS_TYPE_AUD_DEF			"AUD_DEF"
#define	FILEACCESS_TYPE_AUD_DEFAULT		"AUD_DEFAULT"
#define	FILEACCESS_TYPE_AUD_SAVE		"AUD_SAVE"
#define	FILEACCESS_TYPE_AUD_EXPORT		"AUD_EXPORT"


#define	FILEACCESS_TYPE_SSL				"SSL"
#define	FILEACCESS_TYPE_TELEMETRY		"TELEMETRY"

#define	FILEACCESS_TYPE_FOG				"FOG"
#define	FILEACCESS_TYPE_FUR				"FUR"
#define	FILEACCESS_TYPE_CAR				"CAR"
#define	FILEACCESS_TYPE_WATER			"WATER"
#define	FILEACCESS_TYPE_WIND			"WIND"
#define FILEACCESS_TYPE_WIND_NOISE      "WIND_NOISE"
#define FILEACCESS_TYPE_ENVCOL	        "ENVCOL"
#define FILEACCESS_TYPE_ANIACTION		"ANIACTION"



#define	FILEACCESS_TYPE_CPMGUI_IMGSETS	"CPMG_IMGSETS"
#define	FILEACCESS_TYPE_CPMGUI_LAYOUTS	"CPMG_LAYOUTS"

#define	FILEACCESS_TYPE_CONFIG			"CONFIG"
#define	FILEACCESS_TYPE_PERM_ASSETS		"PERM_ASSETS"
#define	FILEACCESS_TYPE_PERM_ASSETS_INIT		"PERM_ASSETS_INIT"
#define	FILEACCESS_TYPE_PERM_ASSETS_LANG	"PERM_ASSETS_LANG"

#define FILEACCESS_TYPE_MODEL_CONFS		 "MODEL_CONFS"
#define FILEACCESS_TYPE_TERRAIN_CONFS	 "TERRAIN_CONFS"
#define FILEACCESS_TYPE_HITFX_DAMAGE_TEMPLATES "HITFX_DAMAGE_TEMPLATES"

#define FILEACCESS_TYPE_ANIMLIST		"ANIMLIST"
#define FILEACCESS_TYPE_TRAVEL_ANIM		"TRAVEL_ANIM"
#define FILEACCESS_TYPE_DIE_ANIM		"DIE_ANIM"
#define FILEACCESS_TYPE_HIT_ANIM		"HIT_ANIM"

#define FILEACCESS_TYPE_POSTFX				"POSTFX"
#define FILEACCESS_TYPE_SCENE				"SCENE"
#define FILEACCESS_TYPE_SCENE_SWATCH		"SCENE_SWATCH"


#define FILEACCESS_TYPE_PROGRAM_DEC			"PROGRAM"
#define FILEACCESS_TYPE_PROGRAM_ENC			"PROGRAM_ENC"

// Release always uses program_enc, but debug mode chooses based on if we're testing dongle or not
#ifdef SYS_BB
#define FILEACCESS_TYPE_PROGRAM FILEACCESS_TYPE_PROGRAM_ENC
#else
#define FILEACCESS_TYPE_PROGRAM ((gWibuDongleActive || gMasterDongle) ? FILEACCESS_TYPE_PROGRAM_ENC : FILEACCESS_TYPE_PROGRAM_DEC)
#endif

#define FILEACCESS_TYPE_PROGRAM_TEMPLATE	"PROGRAM_TEMPLATE"

#define FILEACCESS_TYPE_PARTICLES			"PARTICLES"
#define FILEACCESS_TYPE_PARTICLES_SWATCH	"PARTICLES_SWATCH"

#define FILEACCESS_TYPE_MOTION_DOTS			"MOTION_DOTS"
#define FILEACCESS_TYPE_VID_COLS			"VID_COLS"

#define	FILEACCESS_TYPE_DIALOG				"DIALOG"

// There are five ways to use the system

// 1) single file
// 2) list of files
// 3) single path 
// 4) list of paths to be searched for files, filenames known
// 5) list of paths to be walked for subdirs/files


// how to return the results of file/dir search/walk funcs
enum
{
	FILEACCESS_RESULT_MODE_BASENAMES,
	FILEACCESS_RESULT_MODE_FULLPATHS,
	FILEACCESS_RESULT_MODE_SEARCH_RESULT_STRUCTS,
};


typedef struct FileAccessSearchResult
{
	char basename[FILEACCESS_MAX_FILENAME_LEN];
	char fullpath[FILEACCESS_MAX_FILENAME_LEN];
} FileAccessSearchResult;


int FileAccess_Init();
int FileAccess_Destroy();


int FileAccess_SetTypeFlags( char *type, uns32 flags );
int FileAccess_ClearTypeFlags( char *type, uns32 flags );


int FileAccess_AddRootPath( char *path );
int FileAccess_RemRootPath( char *path );

int FileAccess_RemTypePath( char *type, char *path );
int FileAccess_AddTypePath( char *type, char *path );
int FileAccess_TypePathsFree( HashNode *hn, void *stuff );

// returns most specific path of type, so 
int FileAccess_GetPath( char *type, char *path, int path_size );

// returns either result of FindFile or file prepended with GetPath result, use to query where to save a file
int FileAccess_GetFileSavePath( char *type, char *subdir, char *file, char *fullpath, int fullpath_size );


// these wrappers should be used to load files, only exception is G5s that are loaded w/ NBLoad stuff
int FileAccess_LoadFile( char *type, char *fullpath, void **data, int *data_len, void *(*malloc_func)(size_t size), int nullTerminate  );
// will load as flat txt, unicode txt, flat xml or wide xml, depending on file extension and first two bytes of file
int FileAccess_LoadFileRTDT( char *type,  char *fullpath, char *datatype, void *data );
int FileAccess_LoadFileHashTableStringByString( char *type,  char *fullpath, HashTable *ht, void *(*malloc_func)(size_t size) );
int FileAccess_LoadFileHashTableStringByStringMultiline( char *type,  char *fullpath, HashTable *ht, void *(*malloc_func)(size_t size) );

int FileAccess_SaveFile( char *type, char *fullpath, void *data, int data_len );
int FileAccess_SaveFileRTDT( char *type,  char *fullpath, char *datatype, void *data, int save_as_unicode );
int FileAccess_SaveFileHashTableStringByString( char *type,  char *fullpath, HashTable *ht );





int FileAccess_FindFile( char *type, char *subdir, char *file, char *fullpath, int fullpath_size );

int FileAccess_FindAndLoadFile( char *type, char *subdir, char *file, void **data, int *data_len, void *(*malloc_func)(size_t size), int nullTerminate  );



int FileAccess_FindFiles( char *type, char *subdir, int results_mode, List *files );
int FileAccess_WalkFiles( char *type, char *subdir, int results_mode, int (*OperationFunc)(char *, void *), void *data );








int FileAccess_GetDirList( char *type, char *subdir, int results_mode, List *dirs );


#endif // ndef FILE_ACCESS_H
