#ifndef FONT_H
#define FONT_H
// font.h
// Copyright 2004 Play Mechanix Inc. All Rights Reserved
//
// JFL 11 Oct 04


///////////////////////////////////////////////////////////////////////////////
// SYSTEM DEPENDENT

//-----------------------------------------------------------------------------
// SYS_JP5500
#if SYS_JP5500

#endif // SYS_JP5500
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// SYS_PC
#if SYS_PC|SYS_BB

#if SYS_BB
// drawing bitmap on linux side needs more space -- not sure why
#define FONT_SYSSPACE 2
#else // !SYS_BB
#define FONT_SYSSPACE 0
#endif // !SYS_BB

#endif // SYS_PC|SYS_BB
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
// SYSTEM INDEPENDENT

extern int FontInit(void);
extern void FontFinal(void);
extern int FontReset(uns reset);

extern int FontColor(uns32 rgb);
extern int FontSet(char *s1,char *s2);

int FontGetQuadCnt( ResDataDisp5* disp5 );
ResDataDisp5* FontGetResDataDisp5( ObjDispRes* odr );
void FontMoveChar( ResDataDisp5* disp5, ResDataDisp5* disp5_ref, int quad_indx, crd* offset );
void FontTransformChar( ResDataDisp5* disp5, ResDataDisp5* disp5_ref, int quad_indx, float* move, float* scl, float deg );
void FontScaleChar( ResDataDisp5* disp5, ResDataDisp5* disp5_ref, int quad_indx, crd* scale );
void FontUseCharColor( ObjDispRes* odr );
void FontSetCharColor( ResDataDisp5* disp5, int quad_indx, crd* rgba );

#define M_FONTGAMESTR_CREATE  0x01
#define M_FONTGAMESTR_NUMC    0x02 // pass in numc
#define M_FONTGAMESTR_HJUST0  0x04
#define M_FONTGAMESTR_HJUST1  0x08
#define M_FONTGAMESTR_VJUST0  0x10
#define M_FONTGAMESTR_VJUST1  0x20
#define M_FONTGAMESTR_HLEFT   (M_FONTGAMESTR_HJUST0)
#define M_FONTGAMESTR_HCENTER (M_FONTGAMESTR_HJUST1)
#define M_FONTGAMESTR_HRIGHT  (M_FONTGAMESTR_HJUST0|M_FONTGAMESTR_HJUST1)
#define M_FONTGAMESTR_VTOP    (M_FONTGAMESTR_VJUST0)
#define M_FONTGAMESTR_VCENTER (M_FONTGAMESTR_VJUST1)
#define M_FONTGAMESTR_VBOTTOM (M_FONTGAMESTR_VJUST0|M_FONTGAMESTR_VJUST1)
#define S_FONTGAMESTR_N 8 

// Font format, DEFAULT: HLEFT, VTOP
#define M_FONTFORMAT_HCENTER	0X01		// horizontal center
#define M_FONTFORMAT_HRIGHT		0X02		// horizontal right
#define M_FONTFORMAT_VCENTER	0X04		// vertical center
#define M_FONTFORMAT_VBOTTOM	0X08		// vertical bottom

//
// M_FONTSYSFLAG_
//	flags for usage in FontSys()
//
#define M_FONTSYSFLAG_NOBLEND 0x01

// Pixel width of the mono-spaced character rendered in the tool GUI
#define GUI_CHAR_WIDTH		8

typedef struct _FontGameStrInfo
{
    char    *s1;
    char    *s2;
    uns     flags;
    float   xScale;
    float   yScale;
    float   xPos;
    float   yPos;
    void    *ostr;
}FontGameStrInfo;

int FontBuildDisp5( Res *DispRes, char* identifier );

int FontGetBoundingBox( Res* DispRes, float *min, float *max  );
int ClipFont( ObjDispRes* dispRes, ObjClipRegion* clip_region, float* text_vm );
int ClipFontRecursive( Obj* obj_p, ObjClipRegion* clip_region );
void RenderClipRegion( ObjClipRegion* clip_region );

int ObjFontFit( ObjDispRes* dispRes, float width, float height  );
int ObjFontUpdateScrollBox( ObjDispRes* dispRes, int elapsed_time );

//extern int   FontGameStrEx(FontGameStrInfo * const pInfo);
//extern int   FontGameStr(char *s1,char *s2,uns flags,float xs, float ys, void *ostr);
//extern int   FontGameStrScale(char *s1,char *s2,uns flags,float xs, float ys, float xPos, float yPos, void *ostr);
//extern int   CalcFontGameWidth(char *s1,char *s2,uns flags,float xs, float ys, float *hsize, float *vsize, int lineNumber);
extern int   FontSys(char *str,int x,int y,int w,int h,uns flags,void *colobj);
#define      FontSysSimple(str,x,y) FontSys((str),(x),(y),VidDisp.screenW,VidDisp.screenH,0,NUL)
#define      FontSysSimpleFlags(str,x,y,flags) FontSys((str),(x),(y),VidDisp.screenW,VidDisp.screenH,(flags),NUL)
extern int   FontObjText(void *objtext);
//extern void* FontXYStr(float x,float y,char *str);
//extern int	 FontGameTranslateStr(uns32 colorARGB,char *str,uns flags,float xScale, float yScale, float xPos, float yPos, void **postr,...);
//extern int   FontGameUniStr(char *format,wchar_t *wcstr,uns flags,float xScale, float yScale, float xPos, float yPos, void **postr );


extern int FontSysX(unsigned char *str, int x, int y, int w, int h);
extern int FontSysXSClip(unsigned char *str, int x, int y, int w, int h, int cx, int cy, int cw, int ch);

// pass in 'l'=left-justify 'r'=right-justify 'h'=horiz-center
// pass in 't'=top-justify 'b'=bottom-justify 'v'=vert-center
// pass in 0x00RRGGBB
//extern void* FontXYJCStr(float x,float y,char hjust,char vjust,uns32 rgb,char *str);

enum {
	FONTINFO_NONE,
	FONTINFO_CHAR_WIDTH_HEIGHT, // int size[2];
};

//extern int FontObjInfo(void *objtext,int info,void *res);
extern int  FontObjInfo();
extern void FontConOutputStart(void);
extern void FontConOutputFinish(void);

extern unsigned char gNoisePool3[1024]; // used by dongle noise function

// 128x128 uncompressed image in GL_LUMA format,
// 32768 bytes, gzipped down to 962 bytes

static unsigned char fontsys_data[962] = {
0x78, 0x9C, 0xED, 0x98, 0xE1, 0x8E, 0xE3, 0x30, 0x08, 0x84, 0xFB, 0xFE, 0x2F, 
0xDD, 0xD3, 0xE9, 0x14, 0x69, 0x13, 0xCF, 0xC0, 0x80, 0x1D, 0x37, 0xD7, 0x9D, 
0xEF, 0xCF, 0x2A, 0x2C, 0xB6, 0x21, 0x26, 0x66, 0xDC, 0xD7, 0xCB, 0x18, 0xC6, 
0xFB, 0x8D, 0x2C, 0x9A, 0x15, 0x7B, 0x75, 0xD7, 0x45, 0x56, 0xCD, 0xAB, 0xBA, 
0x76, 0x3C, 0x8A, 0x65, 0x3A, 0xDA, 0xFE, 0x92, 0x79, 0x1D, 0x9E, 0x5A, 0xB4, 
0xCA, 0x1B, 0x3E, 0xD0, 0x66, 0xCC, 0x50, 0xF2, 0x47, 0x6B, 0xEA, 0x7B, 0x30, 
0xE7, 0x39, 0x37, 0x63, 0x8E, 0x36, 0x8A, 0xED, 0xFF, 0x68, 0x41, 0xD6, 0x99, 
0x58, 0x6A, 0xFB, 0x83, 0xEA, 0xA2, 0xBE, 0x26, 0xF6, 0xD2, 0xA3, 0x1B, 0x47, 
0xEA, 0xAB, 0xF4, 0xE2, 0xAB, 0x79, 0x56, 0xC1, 0xF5, 0xCF, 0xBE, 0xC1, 0xF1, 
0x4C, 0xA8, 0xBC, 0x65, 0xC5, 0xA6, 0x8F, 0x5E, 0x35, 0x4A, 0x3B, 0xFD, 0xB1, 
0xB5, 0x52, 0x97, 0xFD, 0x6F, 0xE7, 0xDE, 0xFD, 0x9F, 0xEB, 0x75, 0xB5, 0x95, 
0xEE, 0x5E, 0xC3, 0x18, 0x63, 0xEA, 0x20, 0xD5, 0x11, 0xFB, 0x30, 0x3F, 0xD6, 
0x11, 0x72, 0xDD, 0x79, 0xD8, 0x15, 0xBF, 0xB5, 0xE0, 0xBE, 0x9E, 0x6B, 0x02, 
0x34, 0x7A, 0xB4, 0x21, 0x9D, 0x50, 0xD5, 0xB9, 0xBB, 0xF3, 0x9F, 0x55, 0x05, 
0xA3, 0x07, 0xDE, 0xD7, 0xCC, 0x16, 0xAD, 0x1C, 0x51, 0x51, 0xCD, 0xAC, 0x06, 
0x95, 0xFC, 0x95, 0x9A, 0x88, 0x22, 0x41, 0xF7, 0xAE, 0x3D, 0x7A, 0x84, 0xCF, 
0xCE, 0xE2, 0xA8, 0xC4, 0xD5, 0xB9, 0x15, 0xEB, 0xEB, 0xA2, 0x79, 0xBA, 0xEF, 
0x88, 0x47, 0x9A, 0xC5, 0x11, 0xED, 0xE7, 0xDE, 0xFC, 0x75, 0x0D, 0xAE, 0xC5, 
0xC5, 0xBF, 0xCD, 0xBC, 0xAE, 0x6B, 0xE7, 0x7F, 0xB6, 0x2E, 0x8B, 0xB0, 0xEA, 
0xB1, 0x7E, 0xA4, 0x31, 0xC6, 0x3C, 0x81, 0x8A, 0x0E, 0xDF, 0x11, 0x0B, 0xB2, 
0xE5, 0xF1, 0x55, 0x7A, 0xC2, 0xD9, 0xE3, 0xFC, 0x97, 0xD9, 0xAA, 0x64, 0x63, 
0x79, 0x97, 0xC3, 0x5A, 0x3F, 0x8B, 0xAF, 0x9F, 0x07, 0x7A, 0x97, 0x68, 0x1C, 
0xEB, 0xCC, 0x8A, 0x86, 0x67, 0xBD, 0x1E, 0xC7, 0x33, 0x3E, 0x5D, 0xB5, 0xC3, 
0xE8, 0x89, 0x57, 0x40, 0xF3, 0xE3, 0xD5, 0xD4, 0xDB, 0x44, 0x3E, 0x5F, 0x65, 
0x14, 0xAA, 0xD8, 0xF1, 0xF9, 0x7A, 0x9F, 0xC2, 0x7A, 0x6A, 0x1C, 0xDF, 0xCF, 
0x5F, 0x1F, 0x8F, 0xE7, 0xEC, 0xEF, 0xBF, 0x96, 0xFF, 0xE8, 0xBB, 0x3E, 0xFF, 
0xBE, 0x2A, 0x56, 0x46, 0xAB, 0xEF, 0x56, 0xAF, 0x7F, 0xCD, 0x86, 0x56, 0xC0, 
0xE7, 0x26, 0xF2, 0x53, 0x6C, 0x33, 0xE4, 0xDF, 0xC3, 0x4F, 0x5B, 0xF6, 0xFD, 
0xDF, 0xDF, 0xC7, 0x76, 0xF4, 0xC8, 0xFE, 0x59, 0x64, 0x8C, 0x31, 0x11, 0xAC, 
0x33, 0x8F, 0xFA, 0x82, 0x7B, 0x9E, 0x2D, 0xE7, 0xBF, 0xCC, 0xEF, 0xEA, 0xCF, 
0xC6, 0xEA, 0xE7, 0x7F, 0x6D, 0xDD, 0xEE, 0x9A, 0xBD, 0x38, 0x78, 0xA7, 0x18, 
0xC7, 0xE6, 0x1A, 0xF6, 0x5F, 0x4E, 0xE3, 0xFE, 0xA8, 0xB1, 0xA0, 0xF9, 0x73, 
0x2D, 0xAD, 0xE9, 0x6B, 0x5C, 0x27, 0x7A, 0xFE, 0x6A, 0xFF, 0x47, 0x9A, 0x50, 
0xD1, 0x89, 0x6C, 0xFD, 0x99, 0xFD, 0xCF, 0xE6, 0x8B, 0x62, 0x40, 0x7B, 0xD8, 
0x59, 0x43, 0xD5, 0x89, 0xFA, 0x5C, 0x6A, 0x1C, 0xDA, 0x9A, 0xEC, 0xB9, 0xBB, 
0xFF, 0x2C, 0xE6, 0x4F, 0xEC, 0xFF, 0xCA, 0xFC, 0xB5, 0xEF, 0x5F, 0x5B, 0x57, 
0xCB, 0x5F, 0x3D, 0xFF, 0xF9, 0xE8, 0x2C, 0x36, 0x7C, 0x0E, 0x6B, 0x63, 0xA3, 
0x58, 0x56, 0x9C, 0xFF, 0xC6, 0x18, 0xF3, 0x5B, 0x51, 0x7F, 0x37, 0xA9, 0xFA, 
0x75, 0xCE, 0xEB, 0x39, 0x1B, 0x8F, 0xF0, 0x15, 0xB0, 0x22, 0x0E, 0x36, 0x67, 
0x67, 0xBE, 0x51, 0x03, 0x31, 0x55, 0xA4, 0xA9, 0x94, 0xAC, 0xFB, 0xCD, 0x69, 
0x09, 0xDC, 0x5F, 0xD7, 0xE6, 0xBF, 0x46, 0x8F, 0x33, 0xA2, 0x7B, 0x0D, 0xAA, 
0x2F, 0x3C, 0x87, 0xF6, 0xFC, 0xCC, 0xFC, 0xB1, 0xED, 0x1A, 0x07, 0xAB, 0x38, 
0x34, 0xC7, 0xFD, 0xF5, 0xAF, 0x57, 0x7A, 0x37, 0x7F, 0xAC, 0xC3, 0x15, 0xAD, 
0xAB, 0xE5, 0x1F, 0x65, 0xAA, 0x68, 0x78, 0xA5, 0x46, 0x22, 0x7B, 0xFC, 0xFF, 
0xE3, 0x17, 0x86, 0x2C, 0x0E, 0xED, 0xFB, 0xAF, 0xF8, 0xF5, 0xDF, 0x71, 0xED, 
0x77, 0x87, 0xDF, 0x82, 0xEF, 0x3F, 0xC6, 0x18, 0x46, 0x74, 0xFE, 0x6B, 0xBE, 
0x75, 0x9F, 0x8A, 0x5F, 0xCD, 0xBF, 0x73, 0xDA, 0xA9, 0xBA, 0x21, 0x52, 0x40, 
0xD9, 0x48, 0xB5, 0x2F, 0xA9, 0x1A, 0xB3, 0xD2, 0x77, 0xB3, 0xA8, 0x6B, 0x9A, 
0xB6, 0xDF, 0x4D, 0x90, 0xD6, 0x51, 0x7A, 0xBD, 0xAA, 0x45, 0xD8, 0x73, 0x5E, 
0x13, 0xDD, 0x5A, 0x57, 0xC1, 0x77, 0x8C, 0x51, 0x51, 0x46, 0x5A, 0x14, 0xDD, 
0x45, 0xB2, 0x9A, 0x50, 0xF7, 0xEC, 0xDE, 0xFC, 0xB9, 0xFA, 0xBD, 0x5A, 0xA3, 
0x38, 0x32, 0x3F, 0x64, 0xD7, 0x6B, 0x56, 0xBD, 0x4D, 0x74, 0xC0, 0x55, 0x58, 
0xB9, 0x13, 0x31, 0x1B, 0x5E, 0x29, 0x1E, 0xAB, 0x8D, 0x5C, 0x5F, 0xFF, 0xF8, 
0x04, 0xC3, 0x2A, 0x1E, 0xD9, 0xB4, 0xD1, 0x68, 0x65, 0xAD, 0x27, 0xE0, 0xAF, 
0xF3, 0x3B, 0xB8, 0x27, 0x97, 0xF7, 0x85, 0x3B, 0xD6, 0x98, 0xE5, 0xB9, 0x91, 
0x19, 0xF3, 0x04, 0x54, 0xF5, 0x8B, 0xAC, 0x95, 0xBB, 0xC3, 0x53, 0x41, 0x3A, 
0xB4, 0x32, 0x56, 0xB1, 0xE3, 0xFE, 0x87, 0xD6, 0xC5, 0x1A, 0x0E, 0xFB, 0x69, 
0x8A, 0x3A, 0xCB, 0x0D, 0xEB, 0xD0, 0xCA, 0xFE, 0xE7, 0xDA, 0xE1, 0x88, 0x15, 
0xAD, 0x91, 0x6B, 0x1D, 0xE6, 0x97, 0x2B, 0xEA, 0x4A, 0xDF, 0xC7, 0xEA, 0x0C, 
0xFB, 0xFE, 0x7C, 0x56, 0xB4, 0x33, 0xCE, 0x7F, 0xF4, 0x9D, 0xB1, 0xB1, 0x48, 
0xF2, 0xFD, 0x57, 0xE7, 0x62, 0x75, 0xAD, 0xF8, 0x7D, 0x2A, 0xFF, 0xEC, 0x3F, 
0xDF, 0xBD, 0xFF, 0x8A, 0xFE, 0x67, 0x73, 0xA9, 0x79, 0x31, 0xD5, 0x7D, 0xF5, 
0x53, 0xF2, 0xC7, 0xE7, 0x9F, 0x7E, 0x27, 0x40, 0x11, 0xAF, 0xE9, 0x46, 0x2C, 
0xD3, 0xFB, 0x51, 0xF6, 0x6E, 0x4F, 0x1C, 0x9F, 0xE9, 0xEA, 0x4F, 0xC9, 0xDF, 
0x98, 0x6F, 0xE4, 0x79, 0xDF, 0x50, 0xAE, 0x27, 0xD7, 0x45, 0x1D, 0xFF, 0x8E, 
0x54, 0xEF, 0xC3, 0x71, 0x67, 0xBE, 0xEA, 0x5A, 0xED, 0x37, 0x2C, 0xDD, 0xD6, 
0x21, 0x9A, 0xBB, 0xAB, 0x4D, 0x94, 0xF7, 0xA9, 0x5B, 0x9E, 0x9D, 0xFF, 0x98, 
0xAF, 0xAE, 0xC3, 0x66, 0x72, 0x65, 0xAA, 0xAB, 0xFA, 0x5E, 0x3E, 0x97, 0x3F, 
0xD3, 0x6B, 0x6A, 0x8C, 0x48, 0x63, 0xD6, 0xAB, 0x02, 0x29, 0xBB, 0xC3, 0x86, 
0x55, 0xEC, 0x68, 0xD1, 0xEA, 0xFF, 0x3C, 0x1F, 0x8B, 0x74, 0x77, 0xFD, 0xE3, 
0xB7, 0x86, 0x6B, 0x09, 0xEF, 0x6C, 0xAE, 0xEA, 0xD1, 0x7C, 0x4F, 0x39, 0xFF, 
0x76, 0xA2, 0xC4, 0xFC, 0x9D, 0xF9, 0x57, 0xEE, 0x61, 0x3B, 0xFB, 0xBF, 0x31, 
0xC6, 0x18, 0x63, 0x8C, 0x31, 0xC6, 0x18, 0x63, 0x8C, 0x31, 0xC6, 0x18, 0x63, 
0x8C, 0x31, 0xC6, 0x18, 0x63, 0xCC, 0xFF, 0xCC, 0x1F, 0x7C, 0x69, 0x23, 0x5C, 
};

extern int FontSysInit(void);
extern int FontSysQuit(void);

#endif // FONT_H

//EOF

