#ifndef GAME_CORE_H
#define GAME_CORE_H
// game_core.h
// Copyright 1998-2004 Play Mechanix Inc. All Rights Reserved
//
// JFL 06 Sep 04


#define M_GAMELP_PAUSE  0x0001
#define M_GAMELP_STEPF  0x0002
#define M_GAMELP_STEPB  0x0004
#define M_GAMELP_LOCK_STEP  0x0010 // forces GameLoopDelta to be RES_FRAME_TIME
#define M_GAMELP_FRAMEPAUSE 0x0020   // pauses after frame flip, doesn't let up until hit ctrl-s 

#if defined DEBUG
//#define PRINT_PERF_TIMER	1
#endif //defined DEBUG


#define GAMEWAVETIME_FRAMES(f) (GameWaveTime+((f)*RES_FRAME_TIME))




typedef struct {
	uns flags;
	uns64 pauseTime;
	uns32 stepTime;
} GameLpRec;


int GameCoreStart();
void GameCoreObjSetup();
int GameCoreLoop(int loop);

void GameCoreClockLoop();

int GameCoreWipeout(int disable_flip);
int GameCoreWipeoutDone(void);
int GameCoreWipeoutSetGameCB( void (*GameWipeoutCB)(void) ); // GameWipeoutCB will be called whenever a wipeout happens

void GameCoreResetGameWaveTime();
void GameCoreSetGameWaveTime(uns32 time);

// force a reboot now
void GameCoreReboot();
void GameCoreRebootLoop();

void GameCorePeriodicRebootOn(uns64 cycleTime, uns64 idleTime);
void GameCorePeriodicRebootOff();

void GameCoreTriggerRestartLevel();
void GameCorePerformLevelRestart();

void GameCoreFrameSpikeLoggingOn();
void GameCoreFrameSpikeLoggingOff();


EXTERN_C uns32 GameWaveTime; // milliseconds since the start of the wave
EXTERN_C float GameWaveTimef; // milliseconds since the start of the wave
EXTERN_C uns32 GameWaveTimeRaw; // milliseconds since the start of the wave with no game speed applied
EXTERN_C float GameLoopDeltaRaw;  // how long the since the last loop
EXTERN_C uns64 GameLoopClock;
EXTERN_C uns64 GameLoopClockSceneStart;
EXTERN_C float GameLoopDelta;  // how long the since the last loop (adjusted for game speed)
EXTERN_C float GameLoopDeltaGun;  // how long the since the last loop (adjusted for gun speed)
EXTERN_C float GameLastLoopDelta; // value for 'GameLoopDelta' from the last loop (who really needs this one?)
EXTERN_C float gGameSpeed; // how fast time moves for game (1.0=full speed, 0.5=half speed)
EXTERN_C float gGameGunSpeed; // how fast time moves for player gun and bullets (1.0=full speed, 0.5=half speed)
EXTERN_C GameLpRec GameLp;
EXTERN_C uns32 g_GameCoreRestartLevel;

#endif // ndef GAME_CORE_H
