#ifndef KEYBOARD_H
#define KEYBOARD_H

// keyboard.h
// Copyright (c) 2011 Play Mechanix Inc. All Rights Reserved

// --------------------------------------------------------------------------
// often-used defines
// --------------------------------------------------------------------------

// keyboard proc name for proc system
#define KEYBOARD_PROC	"KeyboardProc"

typedef int (*keyboard_validate_func_t)(wchar_t *input_buffer, void *param);
typedef void (*keyboard_exit_func_t)(void *kbd_state);

#define KB_SHIFT_AT_START					(1 << 0)
#define KB_SHIFT_AFTER_SPACE				(1 << 1)	// not implemented
#define KB_CAPSLOCK_AT_START				(1 << 2)
#define KB_ALTLOCK_AT_START					(1 << 3)
#define KB_TRIM_SPACE						(1 << 4)	// not implemented
#define KB_SOUNDS_DEFAULT_ONLY				(1 << 5)	// or turn off by setting volume to zero
#define KB_KILL_DISPLAY_AT_EXIT				(1 << 6)
#define KB_THROB_NEXT_ON_VALID_INPUT		(1 << 7)

#define KB_NO_KEYBOARD_FLAGS				0
#define KB_DEFAULT_NAME_FLAGS			(KB_SHIFT_AT_START | KB_SHIFT_AFTER_SPACE | KB_TRIM_SPACE | KB_SOUNDS_DEFAULT_ONLY)
#define KB_DEFAULT_INITIALS_FLAGS		(KB_CAPSLOCK_AT_START | KB_TRIM_SPACE | KB_SOUNDS_DEFAULT_ONLY)
#define KB_DEFAULT_EMAIL_FLAGS			(KB_TRIM_SPACE | KB_SOUNDS_DEFAULT_ONLY)
#define KB_DEFAULT_PASSWORD_FLAGS		KB_SOUNDS_DEFAULT_ONLY
#define KB_DEFAULT_NUMERIC_FLAGS		(KB_TRIM_SPACE | KB_SOUNDS_DEFAULT_ONLY)

enum
{
	KEYBOARD_NO_INPUT,
	KEYBOARD_ANALOG_INPUT,
	KEYBOARD_DIGITAL_INPUT,
};

enum
{
	KEYBOARD_DATA_ENTRY,	// data entry must be zero
	KEYBOARD_PREV,
	KEYBOARD_NEXT,
	KEYBOARD_CANCEL,
	KEYBOARD_DELETE,
	KEYBOARD_SHIFT,
	KEYBOARD_CAPS_LOCK,
};

#define KEYBOARD_INPUT_MAX		128

// current state of keyboard (shift pressed, caps locked, ...), initially set by keyboard_setup_data_t for each KeyboardStart
typedef struct _keyboard_state_data_s
{
	uns32						oid; // oid of currently selected item
	int							shift_pressed;
	int							caps_locked;
	int							alt_locked;
	uns32						kb_flags;
	wchar_t						*inbuf_ptr;
	wchar_t						input_buffer[KEYBOARD_INPUT_MAX];
	int							max_len;
	keyboard_validate_func_t	validate_func;
	void						*validate_params;
	int							exit_code;
	char						err_text_id[PM_NAME_SIZE];
	keyboard_exit_func_t		exit_func;
} keyboard_state_data_t;

// keyboard properties that change less frequently than those set by keyboard_setup_data_t
typedef struct _keyboard_properties_s
{
	char	select_sound[PM_NAME_SIZE];
	float	select_volume;
	char	hover_sound[PM_NAME_SIZE];
	float	hover_volume;
	int		input_type;
	int		select;			// select input
	int		cancel;			// future cancel input, right now assume layout has selectable exit button
	float	*x_pos;			// input digital x position
	float	*y_pos;			// input digital y position
	int		forward;		// analog forward input
	int		backward;		// analog backward input
	int		kill_display;
} keyboard_properties_t;

typedef struct _keybutton_properties_s
{
	char base[PM_NAME_SIZE];		// base info (initially format text id, then sound filename)
	char upper[PM_NAME_SIZE];		// text id for upper case
	char lower[PM_NAME_SIZE];		// text id for lower case
	char alternate[PM_NAME_SIZE];		// text id for alternate
} keybutton_properties_t;

typedef struct _keyboard_hover_info_s
{
	float coordinates[3];
	float hover_xscale;
	float hover_yscale;
} keyboard_hover_info_t;

// exposed keyboard functions
EXTERN_C int KeyboardIsActive(void);
EXTERN_C int KeyboardInit(void);
EXTERN_C int KeyboardStart(keyboard_setup_data_t *ksd);
EXTERN_C int KeyboardStop();
EXTERN_C int KeyboardWipeout();
EXTERN_C int KeyboardGetData(wchar_t *output, int outSize);
EXTERN_C int KeyboardGetErrTextId(char *output, int outSize);
EXTERN_C void KeyboardSetAnalogInput(int select, float *x_pos, float *y_pos);
EXTERN_C void KeyboardSetDigitalInput(int select, int forward, int backward);
EXTERN_C void KeyboardSetDisplayParams(int kill_display, float hover_xscale, float hover_yscale);
EXTERN_C void KeyboardSetSoundParams(char *select_sound, float select_volume, char *hover_sound, float hover_volume);
EXTERN_C uns32 GetKeyboardOID();

// canned validation functions
EXTERN_C int KeyboardValidateMinInput(wchar_t *input, void *params);
EXTERN_C int KeyboardValidatePhoneInput(wchar_t *input, void *param);
EXTERN_C int KeyboardValidateBdayInput(wchar_t *input, void *param);
EXTERN_C int KeyboardValidateEmailInput(wchar_t *input, void *param);
EXTERN_C int KeyboardValidatePostalCodeInput(wchar_t *input, void *param);
EXTERN_C int KeyboardValidateIpAddressInput(wchar_t *input, void *param);

#endif	// KEYBOARD_H
