#ifndef MOTIONPLATFORM_H
#define MOTIONPLATFORM_H
// motion_platform.h
//

#define MOTION_LEFTRIGHT_FLIP -1.0f

#define COMPRESSOR_ATTRACT_PURGE_TIME_MS ((4.0f) * (60.0f*60.0f*1000.0f)) // 4 hrs


// Taken from game.h so this will just work.
typedef enum CabType
{
	CAB_TYPE_NONE,
	CAB_TYPE_MOTIONDELUXE,
	CAB_TYPE_MOTIONBENCH,
	CAB_TYPE_ENVIRONMENTAL,
	CAB_TYPE_UPRIGHT,	
	CAB_TYPE_MOTIONTEST1,
	CAB_TYPE_NAMCOKIT,
	CAB_TYPE_UNIS_MOTION,

	NUM_CAB_TYPES

} CabType;


extern int g_enable_motion;
extern float g_motion_strength_multiplier;
extern float g_motion_pitch_multiplier;
extern float g_motion_tilt_multiplier;
extern float g_motion_height;
extern int g_enable_motion_vehicle;
extern float g_motion_base_height_pct;

void Motion_Initialize();
void Motion_UpdateConnection();
void Motion_UpdateMotion();
void MotionUpdate_DebugInput();
void Motion_SubmitMotion(double tilt, double pitch, double height, double jerk, int time);
void SetMotionPlatform(double tilt, double pitch, double height, double jerk, int time);
void SetMotionBaseHeightPct(float pct);
float GetMotionBaseHeight();
void SetMotionAngle(float angle, float radius, int time);
void SetMotionDirection(int direction, int hitstopper, int time);
void SetMotionPlatformCustom(char* name);
void SetMotionPlatformOneShot(char* name);
void SetMotionDefaultPosition();
void SetMotionContinueScreen();
void SetMotionContinueScreenResumed();
void SetMotionCompressorPurgeTime(float seconds);
void SetMotionCompressorPurge(int open);


// Diag functions ONLY!!! They ignore safety stops
void Motion_InflateSingleActuator(int actuator);
void Motion_DeflateSingleActuator(int actuator);
void Motion_InflateAll();
void Motion_DeflateAll();
void Motion_TurnOffSolenoids();

void  Motion_GetActuatorHeights(double* heights, double* targets);
int Motion_GetActuatorRawHeight(int actuatorNum);
float Motion_GetHeight();
float Motion_GetTilt();
float Motion_GetPitch();

float Motion_GetTargetHeight();
float Motion_GetTargetTilt();
float Motion_GetTargetPitch();

void Motion_ButtonStop();
int Motion_ButtonIsStopped();
int Motion_ButtonCanResume();
// resume motion if the cooldown period since the last stop is over.
void Motion_ButtonResumeManual();
// ignore the cooldown and force resume.
void Motion_ButtonResumeAuto();

void Motion_MatUpdate();
uns32 Motion_MatIsStopped();
uns32 Motion_MatIsPressed();

float Motion_GetPSI();

#if DEBUG
	void Motion_UpdateDbg();
#endif


extern int gMotionSelfTestState;
#define MOTION_SELF_TEST_RESULT_SUCCESS -1
#define MOTION_SELF_TEST_RESULT_FAILED -2
#define MOTION_SELF_TEST_RESULT_ABORTED -3
void Motion_UpdateSelfTest();

extern int gMotionCalibrateState;
#define MOTION_CALIBRATE_RESULT_SUCCESS -1
#define MOTION_CALIBRATE_RESULT_FAILED -2
#define MOTION_CALIBRATE_RESULT_ABORTED -3
void Motion_UpdateCalibrate();

#endif // ndef MOTIONPLATFORM_H
