#ifndef OBJ_H
#define OBJ_H

/*
#include <psm_engine/psm_engine.h>
#include "particles.h"
#include "vec.h"
#include <pmrt_utils/pmrt_utils.h>
*/

#define COLEPSILON 0.00001
#define COLEPSILON0G 0
#define COLEPSILON1L 1


// obj.h
// Copyright 2004 Play Mechanix Inc. All Rights Reserved
//
// JFL 08 Oct 04

///////////////////////////////////////////////////////////////////////////////
// SYSTEM INDEPENDENT

#ifdef __cplusplus
#define EXTERN_C	extern "C"
#else
#define EXTERN_C	extern
#endif

#define USE_OBJ_OID_TABLE	0

// if oids rolls over, start with this value instead of 0
#define OBJ_OID_ROLLOVER_VALUE 2000
// if reseting gNextObjectID on wipeout, use this value
#define OBJ_OID_WIPEOUT_RESET_VALUE 1000

// see ADDING NEW OBJ TYPES

enum {
	// must match resObjSubTable[], resObjSubSizes[]
	/* these two defined in pmrt_utils.h
	OBJSUB_NONE,                //0
	OBJSUB_DATA,                //1
	*/
	OBJSUB_CAM = 2,             //2
	OBJSUB_STR,                 //3
	OBJSUB_TEXT,                //4
	OBJSUB_FUNC,                //5
	OBJSUB_DISPRES,             //6
	OBJSUB_IMGRES,              //7
	OBJSUB_VIDRES,              //8
	OBJSUB_CPMGUIRES,			//9
	OBJSUB_SKELRES,             //10
	OBJSUB_COLRES,              //11
	OBJSUB_BUF,                 //12
	OBJSUB_LIGHTRES,            //13
	OBJSUB_MARKER,              //14
	OBJSUB_VEL,                 //15
	OBJSUB_COLACT,              //16
	OBJSUB_GROUP,               //17
	OBJSUB_EXECONTEXT,		    //18
	OBJSUB_PARTEMITTER,			//19
	OBJSUB_ATTACHMENT,			//20
	OBJSUB_TRANSFORM,			//21
	OBJSUB_FXGROUP,				//22
	OBJSUB_ANIACTION,			//23
	OBJSUB_PHYSJOINT,			//24
	OBJSUB_RTDTDATA,			//25
	OBJSUB_UVANIM,				//26
	OBJSUB_KAN	,				//27
	OBJSUB_WIND,				//28
	OBJSUB_TRACKANIM,			//29
	OBJSUB_GROUNDCOL,			//30
	OBJSUB_BONEACTION,			//31
	OBJSUB_CLIP_REGION,			//32
	OBJSUB_MOVIETEX,			//33
	OBJSUB_SCROLLINGLIST,		//34
	OBJSUB_VEL_CONTROLLER,		//35
	OBJSUB_FLYER,				//36

	OBJSUB_MAX
	// must match resObjSubTable[]
};

// careful about using M_N2 fields
#if M_N2_TYPE != 0x0f00
#error -- type uses 4 bits of flags
          these are flags the type can share with the N2 structure
		  o->lnk.flags
          you must be careful when using these flags to only use those
		  flags set in the M_N2_TYPE define
          search on M_N2 to find all uses
#endif // M_N2_TYPE


#define OBJ_LIST_DEPTH 24

#define OBJ_MAX_LIGHT_GROUPS 202
#define OBJ_SHAD_LIGHT_GROUP 201 // shadow light is used for all objs so we put it in a special group

#define M_N2OBJ_RESOLVED    0x0100 // shared using M_N2_TYPE
#define M_N2OBJ_PUSHPOPMAT  0x0400 // shared using M_N2_TYPE
#define M_N2OBJ_UPDATED		0x0800 // shared using M_N2_TYPE





#define OBJFLAGS_RENDERLISTED				  0x00000001  // obj has been added to a render list
#define OBJFLAGS_USEPARENTVM				  0x00000002  // start with the parent's transform when building a vm for this obj
#define OBJFLAGS_RENDERED					  0x00000004  // obj was actually rendered by some render list in the last frame
// next two flags are useful for odrs that have been put into groups (eg ogrp actors or cpmgui layouts)
#define OBJFLAGS_FORWARD_EVENTS_TO_PARENT	  0x00000008  // If obj recieves an event via ObjDoEventActions, event will be sent to parent obj instead
#define OBJFLAGS_USE_PARENT_FOR_COL_OBJ		  0x00000010  // If cola collides with obj, use cola as cobj instead of the obj
//#define OBJFLAGS_UPDATE_WHEN_PAUSED		     0x000000020  // update object even when game is paused

// base obj struct
// every obj struct must
// begin with these same elements
// use a define for this so it can be propagated easily..

// IF YOU CHANGE THIS, THIS HAS RAMIFICATIONS IN THE LIBRARY!!
// (AND THEREFORE ALSO ANY BIG BUCK GAME UNTIL WE FIX THIS!)
// because this is used by the definition of ObjData below,
// any changes will affect the library's definition of
// ObjData.  KPJ 06-apr-2009
#define OBJ_COMMON_FIELDS \
	N2 lnk; \
	uns32 oid; \
	uns32 oflags; \
	void *destructor; \

// all transforms get these fields also
#define TRANSFORM_COMMON_FIELDS \
	uns8  transform_type; \
	uns32 tflags; \
	VMType vm; // baked out transform for current frame... \



typedef struct Obj{
	OBJ_COMMON_FIELDS
} Obj;

typedef void (*ObjDestructorFunc)(Obj* vo);



enum M_OBJTRANSFORM_TYPES
{
	M_OBJTRANSFORM_TYPE_OBJECT, // apply transform to entire object
	M_OBJTRANSFORM_TYPE_JOINT, // apply transform to a joint
	M_OBJTRANSFORM_TYPE_UV0,  // apply transform to uv set 0
};

typedef struct {
	OBJ_COMMON_FIELDS
	TRANSFORM_COMMON_FIELDS
} ObjTransform;


/* defined in pmrt_utils.h
// IF YOU CHANGE THIS, THIS HAS RAMIFICATIONS IN THE LIBRARY!!
// (AND THEREFORE ALSO ANY BIG BUCK GAME UNTIL WE FIX THIS!)
// ObjData is defined in libraries/pmrt/pmrt_utils/node.h
// keep it up-to-date or it'll only lead to sadness.
// KPJ 06-apr-2009
typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	void *data;
} ObjData;
*/


#define OBJGROUP_SKIP		0x0001
#define OBJGROUP_CHARACTER	0x0002
#define OBJGROUP_STATIC_VM	0x0004

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	char showgroup[PM_NAME_SIZE]; // group's show group (decides if it will update its children)
	uns32 flags;
	float static_vm[VM_SIZE];
} ObjGroup;


// Hongwen camera
#define M_OBJCAM_USECONTROL  0x0001
#define M_OBJCAM_ORTHO       0x0002
#define M_OBJCAM_NOZ         0x0004
#define M_OBJCAM_USEGMOTION  0x0008
#define M_OBJCAM_SIMPLE      0x0010
#define M_OBJCAM_CLEARZ      0x0020
#define M_OBJCAM_RENDERTOAUX 0x0040
//#define M_OBJCAM_GHOSTCAM    0x0080
//#define M_OBJCAM_GHOSTSAVED  0x0100
//#define M_OBJCAM_USEVM       0x0200
#define M_OBJCAM_ROTATERIGHT 0x0400 // rotate camera to the right (for displaying on vertically mounted monitors, see also VidDisp flags)
#define M_OBJCAM_USESHAKE	0x0800
#define M_OBJCAM_USEOFFSET	0x1000	// use cam's kan position, but with offset (set with debug controls)
#define M_OBJCAM_STARTED	0x2000
#define M_OBJCAM_HAS_PREV_MAT	0x4000

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	Res *res; // if based on res
	char name[PM_NAME_SIZE];

	// set by user
	uns32 flags;

	float fov;
	float aspect_ratio;
	float viewNear;
	float viewFar;

	float orthoWidth;
	float orthoHeight;

	float x_clip_slope;
	float y_clip_slope;
	float clipNear;  // in theory same as viewNear/viewFar, but may change to get different culling/rendering...
	float clipFar; // in theory same as viewNear/viewFar, but may change to get different culling/rendering...

	float aperture[2]; // aperture width & height
	float lens;	// focal length

	crd vm[VM_SIZE]; // cameras view matrix, camera space to world space
	crd ivm[VM_SIZE]; // inverse view matrix, world space to camera space
	crd vmOrig[VM_SIZE]; // saves vm without control transform
	crd ivmOrig[VM_SIZE];
	crd m4[M4_SIZE]; // cameras view matrix as an gl ready m4
	crd m4Orig[M4_SIZE];
	crd proj[M4_SIZE]; // cameras projection matrix
	crd iproj[M4_SIZE]; // inverse cameras projection matrix
	crd projmv[M4_SIZE]; // full on projection model view matrix
	crd projmvOrig[M4_SIZE];
	crd projmvPrev[M4_SIZE]; // from previous frame

	float view_vec[3]; // cam space (0,0,-1) converted into world space
//	crd shake[3];
//	crd gunmotion[3];

	float tvel[3];  // this is computed based on the cam's spline

	int auxTarget;	// which aux target to render to if M_OBJCAM_RENDERTOAUX is set

	int lookAt_oid; // oid of the obj we are forcing cam to look at
	int lookAt_bone;

	QuatType lookAt_fromQuat; // last quat orientation used in non-lookAt frame
	QuatType lookAt_lastQuat; // last quat orientation used in a lookAt frame
	float lookAt_percent;	  // percent camera has switched from not looking at target to looking at target (also used when moving from looking to not looking)
	float lookAt_slerpTime;	  // time camera should spend switching from looking at to not looking at
	float lookAt_smoothing;	  // smoothing parameter when camera is looking at target (0.94 - 0.97 is a good range)


} ObjCam;

/*
#define M_OBJSTR_HASALPHA	0x01
#define	M_OBJSTR_DONTRENDER	0x02
typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	uns8 flags;
	uns8 xx1;
	uns16 xx2;
	uns32 texGen;
	float user[4];
	float xyadj[2];
	float wh[2]; // width & height of string in camera coords
	float scale[2];
	uns16 numc;
	uns16 maxc;
	// followed by ObjStrChar
} ObjStr;

typedef struct {
//	uns32 rgb;
	float rgba[4];
	float user[2];
	float xyadj[2];
	float uvbox[BOX_SIZE]; // top left right bottom
	float verts[3*2]; // x0,y0,z0 .. x1,y1,z1 (topleft bottomright)
} ObjStrChar;
*/

#define M_OBJLIGHTRES_SHADDIR 0x0001
#define M_OBJLIGHTRES_SHADPOS 0x0002
#define M_OBJLIGHTRES_NOLIGHT 0x0004
#define M_OBJLIGHTRES_NOSPEC  0x0008
#define M_OBJLIGHTRES_NODIFF  0x0010
#define M_OBJLIGHTRES_AUTOANI 0x0020  // go look for a kan to match with this
#define M_OBJLIGHTRES_SHADINTENSITY 0x0040
#define M_OBJLIGHTRES_FLASHLIGHT	0x0080
#define M_OBJLIGHTRES_LIGHTNING		0x0100 // light color gets multiplied by lightning intensity

#define MAX_FLASHLIGHTS	1


typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	char group[PM_NAME_SIZE];

	uns8 setup; // has this light been setup yet?
	uns type;  // we re-use RESDATALIGHT_TYPES for this
	uns lightgroup;  // what light group is it part of?
	Res *res; // if based on res
	uns flags;

	uns32 spec_light_oid; // sometimes we use a seperate light for spec, if so here is the oid of that light...

	// initial values
	float base_color[4];
	float intensity;
	float shad_intensity;

	// current values
	float color[4];
	float pos[3];
	float dir[3];

	float spot_cone;
	float spot_exponent;

	float constant_attenuation;
	float linear_attenuation;
	float quadratic_attenuation;

	int static_pos; // recompute our pos based on oar/ov/objattach every frame??
	// shadow light stuff

	int flashlightNum;

	uns32 last_blink_time;
	uns32 next_blink_time;
	uns32 blink_period;
	uns32 blink_on_time;
	uns32 blink_fade_time;
	float blink_min;
	int blink_random;

} ObjLightRes;


typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	char *str;
	int16 box[4];
	uns16 flags;
	uns16 n;
	uns16 size;
} ObjText;

#define M_OBJFUNC_RENDERLOOP			0x000001 // run func during renderloop instead of update loop, use for functions that make direct GL calls

typedef void (Func_FreeFunc)(void* data);
typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	void *func; // pointer to function	
	uns8 ftype; // type of function -- see pm.h ftype
	uns32 flags;
	uns8 k;
	uns16 n;
	void *d;
	Func_FreeFunc *freeFunc;
} ObjFunc;

#define M_OBJDISPRES_SKELCOL		0x00000001 // collide against skelcol geometry
#define M_OBJDISPRES_PROPCOL		0x00000002 // collide against propcol geometry
#define M_OBJDISPRES_2DCOL			0x00000004 // collide against 2D geometry

#define M_OBJDISPRES_AUTOANI		0x00000008 //  automatically attach anim
#define M_OBJDISPRES_AUTOCOL		0x00000010 //  automatically attach col

#define M_OBJDISPRES_SHADCAST		0x00000020 // cast shadows
#define M_OBJDISPRES_SHADRECV		0x00000040 // recv shadows
#define M_OBJDISPRES_REFLECT		0x00000080 // cast reflections
#define M_OBJDISPRES_PRELIGHT		0x00000100 // render using the prelight buffer
#define M_OBJDISPRES_FUR			0x00000200 // render using the prelight buffer

#define M_OBJDISPRES_DONTRENDER		0x00000400 // update but do not render this obj, useful for hiding or manually culling obj
#define M_OBJDISPRES_DISPGRPPART	0x00000800 // are we part of a disp group?
#define M_OBJDISPRES_RENDERFIRST	0x00001000 // force object to start of line in render list sorts (useful for ground or sky objs that you want to render first despite location...)
#define M_OBJDISPRES_RENDERLAST		0x00002000 // force object to end of line in render list sorts (useful for ground or sky objs that you want to render last despite location...)
#define M_OBJDISPRES_USEOBJLIGHTS	0x00004000 // check obj's kid list for extra lights to use when rendering this obj

#define M_OBJDISPRES_USESKELETON	0x00008000 // use attached obj skel res 
#define M_OBJDISPRES_ANIMATEUVS		0x00010000 // build and use transform matrix for uvs
#define M_OBJDISPRES_SHADGEO		0x00020000 // obj is for casting shadows only, don't render in shad recv and normal render passes...
#define M_OBJDISPRES_OUTLINE		0x00040000 // add 'glow' outline to object
#define M_OBJDISPRES_OUTLINEPARTS	0x00080000 // add 'glow' outline to one or more of the object's parts (disp grp only)
#define M_OBJDISPRES_CULL			0x00100000 // do cull test on this object
#define M_OBJDISPRES_REFLECTCUBE	0x00200000 // reflect cube map
#define M_OBJDISPRES_ENEMY			0x00400000 // For debug rendering of enemies
#define M_OBJDISPRES_BONEFX			0x00800000 // update bonefx children

#define M_OBJDISPRES_DYN_MOTION_BLUR	0x01000000 // treat the object as moving, not static, for motion blur
#define M_OBJDISPRES_PARTICLE			0x02000000 // created by a particle emitter
#define M_OBJDISPRES_WIREFRAME			0x04000000 // draw as wireframe
#define M_OBJDISPRES_FRIENDLY			0x08000000 // this is a friendly character

#define M_OBJDISPRES_FONTOBJ		0x10000000 // this is a font obj, res is mallocd with obj, free it's data on obj free
#define M_OBJDISPRES_CPMGOBJ		0x20000000 // this is a cpmg obj, res is mallocd with obj
#define M_OBJDISPRES_FNT_NO_REBUILD	0x40000000 // dont rebuild this font disp res every frame



enum ObjDispResTypes
{
	M_OBJDISPRES_TYPE_SINGLE,  // a single disp1/2/3 obj
	M_OBJDISPRES_TYPE_DISP_GROUP,   // a group of disp3 (skel) objs
};



#define M_OBJDISPRES_MAX_TEXTURES 8
#define M_OBJDISPRES_MAX_VERTDATA 8
#define M_OBJDISPRES_MAX_UNIFORMDATA 8

typedef struct RenderDataPtr
{
	int count;
	int stride;
	float *data;
} RenderDataPtr;

typedef struct {
	char name[PM_NAME_SIZE];
	int type;
	int count;
	void *values;
} RenderDataShaderUniformData;


/*
enum
{
	CBOX_FUR, // front upper right
	CBOX_FUL, // front upper left
	CBOX_FLR, // front lower right
	CBOX_FLL, // front lower left

	CBOX_BUR, // back upper right
	CBOX_BUL, // back upper left
	CBOX_BLR, // back lower right
	CBOX_BLL, // back lower left
};
*/

// for shader115, swipe on/off
typedef struct ObjRenderData_Swipe_Params
{
	float swipe_center[2];
	float swipe_pad[2];
	int   swipe_dir[2];
} ObjRenderData_Swipe_Params;


// structs for storing obj render data
// sometimes this struct just points to resdata (disp2)
// other times it points to a malloc'd block of mem
// where processed data is stored (disp3)
typedef struct ObjRenderData
{
	uns32 type;  // GL_QUADS, GL_TRIANGLES, etc...
	int count; // count as expected by glDrawArrays or glDrawElements...
	int vert_count; // how many unique verts are there
	int use_indices; // render with glDrawElements and rd->indices, instead of glDrawArrays
	int use_dlist; // build/use dlist for rendering this obj
	int use_vbo; // use vert buffer obj for rendering
	int use_vio; // use vert buffer obj for element indices when rendering
	int vbo;
	int vio;
	uns32 vert_arrays; // VERT_ATTRIB_MASK_ARRAY0 ...

	// vert data pointers
	RenderDataPtr verts;
	RenderDataPtr vcolors;
	RenderDataPtr normals;
	RenderDataPtr tangents;
	RenderDataPtr uvs[M_OBJDISPRES_MAX_TEXTURES];
	RenderDataPtr vertdata[M_OBJDISPRES_MAX_VERTDATA];
	RenderDataPtr joint_weights;
	RenderDataPtr joint_indices;
	RenderDataShaderUniformData uniformdata[M_OBJDISPRES_MAX_UNIFORMDATA];
	int *indices;
	float *color; // used if vcolors is null

	// buffers that store various vert data
	// if we malloced our own copy of indices and vert datas then here are the ptrs for that
	int *indices_buf; 
	uns32 indices_buf_size;
	uns8 *data_buf;
	uns32 data_buf_size;

	// sorting flags...
	int facesort;
	int meshsort;

	// scratch space for sorting...
	Disp5_Mesh *meshsort_buf;
	Disp5_Face *facesort_buf;



	// object data
	float vm[VM_SIZE]; // obj's model view matrix in VM format
	float m4[M4_SIZE]; // obj's model view matrix in GL format
	int   use_vm;      // if 0 then mvm is ignored (use for obj's with mvm = identity)
	int cull;  // attempt to cull obj
	float xyzrCull[4]; // obj center and radius for culling
	float rCullScale; // how much to scale culling radius by (largest of obj's xyz scales)

	// save the model-to-world transform applied in the previous frame.
	// used for motion blur.
	float prevMat[M4_SIZE];
	int hasPrevMat;

	float line_width; // if type GL_LINES/LINE_STRIP then how many pixels wide to draw it

	// for now we only allow one universal texture vm
	int			  use_uv_vm;
	float		  uv_m4[M4_SIZE];
	char		  uv_mat_buffername[32];

	uns32		  tex[M_OBJDISPRES_MAX_TEXTURES];

	uns32 cubeTex;
	// rotates the world-space vector used to sample the cubemap
	float cubeRotateSpeed;


	float Material_SpecularShininess; // spec exponent
	float Material_Reflectivity; // lerp between diffuse texture and environment reflection

	// various render mode options...
	int fog;			// use fog when rendering object?
	int blend;			// use alpha blending when rendering...
	int additive;		// use additive blending
	// New
	int depthmask;		// use depth mask when rendering
	int depthtest;
	int billboardy;		// use y-axis billboarding when rendering object
	int billboard;		// regular billboarding not constrained to an axis
	int doublesided;	// render w/ cull face disabled
	int vertcolors;     // if available, use vert colors for this obj

	// shader7 special params
	char furparams[32];
	// shader112 params
	char waterparams[32];
	// shaderCar params
	char carparams[32];

	// bloom override
	float bloom_threshold;

	// shader 9 Hongwen
	float para_height;
	float para_bias;

	// shader115
	ObjRenderData_Swipe_Params swp;

	// shader116
	int   clip_world_space; // what space are origin, size, etc in?
	float clip_origin[3];
	float clip_size[3];
	float clip_border[3];

	char skelmats_buffername[32];
	char skelmatsprev_buffername[32];

	// face sort stuff
	float facesort_resort_threshold; // cos of threshold angle for resorting

}ObjRenderData;


typedef struct
{
	int active;
	int axis;
	float clip_size;
	float border_size;
	float speed;
	int   pause_delay;
	int   pause_counter;
	float pos;
	
	// final result (copy to rd if needed)
	float origin[3];
	float size[3];
	float border[3];
	int   no_export_scroll_to_rd;

} FontScrollParams;


typedef struct {
	OBJ_COMMON_FIELDS		// see above for this define

	char name[PM_NAME_SIZE];
	uns8 type;

	uns8 lightgroup;		// what light group to use when rendering 

	char group[PM_NAME_SIZE];

	Res *res;
	void *instance_res;		// if we are an instance obj, here is the the instance res 
	uns32 flags;
	uns8 tag;
	uns32 gameFlags;		//game-side use, defined in game_app.h

	// LOD stuff
	Res *lod_res;
	float lod_distsq;
	uns32 swapTexNum;	// Save off texture and texture number if swapped so LOD knows to do that too.
	uns32 swapTexGen;

	float rgba[4];

	uns8 curBlend;
	uns32 dList;			// display list index
	uns32 dListVertsOnly;	// display list for shad map render passes (only sends verts)

	uns32 physxStatus;		// 0 - not in physx engine, 1 - static physx object, 2 - dynamic physx object

	Res *shader;			// what shader resource to use when displaying this object
	Res *shader_at_create;	// what shader resource was assigned when obj was created (used to restore shader if swapping it for effect)

	ObjRenderData rd;		// compiled info for rendering obj
	void *rd_buf;			// buffer for storing verts, uvs, etc, if required (disp3 objs)
	uns8  rd_setup;
	int	 static_vm;			// vm should not be updated every frame 
	uns32 last_vm_update;	// gamewavetime at last vm update, let's one know if vm is up-to-date or not...
	uns8 renderlisted;

	float outline_size;
	float outline_rgba[4];
	float inside_rgba[4];

	float glow_threshold;	// threshold to use if/when rendering to the glow buffer...
	float glow_intensity;

	// font stuff
	float font_width;
	float font_height;
	FontScrollParams scroll_params;

	float sort_pri; // used to order rendering alpha blended objects

	TravelAnimCollection* travelAnimOverride;

	// current rotation around the y-axis (used with the cubemap)
	float curCubeRot;

	int envmapVertAlpha;


//	float CullBox[8][3];

} ObjDispRes;

// Clip region currently only clips text (DispRes text)
typedef struct {
	OBJ_COMMON_FIELDS

	uns32 target_oid;			// The ID of the object (or obj group) to be clipped
	float clip_box_nw[V3_SIZE];	// Northwest corner of the clip region (xyz)
	float clip_box_se[V3_SIZE];	// Southeast corner of the clip region (xyz)

} ObjClipRegion;


// Scrolling List 
// works with shader116 to clip list and soft blend edges of scroll
// in general, list obj should be a sibling of the group/obj to be scrolled
enum {
	OSL_LOOP_BEHAVIOR_BOUNCE,
	OSL_LOOP_BEHAVIOR_SNAP,
	OSL_LOOP_BEHAVIOR_STOP,
};

enum {
	OSL_SCROLL_STATE_NORMAL,
	OSL_SCROLL_STATE_FROZEN,
	OSL_SCROLL_STATE_2X,
	OSL_SCROLL_STATE_3X,
	OSL_SCROLL_STATE_HALF_SPEED,
};


typedef struct {
	OBJ_COMMON_FIELDS

	uns32 target_oid;			// The ID of the object (or obj group) to be scrolled
	float clip_box_origin[V3_SIZE];
	float clip_box_size[V3_SIZE];
	float clip_box_alpha_border[V3_SIZE];

	int scroll_state; // NORMAL, FROZEN, 2X, etc.
	float scroll_speed[V3_SIZE]; // same units as an obj vel (units/millisec)
	int scroll_pause_axis; // 0-2, x,y,z
	int scroll_pause_interval; // pause every so many units moved
	int scroll_pause_offset; // 
	int scroll_pause_delay; // how long to pause for (in ms)
	int scroll_pause_last_pos;
	int scroll_pause_timer; // how long left in our pause
	int (*scroll_pause_repopulate_cb)(void *osl);

	int scroll_loop_count; // loop after so many intevals moved
	int scroll_loop_behavior; // OSL_LOOP_BEHAVIOR_BOUNCE, etc.
	int scroll_loop_interval_counter; // how many intervals have gone by since last loop


	// char loop_trigger[TRIGGER_NAME_SIZE]; // name of trigger to call whenever we 

//  int max_viewable_objs;
//  int (*add_obj_callback)(ObjScrollingList *osl, int obj_num)


} ObjScrollingList;


enum
{
	M_OBJGLCGZ_PLAYMODE_PLAYING,
	M_OBJGLCGZ_PLAYMODE_PAUSED,
	M_OBJGLCGZ_PLAYMODE_COMPLETE,

};

#define M_OBJGLCGZ_UPDATE_GLTEX		0x0001
#define M_OBJGLCGZ_UPDATE_IN_THREAD	0x0002

typedef struct
{
	OBJ_COMMON_FIELDS

	uns32 texGen;
	uns32 glcgzFlags;
	glcgz2_file_type *fileStream;
	int				playMode;
	int				loop;
	int				killWhenDone;
	float			speedFactor;
} ObjGLCGZTex;

typedef struct {
	
	OBJ_COMMON_FIELDS
	Wind wind;
	VMType *parent_transforms;
	uns32 parent_transforms_size;

} ObjWind;

typedef struct {

	OBJ_COMMON_FIELDS
	FlyerGroup flyerGroup;
	VMType *parent_transforms;
	uns32 parent_transforms_size;

} ObjFlyer;

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	// note that 'lnk' for this obj type is not a permanent link -- changes every frame, not initialized
	void *data;
} ObjTmpLink;

#define M_OBJDISPSAVE_SET    0x0001 // stuffed with valid data
#define M_OBJDISPSAVE_BIND   0x0002 // concatenate bind matrix
#define M_OBJDISPSAVE_SINGLE 0x0004 // single mat

/*
typedef struct {
	N2 lnk;
	uns32 oid;
	// end common obj fields

	char name[PM_NAME_SIZE];

} ObjDispGrp;
*/

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	// used to save info about draw state
	// currently only used for saving skeleton joint matrices
	// but can be expanded to save any info about the draw

	uns16 flags;
	uns16 numJoints;
	float *bindMats;
	float *jointMats;

} ObjDispSave;

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	void *res;
} ObjImgRes;

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	void *res;
} ObjVidRes;

/*
#define M_OBJANIRES_LOOP       0x00000001
#define M_OBJANIRES_DONTLOOP   0x00000002
#define M_OBJANIRES_FREEZENEXT 0x00000004
#define M_OBJANIRES_FROZEN     0x00000008
#define M_OBJANIRES_DONE       0x00000010
#define M_OBJANIRES_ENDKILL    0x00000020
#define M_OBJANIRES_KILLPRIME  0x00000040 // kill prime obj (parent) on exit
#define M_OBJANIRES_ROOT       0x00000080
#define M_OBJANIRES_SKEL       0x00000100
#define M_OBJANIRES_GCOL       0x00000200
#define M_OBJANIRES_NOSCALE    0x00000800
#define M_OBJANIRES_NOPITCH    0x00001000
#define M_OBJANIRES_SNAPONCE   0x00002000
#define M_OBJANIRES_BLENDONCE  0x00004000
#define M_OBJANIRES_SYNC       0x00008000
#define M_OBJANIRES_BRIDGE     0x00010000
#define M_OBJANIRES_MOVED      0x00020000 // not a user flag
#define M_OBJANIRES_BLEND2ADD  0x00040000 // not a user flag
#define M_OBJANIRES_CHANGE     0x00080000 // not a user flag
#define M_OBJANIRES_HITEND     0x00100000 // anim looped, cleared when changing
#define M_OBJANIRES_FROZENTIME 0x00200000
#define M_OBJANIRES_SKIPTO     0x00400000
#define M_OBJANIRES_BLENDING   0x00800000
#define M_OBJANIRES_BLEND2     0x01000000
#define M_OBJANIRES_SLEEPCB    0x02000000
#define M_OBJANIRES_JUMP       0x04000000
#define M_OBJANIRES_TAPPED	   0x08000000

enum {
	OAR_CH_RX, // assumed 1st for now
	OAR_CH_RY,
	OAR_CH_RZ,
	OAR_CH_TX,
	OAR_CH_TY,
	OAR_CH_TZ,
	OAR_CH_SX,
	OAR_CH_SY,
	OAR_CH_SZ,
	OAR_CH_N
};

#define M_OAR_CH_R ((1<<OAR_CH_RX)|(1<<OAR_CH_RY)|(1<<OAR_CH_RZ))
#define M_OAR_CH_T ((1<<OAR_CH_TX)|(1<<OAR_CH_TY)|(1<<OAR_CH_TZ))
#define M_OAR_CH_S ((1<<OAR_CH_SX)|(1<<OAR_CH_SY)|(1<<OAR_CH_SZ))

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	TRANSFORM_COMMON_FIELDS // see above for this define

	// top three are preserved when you clear out the OAR
	char name[PM_NAME_SIZE];
	uns32 flags;
	uns32 joint_index; // which skel joint is this oar for

	void *res; // points to resource

	crd   cur[9]; // rx,ry,rz,tx,ty,tz,sx,sy,sz
	crd   prev[9]; // rx,ry,rz,tx,ty,tz,sx,sy,sz for the previous frame...

	float aniTime0; // time base for start of ani
	float speed; // multiplier -- 0==1
	float skipAdj;
	int   blend_count; // number of time ani has been changed w/o completing blend (used to identify compound blending)

	// primary animation
	void  *kan_res; // points to res containing the kan
	void  *kan;
	void  *key0; // source 'from'
	void  *key1; // target 'to'
	float fcGameAcc; // accumulated length
	float fcGameLen; // speed-adjusted length of current segment
	float fcGameInv; // 1/sofarLen
	float fcAnimAcc; // 
	float fcAnimLen; // original segment length
	float fcLoopAcc;
	float fcLoopCount;
	float frameCount; // updated every loop with current/transient value

	// blend animation
	void  *abKan_res; // points to res containing the kan
	void  *abKan;
	void  *abKey0;
	void  *abKey1;
	float abTime0;
	float abGameAcc;
	float abGameLen;
	float abGameInv;
	float abBlendInv;
	float abBlendTime0;

	// second blend animation
	void  *b2Kan_res; // points to res containing the kan
	void  *b2Kan;
	void  *b2Key0;
	void  *b2Key1;
	float b2Time0;
	float b2GameAcc;
	float b2GameLen;
	float b2GameInv;
	float b2BlendInv;
	float b2BlendTime0;

	crd pitch; // shouldn't need extra field
	crd rmax; // rot max per millisecond
	float aTime; // animation time parameter
	float bTime; // animation time parameter
	float aRamp; // animation ramp parameter

	void *nextKan_res; // points to res containing the kan
	void *nextKan;

} ObjAniRes;


enum 
{
	M_OBJANICHAN_MODE_ADD_ROTS,
	M_OBJANICHAN_MODE_ADD_TRANS,
	M_OBJANICHAN_MODE_ADD_ROTSANDTRANS,
	M_OBJANICHAN_MODE_BLEND,

};

enum{	M_OBJANICHAN_BLEND_LINEAR_INTERP_MODE,	M_OBJANICHAN_BLEND_COSINE_INTERP_MODE};typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	char name[PM_NAME_SIZE];
	int channel;
	uns32 flags;

	int blend_mode;
	int blend_interp_mode;	float blend_start;
	float blend_end;
	float blend_cur_time;
	float blend_end_time;
	float blend;

} ObjAniChannel;
*/

#define M_OBJKEYANIM_LOOP									0x00000001
#define M_OBJKEYANIM_PAUSE									0x00000002
#define M_OBJKEYANIM_KILLPRIME								0x00000004
#define M_OBJKEYANIM_GNDCOL									0x00000008
#define M_OBJKEYANIM_GNDCOL_BEGUN							0x00000010
#define M_OBJKEYANIM_NO_ROT									0x00000020

enum
{
	OBJKEYANIM_TYPE_SPLINE,
	OBJKEYANIM_TYPE_SKELETON,
};


typedef struct KeyAnimPos
{
	uns32 joint;
	char name[PM_NAME_SIZE]; // name of joint
	Res *res; // RESSUB_KAN
	uns32 curKey; // save index of current key
	// current position
	float trans[3];
	float rot[3];
	float scale[3];
	float pitch; // used for ground col...
//	VMType vm;
} KeyAnimPos;

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	TRANSFORM_COMMON_FIELDS // see above for this define

	char name[PM_NAME_SIZE];
	uns32 type;
	uns32 flags;
	timed_blend speed_blend;
	float speed;
	float anim_len;
	float time;	
	float frame; // total frame number since start of anim
	float last_frame;
	float loop_time;
	float loop_frame; // frame number since start of last loop
	float loop_last_frame;
	float gndcol_pivots[2]; // how far ahead/back to look when calc gndcol
	Res *res;  // RESSUB_GRP | RESSUB_KAN

	uns32 num_kans;
	KeyAnimPos *kans;

} ObjKeyAnim;


enum 
{
	M_ANIMTRACK_MODE_BLEND,
	M_ANIMTRACK_MODE_ADD_ROTS,
	M_ANIMTRACK_MODE_ADD_TRANS,
	M_ANIMTRACK_MODE_ADD_ROTSANDTRANS,

};

#define M_ANIMTRACK_STICKY	0x00000001  // stick around even if we have been blended out
#define M_ANIMTRACK_BLEND_OUT	0x00000002  // once anim is done, blend to zero


typedef struct AnimTrack
{
	int  active;
	uns32 flags;

	char name[PM_NAME_SIZE];
	uns32 id;
	int  channel;

	uns32 blend_mode;
	float blend_val;
	float blend_target;
	float blend_speed;


	ObjKeyAnim okan;
} AnimTrack;


#define M_OTAN_FLAGS_SORT_TRACKS	0x00000001 // track needs sorting...
#define M_OTAN_FLAGS_USE_REAL_TIME	0x00000002 // Use raw time, instead of game time.
#define M_OTAN_DONTUPDATE	0x00000004


typedef struct 
{
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	uns32 flags;
	uns32 next_id;
	Res *skel_res;


	int num_tracks;
	AnimTrack *tracks;

} ObjTrackAnim;


/*

// ObjAniKeyAnim
// this object type encases a single key frame anim
// it can be used alone for splines and other cases
// where multiple key frame anims and/or blending between two (or more) key frame anims
// is not required/desired
// ObjAniTrack and ObjAniTracks bundle up multiple ObjAniKeyAnims
// for skeleton anim and other cases...
typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	TRANSFORM_COMMON_FIELDS // see above for this define

	char name[PM_NAME_SIZE];
	uns32 flags;
	uns32 joint; 
	float time;
	float speed;
	float frame;
	float loop_time;
	float loop_frame;
	Res *res;

	// current position
	float trans[3];
	float rot[3];
	float scale[3];
	
} ObjKeyAnim;

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	char name[PM_NAME_SIZE];
	uns32 flags;

	Res *res_grp; // will refer to the res group that contains the RESSUB_KANs for the running anim...

	// these fields are propogated from group to okans
	uns32 okan_flags;
	float time;
	float speed;
	float frame;
	float loop_time;
	float loop_frame;

	uns32 num_okans;
	ObjKeyAnim *okans; // key frame anims for each joint

} ObjKeyAnimGroup;


typedef struct KeyAnimTrack
{
	int   active;
	uns32 blend_mode;
	uns32 blend_state;
	float blend_val;
	float blend_speed;
	float blend_time;

	ObjKeyAnimGroup okag;
} KeyAnimTrack;


#define M_OBJ_ANI_TRACKS_MAX_TRACKS 8
typedef struct 
{
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	uns32 flags;


	// ordered list of anim tracks
	// ptrs are from the tracks list below...
	List active_tracks;

	// this is just storage...
	KeyAnimTrack *tracks[M_OBJ_ANI_TRACKS_MAX_TRACKS];

} ObjTrackAnim;
*/


/*
typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	Res *res_grp; // will refer to the res group that contains the RESSUB_KANs for this track...
	uns32 flags;
	uns32 num_kans;
	uns32 blend_mode;
	uns32 blend_state;
	float blend_val;
	float blend_speed;
	float blend_time;
	ObjKeyAnim *kans;
} ObjAniTrack;


typedef struct 
{
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	uns32 flags;


} ObjAniTracks;
*/

/*
#define OBJSEQRES_CB_NUM 8
#define OBJSEQRES_CB_NUM_P1 4
#define OBJSEQRES_CB_NUM_P2 4
typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	void *res;
	char name[PM_NAME_SIZE];

	uns32 flags; // use M_OBJANIRES_

	void *sigdata;
	uns32 time0; // start GameWaveTime
	uns32 hold; // hold till
	void* key0;
	void* signalAbort; // abort adr for signal
	void* cleanupSeq; // cleanup adr for the sequence
	void* everyLoop; // seq to call every loop

	memu cb[OBJSEQRES_CB_NUM]; // callback memory
} ObjSeqRes;
*/

#define M_OBJVEL_SCALE       0x00000002
#define M_OBJVEL_OFF         0x00000004
#define M_OBJVEL_WHOLEUNITSINVM 0x00000008  // pos is rounded down to whole units before being baked into vm, useful for maintaining whole pixel movements in 2d land
#define M_OBJVEL_JOINTATTACH 0x00000010
#define M_OBJVEL_GCOL        0x00000020
#define M_OBJVEL_NOPITCH     0x00000040
#define M_OBJVEL_IGNORE      0x00000080
#define M_OBJVEL_DONTUPDATE  0x00000100  // used for obj whose vel comes from physx (or somewheres else...)
#define M_OBJVEL_USE_PIVOT   0x00000200
#define M_OBJVEL_RESTORE_PIVOT   0x00000400 // restore the original pivot after applying ov's transform
#define M_OBJVEL_GNDCOL		 0x00000800 // restore the original pivot after applying ov's transform

// bounds for ovs
#define M_OBJVEL_USE_TRANS_MIN	0x00001000 // abide by min trans
#define M_OBJVEL_USE_TVEL_MIN	0x00002000 // abide by min tvel
#define M_OBJVEL_USE_ROT_MIN	0x00004000 // abide by min rot
#define M_OBJVEL_USE_RVEL_MIN	0x00008000 // abide by min rvel
#define M_OBJVEL_USE_SCALE_MIN	0x00010000 // abide by min scale
#define M_OBJVEL_USE_SVEL_MIN	0x00020000 // abide by min scale vel
#define M_OBJVEL_USE_TRANS_MAX	0x00040000 // abide by max trans
#define M_OBJVEL_USE_TVEL_MAX	0x00080000 // abide by max tvel
#define M_OBJVEL_USE_ROT_MAX	0x00100000 // abide by max rot
#define M_OBJVEL_USE_RVEL_MAX	0x00200000 // abide by max rvel
#define M_OBJVEL_USE_SCALE_MAX	0x00400000 // abide by max scale
#define M_OBJVEL_USE_SVEL_MAX	0x00800000 // abide by max scale vel
#define M_OBJVEL_USE_MAX			(M_OBJVEL_USE_TRANS_MAX|M_OBJVEL_USE_TVEL_MAX|M_OBJVEL_USE_ROT_MAX|M_OBJVEL_USE_RVEL_MAX|M_OBJVEL_USE_SCALE_MAX|M_OBJVEL_USE_SVEL_MAX)
#define M_OBJVEL_USE_MIN			(M_OBJVEL_USE_TRANS_MIN|M_OBJVEL_USE_TVEL_MIN|M_OBJVEL_USE_ROT_MIN|M_OBJVEL_USE_RVEL_MIN|M_OBJVEL_USE_SCALE_MIN|M_OBJVEL_USE_SVEL_MIN)
#define M_OBJVEL_USE_BOUNDS			(M_OBJVEL_USE_MAX|M_OBJVEL_USE_MIN)

#define M_OBJVEL_COLA_VEL		0x01000000 // this vel is being used to drive an obj cola...
#define M_OBJVEL_USE_TVEL_MAXSPEED	0x02000000 // abide by max tvel speed
#define M_OBJVEL_RAW_TIME			0x04000000 // use raw delta loop
#define M_OBJVEL_CAM_SPACE		0x08000000 // positions obj relative to main 3d gamera (i.e. (0,0,0) is right at camera and (0,0,-1) is right in front of it


typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	TRANSFORM_COMMON_FIELDS // see above for this define
	char name[PM_NAME_SIZE];
	// float  pause_time; // >0 means don't move for pause_time ms, <0 means pause indefinitely
	float pause_raw_time;
	int	   pause_frames; // >0 means don't move for pause_frames frames

	crd rot[3];
	crd trans[3];
	crd scale[3];

	uns32 pxgrp;	// collision group for physx (used for storage when the object isn't yet added to physx)

	uns32 flags;
	uns16 attachNum;

	float accTime; // accumulated time
	float maxTime;
	float t; // time used in calcs

	crd rvel[3];
	crd tvel[3];
	crd svel[3];

	crd racc[3];
	crd tacc[3];
	crd sacc[3];

	float pitch; // used when deer are off splines
	float rmax; // rot max per millisecond

	crd pivot[3];

	// max/min values (must | in flags for these to be used)
	crd rot_min[3];
	crd trans_min[3];
	crd scale_min[3];

	crd rot_max[3];
	crd trans_max[3];
	crd scale_max[3];


	crd rvel_min[3];
	crd tvel_min[3];
	crd svel_min[3];

	crd rvel_max[3];
	crd tvel_max[3];
	crd svel_max[3];
	float tvel_maxspeed;


	float gndcol_pivots[2]; // how far ahead/back to look when calc gndcol
	crd gnd_normal[3];

	struct ObjVelController* controller;

} ObjVel;
//typedef _ObjVel ObjVel

typedef int (VC_UpdateFunc)(ObjVel* objVel, float deltaTime);
typedef int (VC_FreeFunc)(struct ObjVelController* velCtrl);
typedef struct ObjVelController{
	OBJ_COMMON_FIELDS	// see above for this define
	char name[PM_NAME_SIZE];

	VC_UpdateFunc *UpdateFunc;
	VC_FreeFunc *FreeFunc;
	void* data;

} ObjVelController;


typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	
	int numParticles;		// Number of particles in the system
	int groupNumber;		// This particle system's group number
	void* res;				// The particle mesh
	uns8 flags;				// Billboarding flag; 1 - billboarded, 0 - not billboarded
	uns32 dList;			// Particle's display list
	char name[64];			// Particle system's name
	uns32 size;				// Particle size
} ObjPhysPart;

typedef struct {
	OBJ_COMMON_FIELDS

	uns32 OID1;
	uns32 OID2;
	uns8 type;
	crd localAnchor1[3];
	crd localAnchor2[3];
} ObjPhysJoint;

typedef struct {
	void *disp;
} ObjPartDisp;

#define M_OBJEMITEACHPARTICLE_ACTIVE 0x0001

typedef struct {
	uns8 flags;
	uns8 partnum;
	uns16 xx1;
	crd trans[3];
	crd tvel[3];
	crd tacc[3];
	crd scale; // 0 == no scale
	uns8 rgba[4];
	void *disp;
	float tStopTime; // max stop time of this particle
	float t; // last t of this particle
} ObjPartEachPart;


#define M_OBJSKELRES_DONTUPDATE 0x0001

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	Res *res; // points to resource

	uns16 flags;
	uns16 numJoints;
	float *jointRTSs;  // rotation, translation, scale for each joint (joint space)
	float *jointMats;
	float *bindMats;
	float *bindMatsM4;
	float *bindMatsM4Prev;

	int hasPrevBind;

	// uniform scale for collision object
	float colScale;

	// OID of a different ObjSkelRes to use instead of this one
	uns32 overrideOID;

} ObjSkelRes;


#define M_OBJCOLRES_OFF					0x00001 // don't collide this guy
#define M_OBJCOLRES_IGNORE_COLA_RADIUS  0x00002 // don't use radius on colas when colliding with this guy
#define M_OBJCOLRES_ENVCOL				0x00004 // don't do bullet collision, but show in debug render
#define M_OBJCOLRES_USE_ODR_SCALE		0x00008 // use the scale from parent odr

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	char group[PM_NAME_SIZE];

	uns16 flags;
	uns8 colFilterFlags; // see M_COLFILTER_
	uns16 xx1;
	Res *res; // ResDataCol*

	uns32 physxStatus;		// 0 - not in physx engine, 1 - static physx object, 2 - dynamic physx object
} ObjColRes;



typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	uns32 size;
	uns32 next;
} ObjBuf;

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];
	uns flags;
	int id;
} ObjMarker;



#define M_OBJCOLACT_HIT        0x0008 // this cola has hit something
#define M_OBJCOLACT_HITSKELCOL 0x0001 // hit skelcol
#define M_OBJCOLACT_HITPROPCOL 0x0002 // hit propcol
#define M_OBJCOLACT_HITGNDCOL  0x0003 // hit gndcol
#define M_OBJCOLACT_HIT2DCOL   0x0004 // hit 2d col
#define M_OBJCOLACT_HITENUM    0x0007 // first 7 bits are for what type of obj you hit 
#define M_OBJCOLACT_HITFLAGS (M_OBJCOLACT_HIT|M_OBJCOLACT_HITENUM)

#define M_OBJCOLACT_COLFUNC    0x0100 // has col function
#define M_OBJCOLACT_NO_VEL     0x0200 // dont build vm
#define M_OBJCOLACT_XYZR	   0x0400 // use xyzr
#define M_OBJCOLACT_USE_PLANE_COLPOS_ON_RADIUS_COL  0x0800 // if a radius col, take col pos as intersection w/ plane of face (dflt is to use midpoint of face)
#define M_OBJCOLACT_2D		   0x1000 // this is 2D collision actor
#define M_OBJCOLACT_KILLPRIME  0x2000 // find kill parent when cola is killed on maxTime
#define M_OBJCOLACT_CAM_SPACE  0x4000 // adds camera position to pos before calculating	

#define M_OBJCOLACTVS_GND		0x01 // versus
#define M_OBJCOLACTVS_ENEMIES	0x02 // versus
#define M_OBJCOLACTVS_PROPS		0x04 // versus
#define M_OBJCOLACTVS_FRIENDLY	0x08 // versus

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	char name[PM_NAME_SIZE];

	// some optional info used for tracking bullets on game-side
	uns32 bullet_id;

	// used to run the collision tests
	uns32 flags; // all fields are keyed off of these flags & invalid otherwise
	uns32 vsflags;
	uns16 gnum;
	uns16 pnum;
	crd pos[3];
	crd dir[3];
	crd len;
	crd xyzr[4]; // in world space

	uns32 type; // what type of col actor is this (game specific)

	uns32 dispres_oid; // if there is a disp res associated with this cola, put it's oid here...

	// successful best collision info
	crd cpos[3]; // collision point
	crd cnrm[3]; // collision normal
	crd clen; // length of collision
	uns16 cnum; // group number of collision
	void *cobj; // object of collision
	char *cdata; // polygon of collsion
	ftype4 *colfunc;
	ftype2 *updatefunc;
	ftype2 *deathfunc;
	int cjointnum;
	crd cjointoff[3]; // offset from joint in joint space
	crd cjointcnrm[3]; // collision normal in joint space
	crd cbindoff[3]; // offset in bind space
	crd cbindcnrm[3]; // collision normal in bind space

	uns32 maxTime; // kill prime after this time
	uns32 maxFrames;

	float prevPos[3];
	uns32 firstFrame;
	uns32 curFrameNum;

	float bestY;

} ObjColAct;

enum {
	OBJCOLFUNC_NONE,
	OBJCOLFUNC_MCI, // (-,ObjColAct*,MsgColInfo*,NUL)
	OBJCOLFUNC_COLA, // (-,ObjColAct*,NUL,NUL)

	// return codes
	OBJCOLFUNC_STOP=0x1000, // stop processing
};
#define M_OBJCOLFUNC 0xff

#define M_OBJNEW_DONTLINK   0x0000
#define M_OBJNEW_LINKACTIVE 0x0001 // enum
#define M_OBJNEW_LINKAFTER  0x0002 // enum
#define M_OBJNEW_LINKBEFORE 0x0003 // enum
#define M_OBJNEW_LINKLIST   0x000f // enum
#define M_OBJNEW_KIDS       0x0020
#define M_OBJNEW_KIDFIRST   0x0040
#define M_OBJNEW_KIDSFIRST  M_OBJNEW_KIDFIRST
#define M_OBJNEW_DONTCLEAR  0x0080

#define NUM_LOD 3	// number of LOD levels supported
#define LOD_DIST0 35.0
#define LOD_DIST1 60.0
#define LOD_DIST2 500000.0

//
// LOD
//	level of detail entry
//
typedef struct {
	float	dist;		// max distance from camera to show this LOD (if 0, LOD not active)
	void	*dispRes;	// display resource to use for this LOD
	void	*skelRes;	// skeleton information to use for this LOD (NULL = no skel)
} LOD;

//
// ObjLOD
//	LOD information node for object
//
typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define

	int cur;				// current LOD level
	void *curDispRes;		// current display resource ptr
	void *curSkelRes;		// current skeleton resource ptr
	uns	baseNumJoints;		// number of joints in the initial object (for sanity check)
	LOD lod[NUM_LOD];
} ObjLOD;


//
// ObjExeContext
//	StateMachine ExeContext
//

#define M_OBJEXEC_FLAGS_DONT_KILL_ON_PROG_COMPLETE     0x00000001  // don't kill objexe when program completes, default behavior is to die when prog completes
#define M_OBJEXEC_FLAGS_KILLPRIME_ON_PROG_COMPLETE     0x00000002  // kill parent object when program completes
#define M_OBJEXEC_FLAGS_IGNORE_TRIGGER_EVENTS		   0x00000004  // do not listen to trigger events...

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	uns32 flags;

	Res* templateRes; // this is a pointer to the res file that created the actor as a template. 

	PSM_ExeContext context;
	ftype2 *freeFunc;

} ObjExeContext;


#define M_OBJPARTEMITTER_RAW_TIME     0x00000001

typedef struct {
	OBJ_COMMON_FIELDS

	ParticleEmitter emitter;

	float sort_pri; // used to order rendering alpha blended objects
	uns32 flags;
} ObjPartEmitter;


#define M_OBJFXGROUP_LAUNCHME       0x00000001
#define M_OBJFXGROUP_LAUNCHED       0x00000002
#define M_OBJFXGROUP_DONTUPDATE		0x00000004 // don't update emitter (but still update existing particles if any)
#define M_OBJFXGROUP_SORT_IN_FRONT	0x00000008 // high sort priority so they draw in front of most geo tagged with low priorities
#define M_OBJFXGROUP_REPEATING		0x00000010 // Don't kill this when all his particles are dead because it will launch some more later
#define M_OBJFXGROUP_RAW_TIME		0x00000020


typedef struct {
	OBJ_COMMON_FIELDS

	uns32 flags;
	char name[PM_NAME_SIZE];
	char group[PM_NAME_SIZE];
	float scale;
	float scaleVar;
	int streakCam; // for 2d cam streak, see CalcStripBBoardVector

	float sort_pri; // used to order rendering alpha blended objects
} ObjFXGroup;

enum ObjAttachSkelModes{
	M_OBJATTACH_SKEL_NONE = 0,
	M_OBJATTACH_SKEL_JOINTMATS,  // the obj is attached using the joint mat (offsets in joint space)
	M_OBJATTACH_SKEL_BINDMATS, // the obj is attached using the bind mat (offsets in bind pose space) 
};

#define M_OBJATTACH_FLAGS_KILLSELF  0x00000001 // this will kill attachment if target oid is killed
#define M_OBJATTACH_FLAGS_KILLPRIME 0x00000002 // this will kill attachment's parent obj if target oid is killed
#define M_OBJATTACH_FLAGS_NOPOS     0x00000004 // don't copy target obj's pos
#define M_OBJATTACH_FLAGS_NOROT     0x00000008 // don't copy target obj's rotations
#define M_OBJATTACH_FLAGS_2DPROJ    0x00000010 // attach to target's projected 2D position instead of it's 3D world pos and rotation.

#define M_OBJATTACH_FLAGS_STOPPRIME		0x00000020 // this will stop the attachment's parent obj if target oid is killed
#define M_OBJATTACH_FLAGS_BAKETOVEL		0x00000040 // if target oid is killed, bake transform to an ov (looks for OV above attachment, if not found, creates one)
#define M_OBJATTACH_FLAGS_HIDEWITHPRIME 0x00000080 // Hide/dont render the attached object when the prime does

typedef struct {
	OBJ_COMMON_FIELDS
	TRANSFORM_COMMON_FIELDS // see above for this define

	uns32 flags;
	char name[PM_NAME_SIZE];
	uns32 target_oid; // oid of obj we are attaching to
	int skel_attach_mode; // attach to a bone on the obj's skel
	int target_joint; // if a skel attachment, what bone to attach to
	float proj_z; // x value to use when projecting pos back into proj/camera space when using M_OBJATTACH_FLAGS_2DPROJ mode
} ObjAttachment;


#define M_OBJANIACTION_FLAGS_INITD       0x00000001 // we've been setup...
#define M_OBJANIACTION_FLAGS_NOSOUND     0x00000002 // do not play any sounds
#define M_OBJANIACTION_FLAGS_NOFXGROUPS	 0x00000004 // do not create any particle fxgroups 

typedef struct {
	OBJ_COMMON_FIELDS

	uns32 flags;
	uns32 otan_oid; // track anim we are doing actions based on
	uns32 obj_oid; // the disp object that is performing the action
	void *kan_res; // ptr to the kan res for the oar's primary animation channel
	char  kan_name[PM_NAME_SIZE * 4]; // concatentation of kan_res's two parent groups (top parent is g3 res group, 2nd parent is kan group)
	float kan_frame; // what frame is the kan on 

} ObjAniAction;


// this obj type allows us to attach arbitray data to an obj
typedef struct {
	OBJ_COMMON_FIELDS

	char type[RTDT_MAX_FORM_NAME_LEN];  // this is a type in our gRTDTParser
	int count;
	char name[PM_NAME_SIZE];
	void *data;

} ObjRTDTData;


// this guy is for cycling frames on an UV anim...
typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	TRANSFORM_COMMON_FIELDS // see above for this define

	float speed;
	float time_acc;
	int num_frames;
	int cur_frame; // frame displayed as of last update
	int *frames; // this should be a list of frames to use, expected to be allocated as part of the obj's memory
	int rows;
	int cols;

	int loop_counter;
	int max_loops;

	

} ObjUVAnim;

#define M_OBJGNDCOL_FLAGS_OFF  0x00000001
#define M_COLFILTER_SNAP	0x01 // this col makes the deer snap to it
#define M_COLFILTER_BONEFX	0x02 // this makes it collide for footprints and particles

typedef struct {
	OBJ_COMMON_FIELDS	// see above for this define
	uns32 flags;
	uns8 colFilterFlags; // see M_COLFILTER_
	char name[PM_NAME_SIZE];
	char group[PM_NAME_SIZE];


	Res *res;
	PMOCL_BoundingBox_Tree tree;

} ObjGroundCol;

enum {
	OBJ_BONEACTION_TYPE_BINDSPACE,
	OBJ_BONEACTION_TYPE_JOINTSPACE,
	OBJ_BONEACTION_TYPE_NONE,
};

enum {
	OBJ_BONEACTION_COLLISION_GROUNDCOL,
	OBJ_BONEACTION_COLLISION_LOCALSPACE,
};

typedef struct {
	OBJ_COMMON_FIELDS		// see above for this define

	uns32	offsetType;		// see OBJ_BONEACTION_TYPE_
	uns32	collisionMode;	// see OBJ_BONEACTION_COLLISION_
	char	jointName[PM_NAME_SIZE]; // this is used upon loading the file until the animal is known
	uns32	emitLocation;	// see OBJ_BONEACTION_EMIT_
	uns32	jointIndex;
	float	offset[3];		// the position to transform and test
	float	threshold;
	char	type[PM_NAME_SIZE];

	// state-related variables
	int		wasCollidedLastFrame;
	int		lastCollidedObjOID;
	List	whileTouchingEmitters;
} ObjBoneAction;

enum {
	OBJ_CPMG_TYPE_ITEM,
	OBJ_CPMG_TYPE_LAYOUT,
};

enum {
	OBJ_CPMG_STATE_NORMAL,
	OBJ_CPMG_STATE_HOVER,
	OBJ_CPMG_STATE_PRESSED,
	OBJ_CPMG_STATE_DISABLED,
};

typedef struct ObjCPMGuiRes{
	OBJ_COMMON_FIELDS	// see above for this define
	int type;
	uns32 flags;
	char name[PM_NAME_SIZE];
	int state;
	uns32 updates;
	uns32 elaspsed_time;
	uns32 data_oid;
	int (*UpdateFunc)(struct ObjCPMGuiRes *ocpmg);
} ObjCPMGuiRes;



enum
{
	OBJCAM0,
	OBJCAM1,
	OBJCAM2,
	OBJCAM3,
	OBJCAM4,
	OBJCAM5,
	OBJCAM6,
	OBJCAM7,
	NUM_OBJCAMS,
	OBJCAM_3DMAIN = OBJCAM0,
	OBJCAM_2DMAIN = OBJCAM1,
	OBJCAM_CONSOLE = OBJCAM7,
	OBJCAM_TOOL = OBJCAM7,
};

enum EnvColType
{
	ENVCOL_JUMP = 0,
	ENVCOL_CLIMB,
	ENVCOL_KILL,
	NUM_ENVCOL_TYPES
};

#define MAX_ENVCOL_INSTANCES 256

#define ENVCOL_START_TRIGGERED				0x001
#define ENVCOL_SHOW_GROUP_ACTIVE	0x002

typedef struct EnvCol{
	uns32 oid;
	int start_trig_id;
	int end_trig_id;
	uns32 flags;
} EnvCol;

typedef struct {
	uns8 flags;
	uns8 dead;
	N2 activeList;
	N2 colList;
	N2 deadList;
	List killOIDs;
	int ObjNewCalls;
	void *link;
	N2 *lightgroupG;
	N2 *lightgroups[OBJ_MAX_LIGHT_GROUPS];
	uns32 shadlight_oid;
	N2 *dispatcherslink;
	N2 *colaG;
	HashTable  ByOID;

	uns32 cam_oids[NUM_OBJCAMS];
	uns32 cam_fmarker_oids[NUM_OBJCAMS];
	uns32 cam_lmarker_oids[NUM_OBJCAMS];

	List gndcol_oids; // list of oids for gnd col ocrs

	List active_envcols[NUM_ENVCOL_TYPES];
	EnvCol envcol_pool[MAX_ENVCOL_INSTANCES];

} objGlobals;

EXTERN_C objGlobals objG;



EXTERN_C void Obj_AddEnvCol(uns32 oid, uns32 type, char* startTrig, char* endTrig);
EXTERN_C ObjColRes* Obj_ScanEnvCol(uns32 type, crd* pos);
EXTERN_C void Obj_FreeAllEnvCols();






extern int ObjInit(void);
extern void ObjFinal(void);
extern int ObjReset(uns reset);
EXTERN_C int ObjNew(void *objp,uns sub,uns size,uns flags);
EXTERN_C Obj *ObjFindByOID( uns32 oid );
EXTERN_C Obj *ObjFindChildByOID( Obj *parent, uns32 oid );
EXTERN_C Obj* ObjFindByName(char *name, uns32 subtype, N2 *list );


#define M_OBJLOOP_DUMMY 0x0001

#define M_OBJ_STACK_KIDFIRST 0x0001

extern int ObjLoop(uns flags);
extern int ObjUpdate( Obj *o, Obj *oparent, int cam_num );


#define M_OBJ_LISTWALK_FLAGS_SKIP_DEAD 0x00000001

extern int ObjListWalk( int (*ObjFunc)(Obj *o, void *data), void *data, int max_depth, uns32 flags );


extern void* ObjActiveFirst(void);
extern int ObjActiveCount(void);

extern void* ObjColFirst(void);
extern int ObjColCount(void);

extern void ObjDeleteDList(ObjDispRes *odr);


EXTERN_C ObjCam* ObjCamGet(int cam_num);
EXTERN_C int ObjCamSet(int cam_num, ObjCam *cam, ObjMarker *first_mark, ObjMarker *last_mark);


EXTERN_C void *ObjFindSub(Obj *obj, uns subtype);
int ObjKillType(Obj *start, uns8 sub, int recurse );



extern void ObjSetHighLOD(ObjDispRes *odr);

#define ObjFindLOD(o) ObjFindSub((Obj*)o,OBJSUB_LOD)
#define ObjFindSkelRes(o) ObjFindSub((Obj*)o,OBJSUB_SKELRES)
extern int ObjAddLOD(ObjDispRes *pObj, void *vLodRes, int lodLevel); // i think this no longer exists...


extern int ObjScanGndCol(float *pos, float *cpos, float *cnrm, Obj** cobj, uns32 flags);
extern int ObjColScanGndCol(ObjColAct *cola, uns32 flags);
extern void DoGroundCol( float *pos, float *rot, float forw_offset, float back_offset, float *outHeight, float *outPitch );
extern void DoGroundColNormal( float *pos, float *rot, float forw_offset, float back_offset, float *outHeight, float *outPitch, crd *out_normal );
extern void ObjDoCol(ObjColRes *ocr,ResDataCol *col, ObjColAct *pcola);
void ObjDoSkelCol(ObjDispRes* odr, ObjSkelRes* osr, ObjColRes* ocr, ObjColAct* pcola);
extern void ObjDoGCol(ObjGroundCol *ogc, ObjColAct *pcola);
extern int ObjDispRes2dColTest(ObjDispRes *odr, float *vmat, float *pos );

extern void ObjColLoop(void);
extern int ObjIsSub(void *o,uns8 subtype);
EXTERN_C int ObjKill(void *o);
extern int ObjLevelKill(int level);
extern int ObjKillBetween(void *v0,void *v1);
extern int ObjTagKill(int tag);
extern int ObjDispResGroupNameKill(char *name);
extern int ObjLoopEnd(void);
EXTERN_C int ObjSetLinkName(char *name);
extern void* ObjFindLinkName(char *name);
extern int ObjSetLinkObj(void *o);
extern void* ObjGetLinkObj(void);
extern int ObjSetLinkCameraMark(int cam, int first);
extern void *ObjGetCameraMark(int cam, int first);





/*
extern int ObjGetRTVels(void *o,crd *rvel,crd *tvel);
extern int ObjGetRTS(void *o,crd *rot,crd *trans,crd *scale);

#define M_OBJSIGNAL           0x0fff
#define M_OBJSIGNAL_IMMEDIATE 0x1000
extern int ObjSeqSignal( ObjSeqRes *osr, int msig, void *a );
extern int ObjSignal(void *o,int sig,void *a);
extern int ObjSignalOrKill(void *vo,int msig,void *a);
extern int ObjTagSignal(int tag,int sig,void *a);
extern int ObjSeqGetVars(ObjSeqRes *osr,memu **varsp);
int ObjAniChange(ObjAniRes *oar, void *vkan, void *vkan_res, float wavetime);

*/

extern void ObjColaVel(void *cola);

extern int ObjResolveDisp(ObjDispRes *odr);
extern int ObjResolveTex(ObjDispRes *odr,uns flags);

/*
#define M_OBJJOINTMIRROR_ISLEFT 0x01
typedef struct {
	char *jname1;
	char *jname2;
	crd dir[3];
	uns flags;
} ObjJointMirrorRec;

extern int ObjJointMirror(void *odr,ObjJointMirrorRec *ojm,int showonleftside,
						  int *np,crd *off);
extern int ObjJointLR(void *odr,ObjJointMirrorRec *ojm,int n,crd *off);
*/

extern char *ObjColTypeStr(void *objv,char **s2p);
extern char *ObjNameStr(void *objv,char **s2p);


#if DEBUG
extern int DbgObjJointNames(void *odr);
#endif // DEBUG

#define M_OBJDISPRES_CREATE_FLAGS_ADDSKEL          0x00000001
#define M_OBJDISPRES_CREATE_FLAGS_ADDSKELANIM      0x00000002
#define M_OBJDISPRES_CREATE_FLAGS_ADDSPLINE        0x00000004
#define M_OBJDISPRES_CREATE_FLAGS_ADDVEL           0x00000008
#define M_OBJDISPRES_CREATE_FLAGS_ADDPARTS         0x00000010
#define M_OBJDISPRES_CREATE_FLAGS_BLEND            0x00000020
#define M_OBJDISPRES_CREATE_FLAGS_NOFOG            0x00000040
#define M_OBJDISPRES_CREATE_FLAGS_LINKAFTER        0x00000080
#define M_OBJDISPRES_CREATE_FLAGS_ADDCOL		   0x00000100
#define M_OBJDISPRES_CREATE_FLAGS_NO_LOD		   0x00000200



int ObjCreateDisplayRes( int type, char *name, char *res_name, char *group_name, uns32 odr_flags, uns32 create_flags,
						    N2 *link, char *shader, int lightgroup, 			  
							char *progname, char *colname, 
						    ObjDispRes **o );

int ObjCreateFontDisplayRes( char *name, char *identifier, uns32 odr_flags, uns32 create_flags,
						     N2 *link, char *shader, char *progname, ObjDispRes **o  );

int ObjCreateColAct( char* name, ObjColAct **cola, uns32 create_flags, uns32 vsflags, uns32 flags,
					uns32 maxTime, ftype4 *colFunc, uns32 gnum, uns32 pnum, uns32 type, uns32 bullet_id );

int ObjCreateGroup( char* name, N2* link, void** out_group, uns32 grp_flags, uns32 create_flags, char* progName);
int ObjCreateFXGroup( char* name, N2* link, void** ofxgrp, uns32 ofxg_flags, uns32 create_flags, char* progName, int launch);
int ObjFXGroupAttachToCola( ObjFXGroup* ofg, ObjColAct* col, crd* useRot );

enum ObjDispGrpPartsCmds
{
	OBJDISPGRP_PARTS_CMD_ADD,
	OBJDISPGRP_PARTS_CMD_DEL,
};

int ObjDispGrpParts( ObjDispRes *odg, enum ObjDispGrpPartsCmds, char *part, void *opts );

ObjDispRes *ObjDispGrpPartFindGroup( ObjDispRes *part ); // pass in a part and it will find the group node



// funcs for adding new objs of various types
// EXTERN_C int ObjAddExeContext( ObjExeContext **oec, Obj *parent, char *program_name, char *statemachine_name, void *statemachine_params );


EXTERN_C int AddPhysCollisionCallbackFunc(void* obj, void (*function)(void* obj1, void* obj2, void* collisionData));


void ObjDispResBuildRenderDataVM(ObjDispRes *odr);
void ObjDispResSetupRenderData(ObjDispRes *odr); // called once on initial obj setup, setups/malloc's render data ptrs
void ObjDispResPrepRenderData(ObjDispRes *odr); // called once per frame for disp2/disp3 objs, applies current ov, spline and/or skel rotation to verts, normals, etc...


void *ObjShaderFind( void *sr, void *list ); // find the first shader obj using this res

int ObjAddCorePipelineObjs(); // adds cameras, markers, frame start, etc.

void ObjLightResSetup( ObjLightRes *olr );
int ObjKillLights(char *group); // kills all lights in all lightgroups


void* ObjFindSubNth( N2 *list, uns sub, char *name, int n );

int ObjCreateAttachment( ObjAttachment **ppoat, char *name, 
						 uns32 target_oid, 
						 int skel_attach_mode, int joint_num, char *joint_name, 
						 int add_offset_ov, int add_offset_okan, 
						 N2 *link );

int CreateAniActionObj( ObjAniAction **ppoaa, ObjTrackAnim *target_otan, N2 *link );


int ObjDispResGroupNameKill(char *name);
int ObjDispResSetFlagsByGroup( char *group_name, uns32 flags );
int ObjDispResClearFlagsByGroup( char *group_name, uns32 flags );

int ObjColResGroupNameKill(char *name);
int ObjColResSetFlagsByGroup( char *group_name, uns32 flags );
int ObjColResClearFlagsByGroup( char *group_name, uns32 flags );

int ObjGroundColSetFlagsByGroup( char *group_name, uns32 flags );
int ObjGroundColClearFlagsByGroup( char *group_name, uns32 flags );

int ObjFXGroupClearFlagsByGroup( char *group_name, uns32 flags );
int ObjFXGroupSetFlagsByGroup( char *group_name, uns32 flags );

int ObjGroupClearFlagsByGroup( char *group_name, uns32 flags );
int ObjGroupSetFlagsByGroup( char *group_name, uns32 flags );

int ObjFXGroupGroupNameKill(char *name);


int ObjBuildTransformVM( int TransformType, Obj *o, float *vm, float *trans, float *rot, float *scale );
#define ObjBuildVM( o, vm, trans, rot, scale )   ObjBuildTransformVM( M_OBJTRANSFORM_TYPE_OBJECT, o, vm, trans, rot, scale )
int ObjVelBuildVM( ObjVel *ov );
//int ObjBuildVM( Obj *o, float *vm, float *trans, float *rot, float *scale );  // short hand for ObjBuildVM( M_OBJTRANSFORM_TYPE_OBJECT, ... )

int skelMats(ObjSkelRes *osr);
// int skelVerts(ResDataDisp3 *disp3,ObjSkelRes *osr, float *fpbuf,float *npbuf, ResDataDispExtendedData *dispext, float *vdpbuf);

int ObjSkelFindJointByName(ObjSkelRes* osr, char* name);

EXTERN_C int ObjCreateRTDTData( ObjRTDTData **portdtd, char *name, char *type, int count, N2 *link );

void SetupObjLightRes( ObjLightRes *olr );
void ObjLightResLink( ObjLightRes *olr ); // links olr in the right light group, right order


// [recursively] find and kills any and all objs of type sub
// if start==NULL, start=objG.activelist
int ObjKillSub(Obj *start, uns8 sub, int recurse ) ;




int ObjKeyAnimCreate( ObjKeyAnim **okan, uns32 type, ObjSkelRes *osr, N2 *link );
void ObjKeyAnimReset( ObjKeyAnim *okan );
int ObjKeyAnimSetRes( ObjKeyAnim *okan, Res *r );
int ObjKeyAnimUpdate( ObjKeyAnim *okan, float dt );
int ObjKeyAnimStepTime( ObjKeyAnim *okan, float time, int relative );

int ObjKeyAnimSetFlags( ObjKeyAnim *okan, uns32 flags );
int ObjKeyAnimClearFlags( ObjKeyAnim *okan, uns32 flags );
int ObjKeyAnimAddFlags( ObjKeyAnim *okan, uns32 flags );

int ObjKeyAnimSetResByName( ObjKeyAnim *okan, char *name, uns32 oid ); // oid is used for animlist lookup, if not given, okan oid is used...
int ObjKeyAnimSetSpeed( ObjKeyAnim *okan, float speed, float blend_time );
int ObjKeyAnimFindAnimKeys( ObjKeyAnim *okan, KeyAnimPos *kan, ResDataKanKey **pkey1, ResDataKanKey **pkey2  );

int ObjKeyAnimCopy( ObjKeyAnim *src, ObjKeyAnim *dst );


int ObjTrackAnimCreate( ObjTrackAnim **otan, ObjSkelRes *osr, int num_tracks, N2 *link  );
int ObjTrackAnimUpdate( ObjTrackAnim *otan, float dt );

int ObjTrackAnimActivateTrack( ObjTrackAnim *otan, char *name, uns32 *track_id );
int ObjTrackAnimSetTrackRes( ObjTrackAnim *otan, uns32 track_id, Res *r );
int ObjTrackAnimSetTrackResByName( ObjTrackAnim *otan, uns32 track_id, char *name );
int ObjTrackAnimSetTrackFlags( ObjTrackAnim *otan, uns32 track_id, uns32 flags );
int ObjTrackAnimAddTrackFlags( ObjTrackAnim *otan, uns32 track_id, uns32 flags );
int ObjTrackAnimClearTrackFlags( ObjTrackAnim *otan, uns32 track_id, uns32 flags );

int ObjTrackAnimSetTrackSpeed( ObjTrackAnim *otan, uns32 track_id, float speed, float time  );

int ObjTrackAnimSetTrackOkanFlags( ObjTrackAnim *otan, uns32 track_id, uns32 flags );
int ObjTrackAnimAddTrackOkanFlags( ObjTrackAnim *otan, uns32 track_id, uns32 flags );
int ObjTrackAnimClearTrackOkanFlags( ObjTrackAnim *otan, uns32 track_id, uns32 flags );

int ObjTrackAnimSetTrackBlendTarget( ObjTrackAnim *otan, uns32 track_id, float target, float time );
int ObjTrackAnimSetTrackBlendMode( ObjTrackAnim *otan, uns32 track_id, uns32 mode );
int ObjTrackAnimSetTrackChannel( ObjTrackAnim *otan, uns32 track_id, int channel );
int ObjTrackAnimTrackStepTime( ObjTrackAnim *otan, uns32 track_id, float time, int relative );
void ObjTrackAnimClearTrack( ObjTrackAnim *otan, uns32 track_id );
int ObjTrackAnimClearTracks( ObjTrackAnim *otan );

int ObjTrackAnimCopy( ObjTrackAnim *src, ObjTrackAnim *dst );

int ObjTrackAnimAddTrack( ObjTrackAnim *otan, char *name, uns32 *track_id, char *res_name, int channel, uns32 okan_flags, float speed, float blend_target, float blend_time, int blend_mode, uns32 track_flags );

AnimTrack *ObjTrackAnimGetTrack( ObjTrackAnim *otan, uns32 track_id );
ObjKeyAnim *ObjTrackAnimGetTrackOkan( ObjTrackAnim *otan, uns32 track_id );

// these return info for the top most track of any given channel 
uns32 ObjTrackAnimGetChannelTopTrackId( ObjTrackAnim *otan, int channel );
AnimTrack *ObjTrackAnimGetChannelTopTrack( ObjTrackAnim *otan, int channel );
ObjKeyAnim *ObjTrackAnimGetChannelTopOkan( ObjTrackAnim *otan, int channel );
int ObjTrackAnimSetChannelTopTrackSpeed( ObjTrackAnim *otan, int channel, float speed, float time );

uns32 ObjTrackAnimGetChannelBottomTrackId( ObjTrackAnim *otan, int channel );
AnimTrack *ObjTrackAnimGetChannelBottomTrack( ObjTrackAnim *otan, int channel );
ObjKeyAnim *ObjTrackAnimGetChannelBottomOkan( ObjTrackAnim *otan, int channel );
int ObjTrackAnimSetChannelBottomTrackSpeed( ObjTrackAnim *otan, int channel, float speed, float time );

int mGetOkanVels( void* vpOKAN, float *tvel, float *rvel );

int ObjGndColResAddToGndColList( ObjGroundCol *ogc );
int ObjGroundCol_BuildBBoxTree( ObjGroundCol *ogc );



void ObjBuildCullCenterFromBatchedMeshes(Res* res);


int ObjCPMGDisplayResChange( ObjDispRes *odr, int state, float width, float height, uns32 cpmg_flags );
int ObjCreateCPMGDisplayRes( char *name, char *imageset, int state, float width, float height, uns32 cpmg_flags,
							 uns32 odr_flags, uns32 create_flags,
						     N2 *link, char *shader, char *progname, ObjDispRes **o  );

int ObjCreateCPMGuiRes( char *name, int type, int (*UpdateFunc)(ObjCPMGuiRes *o), uns32 create_flags, N2 *link,  char *progname, ObjCPMGuiRes **o  );

int ObjCreateClipRegion( Obj* target_obj, float nw_x, float nw_y, float se_x, float se_y, uns8 is_target_parent );

int ObjGLCGZTex_Setup(Obj* prime, char* vidName, float speed, int loop, int primeKill);
int ObjGLCGZTex_Create( char *movie_name, N2 *link, ObjGLCGZTex **o );
int ObjGLCGZTex_Reset( ObjGLCGZTex *ogt );
int ObjGLCGZTex_Pause( ObjGLCGZTex *ogt );
int ObjGLCGZTex_Resume( ObjGLCGZTex *ogt );
int ObjGLCGZTex_GotoFrame( ObjGLCGZTex *ogt, int frame, uns8 isRelative);

int ObjBoneActionUpdate(ObjBoneAction* oba);

void ObjDispResSetClipRegion( ObjDispRes *odr, float *origin, float *size, float *border, int clip_world_space, int recurse );
int ObjCreateScrollingList( Obj* target_obj, float *origin, float *size, float *border, float *speed, N2 *link, ObjScrollingList **osl  );
int ObjScrollingListUpdate(ObjScrollingList *osl);
int ObjScrollingListReset(ObjScrollingList *osl);

int ObjMakeDestructable(ObjDispRes* odr, Res* instance_res, void** dparams_out);



// NOTE: these two count cameras from 1
void RemoveObjFromRenderlists( Obj *o, int cam_num );
void AddObjToRenderlists( Obj *o, int cam_num );

int ObjWillRender(Obj* o, Obj* oparent);

void ObjAddToTriggerGroup(Obj* obj, char* trigger);


#endif // ndef OBJ_H

