// pm.h
// Play Mechanix - Video Game System
// Copyright 1998-2010 Play Mechanix Inc. All Rights Reserved
#include <pmrt_utils/pmrt_utils.h>

#ifndef PM_H
#define PM_H






#ifdef SYS_BB
float min(float a, float b);
float max(float a, float b);
#endif


///////////////////////////////////////////////////////////////////////////////

#if !(SYS_JP5500|SYS_PC|SYS_BB)
#error -- need to define SYS_XX in preprocessor
#endif

#if !SYS_GL
#error -- define SYS_GL in preprocessor
#endif

///////////////////////////////////////////////////////////////////////////////
// helper macros & pragmas

#define M_STRUCT_ALIGN 3

#define FLOAT_NAN 0x7FBFFFFF
#define DOUBLE_NAN 0xFFFFFFFF7FF7FFFF
#define SKIRT_NOTHIT FLOAT_NAN

// game types for this system
typedef uns32 mtime; // millisecond timer
typedef float crd; // coordinates
#define CRD_IS_FLOAT 1

#define NUL ((void*)0)
#ifndef NULL
#define NULL ((void*)0)
#endif


#if SYS_PC
	#pragma warning(disable:4244) // conversion from 'double ' to 'float '
	#pragma warning(disable:4305) // truncation from 'const double ' to 'float '
					
	#define __STR2__(x) #x
	#define __STR1__(x) __STR2__(x)
	#define __LOC__  __FILE__ "("__STR1__(__LINE__)") "
#endif // SYS_PC

#if SYS_BB
	#define FALSE 0
	#define TRUE 1
#endif // SYS_BB

#ifndef MAX_PATH
	#define MAX_PATH 260
#endif

// SYS_BB


///////////////////////////////////////////////////////////////////////////////
// BASE TYPES

// these come from pmrt_utils now...

/*
//-----------------------------------------------------------------------------
// SYS_PC
#if SYS_PC

// Play Mechanix base types & system
typedef signed char int8;
typedef unsigned char uns8;
typedef signed short int16;
typedef unsigned short uns16;
typedef signed long int32;
typedef unsigned long uns32;
typedef __int64 int64;
typedef unsigned __int64 uns64;
typedef int8 int2;
typedef uns8* ptr;
typedef float f32;
typedef unsigned int uns;
typedef uns sto; // unsigned, big enough for void*,float,int
typedef uns memu;
typedef int memi;

#define NUL ((void*)0)
#ifndef NULL
#define NULL ((void*)0)
#endif

#define FLOAT_NAN 0x7FBFFFFF
#define DOUBLE_NAN 0xFFFFFFFF7FF7FFFF
#define SKIRT_NOTHIT FLOAT_NAN

#define M_STRUCT_ALIGN 3

#define SYSBREAK() __asm {int 3}

// game types for this system
typedef uns32 mtime; // millisecond timer
typedef float crd; // coordinates
#define CRD_IS_FLOAT 1

#pragma warning(disable:4244) // conversion from 'double ' to 'float '
#pragma warning(disable:4305) // truncation from 'const double ' to 'float '
				
#define __STR2__(x) #x
#define __STR1__(x) __STR2__(x)
#define __LOC__  __FILE__ "("__STR1__(__LINE__)") "

#endif // SYS_PC

// SYS_PC SYS_BB
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// SYS_BB
#if SYS_BB

// Play Mechanix base types & system
typedef signed char int8;
typedef unsigned char uns8;
typedef signed short int16;
typedef unsigned short uns16;
typedef signed long int32;
typedef unsigned long uns32;
typedef long long int64;
typedef unsigned long long uns64;
typedef int8 int2;
typedef uns8* ptr;
typedef float f32;
typedef unsigned int uns;
typedef uns sto; // unsigned, big enough for void*,float,int
typedef uns memu;
typedef int memi;


#define NUL ((void*)0)
#ifndef NULL
#define NULL ((void*)0)
#endif

#define FLOAT_NAN 0x7FBFFFFF
#define DOUBLE_NAN 0xFFFFFFFF7FF7FFFF
#define SKIRT_NOTHIT FLOAT_NAN

#define M_STRUCT_ALIGN 3

#define SYSBREAK() __asm__ ("int $3")

// game types for this system
typedef uns32 mtime; // millisecond timer
typedef float crd; // coordinates
#define CRD_IS_FLOAT 1

#define FALSE 0
#define TRUE 1
#endif // SYS_BB

// SYS_BB
//-----------------------------------------------------------------------------
*/


///////////////////////////////////////////////////////////////////////////////
// TRAP MACROS

extern void* SysAdr(void);
extern void SysLockup(int);

#define LOCKUP(x) SysLockup(x)
#define BP(x) SysLockup(x)

///////////////////////////////////////////////////////////////////////////////
// MACROS

#define DONE(x) BAIL:
#define DONEZ(x)
#define err(x) {ec=x;goto BAIL;}
#define berr(x) {BP(x);ec=x;goto BAIL;}

#define VFUNC(x) (void(*)(void))x	// void function
#define COUNT(array) (sizeof(array) / sizeof(array[0]))

#define PI 3.1415926535897932384626433832795
#define PM_MATH_CONST_E 2.7182818284590452353602874713526
#define DEG2RAD(d) (((d)*PI)/180.0)
#define RAD2DEG(r) (((r)*180.0)/PI)
#define RAD_90 1.57080
#define EPS (1e-7)

#define PM_NAME_SIZE 64 // across the system (i.e. in DC)
#define PM_FILEPATH_MAX 2048

#define PM_FRAME_TIME 16 // close to 60 frames per second

///////////////////////////////////////////////////////////////////////////////
// ERROR CODES

//
// ERRORS & EXTENDED RETURN CODES
//
// -1.....-8999 user
// -9000..-9099 system
// -9200..-9299 file
// -9300..-9399 memory
// -9400..-9499 network
// -9500..-9599 video
// -9600..-9699 process
//
//  9000..9099

enum // int16
{
	// system
	ERR_SYS_LAST=-9099,
	ERR_USERABORT,
	ERR_INTERNAL,
	ERR_INTERNAL_ALREADY,
	ERR_INTERNAL_NOTREADY,
	ERR_OPENGL_EXT_NOTFOUND,
	ERR_SYS_FIRST=-9000,

	// res
	ERR_RES_FIND=-9199,
	ERR_RES_LOAD,
	ERR_RES_FILESYNTAX,
	ERR_RES_CRC,

	// file
	ERR_FILE_LAST=-9299,
	ERR_FILE_EOF,
	ERR_FILE_SEEK,
	ERR_FILE_SIZE,
	ERR_FILE_READ,
	ERR_FILE_WRITE,
	ERR_FILE_OPEN,
	ERR_FILE_EXIST,
	ERR_FILE_DELETE,
	ERR_FILE_GENERAL,
	ERR_FILE_NOTFOUND,
	ERR_FILE_INVALID,
	ERR_FILE_DISKFULL,
	ERR_FILE_RENAME,
	ERR_FILE_NOTVALID,
	ERR_FILE_NOTWRITABLE,
	ERR_FILE_NOFAT,
	ERR_FILE_ALIGN,
	ERR_FILE_LOCK,
	ERR_FILE_SYNTAX,
	ERR_FILE_INTERNAL,
	ERR_FILE_FIRST=-9200,

	// memory
	ERR_MEM_LAST=-9399,
	ERR_MEM_ALLOC,
	ERR_MEM_CORRUPT,
	ERR_MEM_INTERNAL,
	ERR_MEM_CHECK,
	ERR_MEM_AVAIL,
	ERR_MEM_AVAIL_LOW,
	ERR_MEM_ALREADYFREE,
	ERR_MEM_SUBWAVE,
	ERR_MEM_NOTACTIVE,
	ERR_MEM_POOL_EMPTY,
	ERR_MEM_POOL_NOT_INITIALIZED,
	ERR_MEM_NOT_DYNAMIC,
	ERR_MEM_FIRST=-9300,

	// network
	ERR_NET_LAST=-9499,
	ERR_NET_EMPTY,
	ERR_NET_RANGE,
	ERR_NET_OVERFLOW,
	ERR_NET_CORRUPT,
	ERR_NET_TIMEOUT,
	ERR_NET_RETRYFAIL,
	ERR_NET_PKALLOC,
	ERR_NET_BDOVERFLOW,
	ERR_NET_INTERNAL,
	ERR_NET_PKSEND_ALLOC,
	ERR_NET_BUSYWAIT,
	ERR_NET_FIRST=-9400,

	// video
	ERR_VID_LAST=-9599,
	ERR_VID_PPC,
	ERR_VID_OVERFLOW,
	ERR_VID_SHADER_ERR,
	ERR_VID_FIRST=-9560,

	// frame buffer
	ERR_FB_LAST=-9559,
	ERR_FB_FIRST=-9530,

	// window
	ERR_WND_LAST=-9519,
	ERR_WND_SETUP,
	ERR_WND_FIRST=-9500,


	// process
	ERR_PROC_LAST=-9699,
	ERR_PROC_FIRST=-9600,

	// setup -- protection
	ERR_SETUP_LAST=-9799,
	ERR_SETUP_USR,
	ERR_SETUP_ENV,
	ERR_SETUP_MISMATCH,
	ERR_SETUP_FIRST=-9700,

	// disk raw
	ERR_DISKRAW_LAST=-9899,
	ERR_DISKRAW_READ,
	ERR_DISKRAW_WRITE,
	ERR_DISKRAW_COLDSTART,
	ERR_DISKRAW_FIRST=-9800,

	// host->target
	ERR_HOST_LAST=-9999,
	ERR_HOST_TIMEOUT,
	ERR_HOST_FIRST=-9900,

	ERR_PARSE_SYNTAX=-10999,
	ERR_PARSE_INCOMPLETE,
	ERR_PARSE_SETUP,
	ERR_PARSE_RANGE,
	ERR_PARSE_FAIL,
	ERR_PARSE_STOP,
	ERR_PARSE_EOF,

	ERR_PROC_STACKOVERFLOW=-11999,
	ERR_PROC_NOTHANDLED,
	ERR_PROC_SYNTAX,
	ERR_PROC_DUPE_FUNC,
	ERR_PROC_DUPE_NAME,
	ERR_PROC_DUPE_ID,
	ERR_PROC_NOT_IN_A_PROC,
	ERR_PROC_START_DEF_NOT_FOUND,
	ERR_PROC_PROC_LIST_FULL,
	ERR_PROC_COULDNT_ALLOC_PROC_PARAMS,
	ERR_PROC_COULDNT_ALLOC_STACK_SAVE,
	ERR_PROC_COULDNT_REALLOC_STACK_SAVE,
	ERR_PROC_ALREADY_RUNNING,
	ERR_PROC_ARGS_NOT_SUPPLIED,
	ERR_PROC_DREAMED_IMPOSSIBLE_DREAM,

}; // ERR_


enum {
	BP_UNASSIGNED=1000,
	BP_FTPBUFGET,
	BP_FTPLIST,
	BP_FTPRVCTL,
	BP_FTPRVCTL2,
	BP_FTPNETBUFFUNC1,
	BP_FTPNETBUFFUNC2,
	BP_FTPNETBUFFUNC3,
	BP_FTPNETBUFFUNC4,
	BP_FTPLOOP1,
	BP_FTPLOOP2,
	BP_FTPLOOP3,
	BP_MAIN,
	BP_MEMDSETUP,
	BP_MEMDPOOLFIRSTNODE,
	BP_MEMDPOOLFIRSTNODE2,
	BP_RESLINK,
	BP_FBSETMODE,
	BP_FSINIT1,
	BP_FSINIT2,
	BP_NETRESET1,
	BP_NETRESET2,
	BP_NETPORTINFO,
	BP_IMGLOAD1,
	BP_IMGLOAD2,
	BP_XRESLOADDISP,
	BP_XRESLOADIMG,
	BP_VIDINIT,
	BP_AUDGAMENUMPL,
	BP_GAMEGETPL,
	BP_REPORTCALCSCORE, //1030
	BP_CABINIT,
	BP_CABGETPL,
	BP_CABGETGUN,
	BP_CABOP,
	BP_CABLOOP,
	BP_FSOP,
	BP_FSOP2,
	BP_FTPRVCTL3,
	BP_GUNSETXY,
	BP_GUNOP1,
	BP_GUNOP2,
	BP_GUNOP3,
	BP_INPJAMMACALLBACK,
	BP_MEMSTACKRESTORE,
	BP_MEMSTACKSAVE,
	BP_OBJNEW,
	BP_OBJANIBODY,
	BP_OBJANIROOT,
	BP_OBJLOOPEND1,
	BP_OBJLOOPEND2,
	BP_OBJCOLTYPESTR,
	BP_ANICHANGE,
	BP_SEQCALLBACK,
	BP_ADDLIGHT,
	BP_ADDLIGHT2,
	BP_OBJLOOP,
	BP_OBJLOOP2,
	BP_PROCLOOP,
	BP_RESINIT1,
	BP_RESINIT2,
	BP_RESINIT3,
	BP_RESINIT4,
	BP_RESNEW,
	BP_RESREAD,
	BP_RESREAD2,
	BP_RESPTOK,
	BP_RESPARSEVARADR,
	BP_RESPDEER,
	BP_RESPBAN,
	BP_RESPBAN2,
	BP_RESPBAN3,
	BP_RESPRAN,
	BP_RESSETCAT,
	BP_RESSETCAT2,
	BP_RESSETCAT3,
	BP_RESSETCAT4,
	BP_RESSETCAT5,
	BP_RESPKILL,
	BP_RESPSEQ,
	BP_RESPSEQ2,
	BP_RESPBIN,
	BP_RESPCREATE,
	BP_RESPCREATE2,
	BP_RESPCREATE3,
	BP_RESPCREATE4,
	BP_RESPCREATE5,
	BP_RESPCREATE6,
	BP_RESPCREATE7,
	BP_RESPCREATE8,
	BP_RESPCREATE9,
	BP_RESPBR,
	BP_RESPBR2,
	BP_RESPFIND,
	BP_RESPFIND2,
	BP_RESPFIND3,
	BP_RESPFIND4,
	BP_RESPFIND5,
	BP_RESPFIND6,
	BP_RESPFIND7,
	BP_RESPFIND8,
	BP_RESPFIND9,
	BP_RESPFIND10,
	BP_RESPFIND11,
	BP_RESPFLAG,
	BP_RESPOBJ,
	BP_RESPOBJ2,
	BP_RESPOBJ3,
	BP_RESPOBJ4,
	BP_RESPOBJ5,
	BP_RESPOBJ6,
	BP_RESPOBJ7,
	BP_RESPOBJ8,
	BP_RESPOBJ9,
	BP_RESPOBJ10,
	BP_RESPMEM,
	BP_RESPLIGHT,
	BP_RESPLIGHT2,
	BP_RESPLIGHT3,
	BP_RESPARSEONE,
	BP_RESPARSEONE2,
	BP_SZTO,
	BP_N2LINKAFTER,
	BP_N2LINKBEFORE,
	BP_N2NAME,
	BP_N2SCAN,
	BP_INTDEVICEENABLE,
	BP_INTDEVICEENABLE2,
	BP_INTDEVICEENABLE3,
	BP_INTDETACH,
	BP_MAIN2,						 //1130
	BP_PROPCOL,
	BP_DRAWPOS,
	BP_DRAWPOS2,
	BP_DRAWPOS3,
	BP_DRAWPOS4,
	BP_DRAWPOS5,
	BP_DISPDRAW2,
	BP_COLDISP3,
	BP_SKELCOL,
	BP_DRAWSKELCOLDISPDBG,
	BP_DISPDRAW3,
	BP_DISPDRAW4SKELDBG,
	BP_DISPDRAW4SKELDBG2,
	BP_DISPDRAW4SKELDBG3,
	BP_DISPDRAW4SKELDBG4,
	BP_DISPDRAW4SKEL,
	BP_DISPDRAW4SKEL2,
	BP_DISPDRAW4SKEL3,
	BP_DISPDRAW4SKEL4,
	BP_DRAWINITSORTONCE,
	BP_DRAWACCOUNTDISP2SORTONCE,
	BP_DRAWALLOCATESORTONCE,
	BP_DRAWADDDISP2SORTONCE,
	BP_DRAWADDDISP2SORTONCE2,
	BP_DRAWADDDISP2SORTONCE3,
	BP_OBJANICHANGE,
	BP_OBJANIKAN,
	BP_SEQCALLBACK2,
	BP_SEQFWD,
	BP_ADDLIGHT3,
	BP_ADDLIGHT4,
	BP_ADDLIGHT5,
	BP_OBJLOOP3,
	BP_OBJLOOP4,
	BP_OBJLOOP5,
	BP_OBJLOOP6,
	BP_OBJLOOP7,
	BP_OBJLOOP8,
	BP_IMGLOAD3,
	BP_IMGLOAD4,
	BP_IMGLOAD5,
	BP_VIDSTART,
	BP_JPSAPPHOOKMSG,
	BP_ATSETUPDEMO,
	BP_ATSETUPDEMO2,
	BP_ATSETUPDEMO3,
	BP_ATSETUPDEMO4,
	BP_ATSETUPDEMO5,
	BP_AUDSETCREDITS,				  //1180
	BP_FCCREATEBOAR,
	BP_BOARKILLSECONDARIES,
	BP_FCINITCRIT,
	BP_FCCRITFALLSTART,
	BP_FCCRITDIEANIM,
	BP_DEERCOLSKEL,
	BP_DEERSETFLAG,
	BP_DEERCLRFLAG,
	BP_DEERFREEZESPLINE,
	BP_DEERFREEZESPLINE2,
	BP_DEERFREEZESPLINE3,
	BP_DEERUNFREEZESPLINE,
	BP_DEERUNFREEZESPLINE2,
	BP_DEERUNFREEZESPLINE3,
	BP_DEERGETRUNSTART,
	BP_DEERGETRUNSTART2,
	BP_DEERGETRUNSTART3,
	BP_DEERKILLSECONDARIES,
	BP_DEERFALLSTART,
	BP_FCINITDEER,
	BP_FCINITREPLAYDEER,
	BP_FCDEERDIEANIM,
	BP_FCDEERDIEANIM2,
	BP_FCDEERDIEANIM3,
	BP_FCCREATEDOVE,
	BP_FCCREATEDUCK,
	BP_FINDDEERPRIMEOBJ,
	BP_FINDDEERORCRITPRIMEOBJ,
	BP_EDITADDNAME,
	BP_EDITADDLIGHT,
	BP_EDITRESPARSEONE,
	BP_FCSETADV,
	BP_FCSETADV2,
	BP_FCSETADV3,
	BP_FCSETADV4,
	BP_FCSETADV5,
	BP_FCKILLSECONDARIES,
	BP_GAMEPROCCB,
	BP_FCCREATEGEAR,
	BP_MFREEZESPLINE,
	BP_MFREEZESPLINE2,
	BP_MFREEZESPLINE3,
	BP_MFREEZESPLINEOBJ,
	BP_MFREEZESPLINEOBJ2,
	BP_MFREEZESPLINEOBJ3,
	BP_PARTWEATHER,
	BP_PARTWEATHER2,
	BP_PARTWEATHER3,
	BP_FCCREATEPHEASANT,
	BP_PROJSHOTDATA,
	BP_PROJSHOTDATA2,
	BP_PROJSHOTDATA3,
	BP_FCCREATEQUAIL,
	BP_SHOTREPORTPROC,
	BP_SHOTREPORTPROC2,
	BP_SHOTREPORTPROC3,
	BP_SHOTREPORT2PPROC,
	BP_SHOTREPORT2PPROC2,
	BP_SHOTREPORT2PPROC3,
	BP_SELTIMERPROC,
	BP_FCCREATEMUG,
	BP_FCWINDCREATEBLADES,
	BP_FCWINDSETCUR,
	BP_AUDLOADALLDATA,
	BP_FCCREATEGOOSE,
	BP_FCCREATEBARREL,
};

enum { // int16
	ERC_ALREADY_DEAD=9000,
}; // ERC_


// seems like all these should be in vec.h

enum {
	MATORDER_XYZ,
	MATORDER_ZYX
};

#define M_MATORDER_ROTS 0x0f
#define M_MATORDER_SCALEFIRST 0x10
#define M_MATORDER_TRANSLATEFIRST 0x10

// VEC_
enum {
	VEC_X,
	VEC_Y,
	VEC_Z,
	VEC_SIZE,
};

enum {
	V2_X,
	V2_Y,
	V2_SIZE,
};


enum {
	V3_X,
	V3_Y,
	V3_Z,
	V3_SIZE,
};

enum {
	XYZ_X,
	XYZ_Y,
	XYZ_Z,
	XYZ_SIZE,
};

// PLANEQ_
enum {
	PLANEQ_NX,
	PLANEQ_NY,
	PLANEQ_NZ,
	PLANEQ_D,
	PLANEQ_SIZE,
};

// Q_
enum {
	Q_X,
	Q_Y,
	Q_Z,
	Q_W,
	Q_SIZE,
};

// M3_
enum {
	M3_11,
	M3_21,
	M3_31,
	M3_12,
	M3_22,
	M3_32,
	M3_13,
	M3_23,
	M3_33,
	M3_SIZE
};

// VM_
enum {
	VM_X,
	VM_Y,
	VM_Z,
	VM_11,
	VM_21,
	VM_31,
	VM_12,
	VM_22,
	VM_32,
	VM_13,
	VM_23,
	VM_33,
	VM_SIZE
};

// VM_
enum {
	VM_TX,
	VM_TY,
	VM_TZ,
	VM_RX,
	VM_RY,
	VM_RZ,
	VM_SX,
	VM_SY,
	VM_SZ,
};

// M4_
enum {
	M4_11,
	M4_21,
	M4_31,
	M4_41,
	M4_12,
	M4_22,
	M4_32,
	M4_42,
	M4_13,
	M4_23,
	M4_33,
	M4_43,
	M4_14, // x
	M4_24, // y
	M4_34, // z
	M4_44,
	M4_SIZE
};
#define M4_X M4_14
#define M4_Y M4_24
#define M4_Z M4_34

enum {
	M4OGL_11,
	M4OGL_21,
	M4OGL_31,
	M4OGL_41,
	M4OGL_12,
	M4OGL_22,
	M4OGL_32,
	M4OGL_42,
	M4OGL_13,
	M4OGL_23,
	M4OGL_33,
	M4OGL_43,
	M4OGL_14, // x
	M4OGL_24, // y
	M4OGL_34, // z
	M4OGL_44,
	M4OGL_SIZE
};

#define M4OGL_X 12
#define M4OGL_Y 13
#define M4OGL_Z 14

// notes on boxes
// choose your own box style, but follow conventions
// x,y,w,h
// l,t,r,b
// l=x,t=y,r=l+w-1,b=y+h-1

enum {
	BOX_LEFT,
	BOX_TOP,
	BOX_RIGHT,
	BOX_BOTTOM,
	BOX_SIZE
};

enum {
	BOX_L,
	BOX_T,
	BOX_R,
	BOX_B
};

enum {
	BOX_X,
	BOX_Y,
	BOX_W,
	BOX_H
};

enum {
	CLIP_LEFT,
	CLIP_TOP,
	CLIP_RIGHT,
	CLIP_BOTTOM,
	CLIP_NEAR, // assumed to be after LTRB & before far
	CLIP_FAR,
	CLIP_NUM
};

#define FRUSTVERT_NUM 8 // numv for near & far plane

#define BOX_WIDTH BOX_W
#define BOX_HEIGHT BOX_H

enum {
	BINBUF_SIZE,
	BINBUF_NEXT,
	BINBUF_FIRST,
};

#define BINBUFSET(f,i,v) {if((f)->binBuf&&((i+BINBUF_FIRST)<(int)(f)->binBuf[BINBUF_SIZE])) \
 (f)->binBuf[(i+BINBUF_FIRST)]=(memu)v;}

///////////////////////////////////////////////////////////////////////////////
// 

#define M_RESET_ALL         0x0000
#define M_RESET_HARDWARE    0x0001
#define M_RESET_MEMORY      0x0002
#define M_RESET_REBUILD     0x0004
#define M_RESET_LEVEL       0x0008

enum {
	LEVEL_PERM,
	LEVEL_WAVE,
	LEVEL_SUBW,
	LEVEL_GAME,
	LEVEL_NUM
};

///////////////////////////////////////////////////////////////////////////////
// 

// extern void errlog(char *msg);
#define errlog SysLog




//typedef ShaderUniformData_Values_Int
/*
typedef struct {
	char name[32];
	int type;
	int count;
	int *values;
} ShaderUniformData_int;

typedef struct {
	char name[32];
	int type;
	int count;
	int *values;
} ShaderUniformData_int;
*/

#endif // ndef PM_H
