// postfx.h
// Play Mechanix - Video Game System
// Copyright (c) 2009 Play Mechanix Inc. All Rights Reserved
//
#ifndef POSTFX_H
#define POSTFX_H

#include "g5.h"


#define M_POSTFX_DLFT_FOCUSMID   1.0f
#define M_POSTFX_DLFT_FOCUSMIDRANGE	  0.0f				// Region that completely clear
#define M_POSTFX_DLFT_FOCUSNEAR  1.0f
#define M_POSTFX_DLFT_FOCUSFAR   20000.0f

#define M_POSTFX_GLOW_SCALE 1.5f
#define M_POSTFX_GLOW_KSIZE 8.0f
#define M_POSTFX_GLOW_STD   2.0f
#define M_POSTFX_GLOW_ALPHA 1.0f
#define M_POSTFX_PARTICLE_GLOW_THRESHOLD 2.0f
#define M_POSTFX_GLOW_RAMP  0.75f

// update with raw delta time instead of regular delta (not effected by slow motion)
#define M_POSTFX_FLAG_RAW_TIME	0x00000001

typedef struct 
{
	char name[64]; 

	float Brightness;
	float Contrast;
	float rgb[3];

	float Saturation;

	float BloomThreshold;
	float BloomScale;
	float BloomKernelSize;
	float BloomSTD;
	float BloomAlpha;

	float NoiseScale;
	float NoiseHoldTime;

	float LevelsInBlack;
	float LevelsInWhite;
	float LevelsInGamma;

	float LevelsOutBlack;
	float LevelsOutWhite;


	float FocusNear;
	float FocusMid;
	float FocusMidRange;					// Ragion that completely clear
	float FocusFar;
	float FocusBlur;

	float FocusBlurTgt;
	float FocusBlurTgtInterp;

	float GlobalLightMult;

	// these here are not saved out at the moment...
	float GlowThresholdRamp;
	float OneMinusGlowThresholdRamp;
	float ParticleGlowThreshold;
	float GlowScale;
	float GlowKernelSize;
	float GlowSTD;
	float GlowAlpha;

	float PaintBlend;
	float PaintGridSize;
	float CanvasUVScale;
	float CanvasBase;
	float CanvasScale;

	int flags;

} PostFxSettings;

extern PostFxSettings gCurPostFxSettings;
extern float gPostFxAddSaturation;

int PostFxInit();
int PostFxReset();
int PostFxFinal();
int PostFxReload();
int PostFxLoop(float dt);

int PostFxSettingsInit(char *name, PostFxSettings *pfx);
int PostFxAdd(PostFxSettings *pfx);
int PostFxRemove(char *name);
int PostFxFind(char *name, PostFxSettings **pfx );

int PostFxSetTarget(char *name, float stepms);

int PostFxSetDoFBlurTarget(PostFxSettings *pfx, float tgt, float interp);


int PostFxLoad();
int PostFxSave(char *name);



#endif

