#ifndef RENDER_DEBUG_H
#define RENDER_DEBUG_H


// render_debug.h
// Copyright 2011 Play Mechanix Inc. All Rights Reserved

#include "g5.h"

// error logging

int LogGLErrors( char *descr );
// this bit of magic removes dbg err logs, expect when you explicitly turn them on
// this is useful for tracking down gl errors
// NOTE: VisStudio < VisStudio 2005 does not support variodic macros
#define VERBOSE_GL_LOG 0
#if !VERBOSE_GL_LOG
	// replace calls with null op, eating inputs along the way
	#define dbgLogGLErrors(...) 
#else 
	#define dbgLogGLErrors LogGLErrors
#endif 



// #define USE_GL_TRACK 1
#ifdef USE_GL_TRACK
void GLTrackInit();
void GLTrackAdd( char *type, uns32 gen, char *descr );
void GLTrackFree( char *type, uns32 gen );
void GLTrackLog( );
#else
#define GLTrackInit(...)
#define GLTrackAdd(...)
#define GLTrackFree(...)
#define GLTrackLog(...)
#endif


// perf counters
#ifdef DEBUG
void UpdateRenderDebugStats(ObjDispRes *o, Res *shader);
void DrawRenderDebugStats();
#else
#define UpdateRenderDebugStats(...)
#define DrawRenderDebugStats(...)
#endif

void ShaderCountsTelemetry();


// drawing debug stuff

#define MAX_DEBUG_TEXTS 200

void DoRenderDebugs(ObjDispRes*odr);
void DrawCol(ObjColRes *ocr, int init_matrix_stack);
void DrawODRCol(ObjDispRes *odr);
void DrawGroundCol( ObjGroundCol *ogc );
void DrawSkel(ObjDispRes *odr );
void DrawColas();
int ObjDrawLight( Obj *o, void *data );


// debug lines
extern int gNumDebugSpheres;
extern int gNumDebugTexts;
#define MAX_DEBUG_LINES 256
void AddDebugLine( crd* pt1, crd* pt2, crd* clr );
void AddDebugSphere( crd* pt, crd radius, crd* clr );
void AddDebugText(crd* worldPos, char* str, ...);
void RenderDebugLines();
void RenderDebugText();


#endif

