#ifndef RENDER_LAYERS_H
#define RENDER_LAYERS_H


// render_layers.h
// Copyright 1998-2011 Play Mechanix Inc. All Rights Reserved

#include "g5.h"



enum {
	RENDERLIST_MAIN,
	RENDERLIST_PARTICLES,
	RENDERLIST_PRELIGHT,
	RENDERLIST_FURBASE,

	// shad cast/recv lists for up to 3 shad map splits
	RENDERLIST_SHADCAST0,
	RENDERLIST_SHADCAST1,
	RENDERLIST_SHADCAST2,
	RENDERLIST_SHADRECV0,
	RENDERLIST_SHADRECV1,
	RENDERLIST_SHADRECV2,

	// reflection map
	RENDERLIST_REFLECTIONS,
	
	// moving objects to be motion blurred
	RENDERLIST_DYN_MOTION_BLUR,

	// objects to be rendered to the dynamic cubemap
	RENDERLIST_DYN_CUBE_MAP,

	/*
	// all the other cams get short shrift...
	RENDERLIST_EXTRACAM1,
	RENDERLIST_EXTRACAM2,
	RENDERLIST_EXTRACAM3,
	RENDERLIST_EXTRACAM4,
	RENDERLIST_EXTRACAM5,
	RENDERLIST_EXTRACAM6, 
	RENDERLIST_EXTRACAM7, // console/tool camera
	*/

	NUM_RENDERLAYER_RENDERLISTS,
} ;


#define RENDERLAYER_FLAGS_CULL                  0x00000001
#define RENDERLAYER_FLAGS_2DSORT                0x00000002
#define RENDERLAYER_FLAGS_DEBUGS                0x00000004
#define RENDERLAYER_FLAGS_SHADMAPS              0x00000008
#define RENDERLAYER_FLAGS_REFLECTIONS           0x00000010
#define RENDERLAYER_FLAGS_PRELIGHT              0x00000020
#define RENDERLAYER_FLAGS_FURBASE               0x00000040
#define RENDERLAYER_FLAGS_PARTICLES_SEPERATE    0x00000080

#define RENDERLAYER_FLAGS_CLEAR_CLR             0x00000100
#define RENDERLAYER_FLAGS_CLEAR_Z               0x00000200
#define RENDERLAYER_FLAGS_CLEAR_STENCIL         0x00000400

#define RENDERLAYER_FLAGS_FOG                   0x00000800
#define RENDERLAYER_FLAGS_ENVMAP				0x00001000

#define RENDERLAYER_FLAGS_POSTFX_BLOOM          0x00100000
#define RENDERLAYER_FLAGS_POSTFX_GLOW           0x00200000
#define RENDERLAYER_FLAGS_POSTFX_BLURD_IMG      0x00400000
#define RENDERLAYER_FLAGS_POSTFX_COLOR_CORRECT  0x00800000
#define RENDERLAYER_FLAGS_POSTFX_CANVAS_FX      0x01000000
#define RENDERLAYER_FLAGS_POSTFX_MOTION_BLUR    0x02000000
#define RENDERLAYER_FLAGS_POSTFX_LENS_FLARE     0x04000000
#define RENDERLAYER_FLAGS_DYN_CUBE_MAP          0x08000000

#define RENDERLAYER_FLAGS_POSTFX_IMG_PROCESSING  (RENDERLAYER_FLAGS_POSTFX_COLOR_CORRECT|RENDERLAYER_FLAGS_POSTFX_CANVAS_FX)
#define RENDERLAYER_FLAGS_POSTFX_ALL			(RENDERLAYER_FLAGS_POSTFX_BLOOM|RENDERLAYER_FLAGS_POSTFX_GLOW|RENDERLAYER_FLAGS_POSTFX_BLURD_IMG|RENDERLAYER_FLAGS_POSTFX_IMG_PROCESSING|RENDERLAYER_FLAGS_POSTFX_MOTION_BLUR/*|RENDERLAYER_FLAGS_POSTFX_LENS_FLARE*/)

#define RENDERLAYER_NUM_SCREEN_CAM	3



// here's how it works, we have one
// RenderLayer per camera
// each RenderLayer has many RenderLists
typedef struct RenderLayer
{
	ObjCam camera;		// Camera for the current frame	
	ObjCam camera_prev;	// Camera from the previous frame
	ObjCam temp_cam;	// Used to save current camera when manipulating view for multiple screens
	ObjCam camera_screen[RENDERLAYER_NUM_SCREEN_CAM];

	uns32 frames_since_cam_set;

	uns32 flags;
	int RenderTarget; // -1 if render to main screen

	// these two can be used to scale the entire rendering layer
	float xscale;
	float yscale;

	// next cube face to render.
	int dynCubeFace;

	RenderList RenderLists[NUM_RENDERLAYER_RENDERLISTS];

} RenderLayer;

enum {
	RENDERLAYER_3DMAIN   = 0,
	RENDERLAYER_EXTRACAM1,
	RENDERLAYER_EXTRACAM2,
	RENDERLAYER_EXTRACAM3,
	RENDERLAYER_EXTRACAM4,
	RENDERLAYER_EXTRACAM5,
	RENDERLAYER_EXTRACAM6, 
	RENDERLAYER_EXTRACAM7, // console/tool camera
	NUM_RENDERLAYERS,
};

extern RenderLayer gRenderLayers[NUM_RENDERLAYERS];


int RenderLayerInit( RenderLayer *layer, int render_target, uns32 flags );
int RenderLayerCopyCamera( RenderLayer *layer, ObjCam *camera );
int RenderLayerAddObj( RenderLayer *layer, Obj *o );
int RenderLayerRemoveObj( RenderLayer *layer, Obj*o );
int RenderLayerSetScreenCam( RenderLayer *layer, int screen );
int RenderLayerRestoreScreenCam( RenderLayer *layer );
int RenderLayerRender( RenderLayer *layer, int screen ); 
int RenderLayersInit();

int RenderLayerSetup( int layer, int render_target, float xscale, float yscale, uns32 flags );


#endif

