#ifndef RENDER_TARGET_H
#define RENDER_TARGET_H


// render_TARGET.h
// Copyright 1998-2011 Play Mechanix Inc. All Rights Reserved

#include "g5.h"


// you can query GL for what the supported max
// of your driver is, but 4 seems to be safe for now...
#define RENDERTARGET_MAX_COLOR_BUFFERS 4



typedef struct RenderTargetSetupData
{
	// setup data
	char name[32];
	int ColorBuffers;
	uns32 ColorBufferFormats;
	int hasDepth;
	int hasStencil;
	uns32 DepthBufferFormat;

	int width;
	int height;

	// default is to just render straight to textures, 
	// if this is set go to render buffers then textures (use for anti-aliasing
	uns8 UseRenderBuffers; 

	// data 
    uns16 active;           //if the texture is currently active
    uns16 valid;            //1 if creation succeeded, 0 otherwise
    uns32 format;           //0x00|Color|Depth|Stencil  = 0x00ffffff
    uns32 frameBuffer;
	uns32 renderBufferId[RENDERTARGET_MAX_COLOR_BUFFERS];
    uns32 colorTexId[RENDERTARGET_MAX_COLOR_BUFFERS];
    uns32 depthTexId;
    uns32 stencilTexId;
    uns32 colorTexFormats[RENDERTARGET_MAX_COLOR_BUFFERS];  //GL_RGB, GL_LUMINANCE.. so on    
	uns32 depthTexFormat;

} RenderTarget;




enum eRenderTargets
{
	RT_MAIN,
	RT_SHADMAP0,
	RT_SHADMAP1,
	RT_SHADMAP2,

	RT_REFLECTMAP,


	RT_BLUR_HIGH,
	RT_BLUR_LOW,

	RT_MOTION_BLUR,

	// for rendering to a cubemap each frame
	RT_DYN_CUBE_MAP,

	RT_OUTLINES,

	RT_PRELIGHT,

	RT_AUX1,	// these are for rendering cameras to texture
	RT_AUX2,
	RT_AUX3,
	RT_AUX4,

	NUM_RENDERTARGETS,
};


extern RenderTarget gRenderTargets[NUM_RENDERTARGETS]; // defined in render_target.c

int RenderTargetInitialize(RenderTarget *rt, uns16 width, uns16 height, char ColorBuffers, char hasDepth, char hasStencil, uns32 ColorBuffersFormat, uns32 DepthBufferFormat );
int RenderTargetSyncTexture(RenderTarget *rt);

#endif

