#ifndef SYS_TIME_H
#define SYS_TIME_H

// sys_time.h
// Copyright 1998-2011 Play Mechanix Inc. All Rights Reserved
//
// JAL 11-MAR-11
// Broke all date/time-related functions out into their own file and cleaned them up

#include <time.h>

typedef struct {
	int sec;	// 0..61(?)
	int min;	// 0..59
	int hour;	// 0..23
	int mday;	// 1..31
	int mon;	// 0..11
	int year;	// eg 2005
	int wday;	// 0..6 Sunday==0	
	int yday;	// since jan 1 0..365
	int isdst;	// is daylight savings time
} SysDate;

#define HOURSTOSECONDS(h) (h*60*60)
#define MINUTESTOSECONDS(h) (h*60)
#define SECONDSTOMICRO(h) (h*1000000)

#define DATEFMT_LITTLEENDIAN	"LittleEndian"	// LittleEndian = DD.MM.YYYY
#define DATEFMT_MIDDLEENDIAN	"MiddleEndian"	// MiddleEndian = MM.DD.YYYY
#define DATEFMT_BIGENDIAN		"BigEndian"		// BigEndian = YYYY.MM.DD

#define M_SYSTIMEDATE_GMT 0x0001

#define NUM_TZ_INDEXES 27
extern tzTab_T gTimeZoneStrTab[];

extern void SysCurrentLocalizedDateTimeString(char *output);
extern void SysLocalizedDateTimeString(time_t *timep, char *output, int outsize);
extern void SysLocalizedDateString(time_t *timep, char *output, int outsize);

extern void SysTimeToDateTime( time_t t, SysDate *dt, uns flags );
extern time_t SysDateTime(SysDate *dt,uns flags);
extern int SysSetDateTime(SysDate *dt);
extern time_t SysGetCMOSDateTime(SysDate *dt, uns flags);
extern int SysSetCMOSDateTime(SysDate *dt);
extern int SysDateTimeStr(SysDate *dt,char *buf,int bufsize,char *fmt);
extern int SysSecondsToDateTime(uns32 t, SysDate *dt, uns flags);
extern uns32 SysDateTimeInSeconds(void);
extern uns32 SysGMTDateTimeInSeconds(void);
extern void SysCopytmToSysDate(struct tm tms, SysDate *dt);
extern void SysCopySysDateTotm(SysDate *dt, struct tm *tmp);
extern uns32 SysDBTimeStrToSeconds(char *buf);
extern uns32 Sysmktime(SysDate *dt);
extern uns SysGetDaysPerMonth(uns month, uns year);

extern int SysSetTZ(int GMToff, int observeDST);

extern void SysSeconds64ToHMS(uns64 sec, uns32* hOut, uns32* mOut, uns32* sOut);
extern void SysSecondsToHMS_f(float sec, uns32* hOut, uns32* mOut, float* sOut);
extern void SysMSecsToHMS_f(uns32 msec, uns32* hOut, uns32* mOut, float* sOut);
extern void SysHMSToString_f(uns32 hours, uns32 minutes, float seconds, char* strDest);
extern void SysSecondsToHMSString_f(float sec, char* strDest);
extern void SysMSecsToHMSString_f(uns32 msec, char* strDest);

#endif //ndef SYS_TIME_H

// EOF

