#ifndef SYSTEM_INFO_H
#define SYSTEM_INFO_H

// system_info.h
// Copyright (c) 2011 Play Mechanix Inc. All Rights Reserved
//
// JAL 28-Feb-2011:  New - System Info is a central location to gather and house system information for display in diag, submitting to databases, etc.

// --------------------------------------------------------------------------
// Define all of the types of system information (these strings will be used as lookups in a hash table)
// These must also match the item names in diag.xml!!!
// Anything added here also needs a callback function for how to retrieve its data
// The pairings need to be added to SystemInfoInitTable in system_info.c
// --------------------------------------------------------------------------
#define SYSINFO_BIOS			"SysInfoBIOS"
#define SYSINFO_KERNEL			"SysInfoKernel"
#define SYSINFO_PROCESSOR		"SysInfoProcessor"
#define SYSINFO_MEMORY			"SysInfoMemory"

#define SYSINFO_ENGINE_VER		"SysInfoEngineVer"
#define SYSINFO_LIBRARIES_REV	"SysInfoLibrariesRev"
#define SYSINFO_G5_REV			"SysInfoG5Rev"
#define SYSINFO_SDL_VER			"SysInfoSDLVer"
#define SYSINFO_SNDLIB_VER		"SysInfoSndLibVer"
#define SYSINFO_IOLIB_VER		"SysInfoIOLibVer"
#define SYSINFO_IRTRACK_VER		"SysInfoIRTrackLibVer"
#define SYSINFO_JAMMA_VER		"SysInfoJAMMAVer"
#define SYSINFO_RIO_VER			"SysInfoRIOVer"
#define SYSINFO_UMC_VER			"SysInfoUMCVer"
#define SYSINFO_CARDLIB_VER		"SysInfoCardLibVer"

#define SYSINFO_GAME_FULLNAME	"SysInfoGameFullName"
#define SYSINFO_GAME_CODE		"SysInfoGameCode"
#define SYSINFO_GAME_VER		"SysInfoGameVer"
#define SYSINFO_GAME_UNITID		"SysInfoGameUnitID"

typedef struct _SystemInfoInitTable_s
{
	char *infoType;
	void(*init_func)(void);
} SystemInfoInitTable_t;

#define SYSTEM_INFO_DEFAULT_VALUE "Unavailable"

// --------------------------------------------------------------------------
// Public Function declarations
// --------------------------------------------------------------------------
int SystemInfoSetup(void);
int SystemInfoSetValue(char *infoType, char *value);
char *SystemInfoGetValue(char *infoType);
int SystemInfoItemIsActive(char *infoType);
int SystemInfoInit(void);
int SystemInfoFinal();

#endif // ndef SYSTEM_INFO_H

